{"version":"NotebookV1","origId":3273046978841218,"name":"BatchIngestion","language":"python","commands":[{"version":"CommandV1","origId":3273046978841219,"guid":"5c6673b8-ff83-4ca2-99d7-e601e300cc25","subtype":"command","commandType":"auto","position":1.0,"command":"%md\n### Notebook Info\nDescription | Comment\n-------------- | -----------\nName | BatchIngestion\nPurpose | This notebook BATCH ingests files from RAW to DELTA Zone, aka \"Base Tables\".\nAuthor |  Larry Edlin, ledlin@hitachisolutions.com<br>\nLast Updated| 1/20/2021|<br>\nLast Updated By | LE\n\n### Input Parameters\nParameter Name | Description\n-------------- | -----------\nExecute Query | Optional (default=TRUE).  Used for testing purposes.\nFileParams   | Path to config file containing tables to process.  Optionally, key-value pair JSON config of same.<br>\n\n### Output Parameters\nOutputs file with the following structure:\n\nColumn Name | Description\n-------------- | -----------\nbatdhId| Batch Id of Ingestion from ADF\nDeltaStartTimeUtc| Object Processing Start Time\nDeltaEndTimeUtc| Object Processing End Time\nDeltaSuccess| 0 (failure) or 1 (success)\nDeltaError| Error message \nDataChanged| 0 (no changes written) or 1 (data change occurred)\n\n###TO DO:\nNr|Description\n----|----\n1|When reading in file, do we want to default to file's schema if it differs from what was passed in or throw error?\n2|Do some basic sanity comparisons between incoming file and existing base table schema.\n3|If incoming file only has 1 column, assume it's probably corrupt (this has happened a couple times)\n4|Compare # columns in incoming file vs base table - fail if variance is less/greater some defined amount, as we don't want schema evolution occuring for a \"bad\" file.\n<br>\n<a href=\"#notebook/3607641936766908/command/3607641936771644\">Jump to Main Cell</a>","commandVersion":2,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"a43545fd-cc3b-4b9f-8800-75bb29326a8d"},{"version":"CommandV1","origId":3273046978841220,"guid":"aba03886-5e44-495e-b5f5-33204e6a11a8","subtype":"command","commandType":"auto","position":2.0,"command":"%md\n#### Housekeeping\nInstall Libraries, create Constants and Widgets (params), and set session-level settings","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"d733d263-d4a0-46fc-a93f-09e17f1c9c97"},{"version":"CommandV1","origId":3273046978841221,"guid":"bf70335e-3a6c-4d87-8aa2-253d21deeaa1","subtype":"command","commandType":"auto","position":3.0,"command":"from datetime import timedelta, datetime\nimport time\n\nfrom pyspark.sql import Row\nfrom pyspark.sql.types import *\nfrom pyspark.sql.functions import *\nfrom pyspark.sql.window import * #import Window\n\nimport pandas as pd\nimport os #for adding filename to dataframe\n#import re #regular expressions\nimport json #Optionally supports JSON-formatted parameter vs CSV file\nfrom delta.tables import *  #for python merge into delta\nimport multiprocessing as mp #for parallelizing file ingestion\nimport random\n\npd.set_option('display.max_colwidth', 60)\n\n# Necessary SESSION-level setting for automatic schema update\nspark.conf.set(\"spark.databricks.delta.schema.autoMerge.enabled\",\"true\")\n# Enable optimized writes\nspark.conf.set(\"spark.databricks.delta.optimizeWrite.enabled\",\"true\")\n# Repartition before write:  https://databricks.com/blog/2020/05/19/schema-evolution-in-merge-operations-and-operational-metrics-in-delta-lake.html\nspark.conf.set(\"spark.databricks.delta.merge.repartitionBeforeWrite.enabled\",\"true\")\n\n# Mount Point for DELTA tables\nDELTA_LOCATION = \"DELTA\"\n\nMETA_COLUMN_PREFIX = \"dl_\"\nSTATE_DB_NAME = 'state'\nSTATE_TBL_NAME = 'tablemetadata' # Case sensitive!!!\n\n# Get current UTC time to be used for entire batch of files to be ingested\ndtUTC = datetime.utcnow()\nBATCH_TIME = dtUTC.isoformat() # UTC timestamp for type 2 column\nBATCH_TIME_STR = dtUTC.strftime('%Y-%m-%dT%H:%M:%S.%fZ') # UTC timestamp for MERGE commands\n\n# Get a count of cores (per worker) to use as a limit to parallel execution threads\n# (Can also use a multiplier to scale up, if desired)\nCORES = mp.cpu_count() #* 2\n\n# Error logging\nCRITICAL = 'CRITICAL'\nERROR    = 'ERROR'\nWARNING  = 'WARNING'\nINFO     = 'INFO'\nDEBUG    = 'DEBUG'\n\n# Initialize list that will contain critical errors for output\nbatch_errors = []\n\n# Initialize a list to contain the currently processing tables\n# This will be used to ensure we aren't ever processing the same table \n# more than once at a time (in the event multiple files arrive for same table)\nprocess_queue = []\nbad_files = []\noutFile = \"\" #placeholder for output file name\n\n#dbutils.widgets.removeAll()\ndbutils.widgets.text(\"FileParams\",\"\")\ndbutils.widgets.dropdown(\"Execute Query\", \"TRUE\", [\"TRUE\", \"FALSE\"])\n\nfp = dbutils.widgets.get(\"FileParams\")\nEXECUTE_QUERY = eval(dbutils.widgets.get(\"Execute Query\").title())\n    \nprint('Execute query:\\t', EXECUTE_QUERY)\nprint('Parameter file:\\t', fp)","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b48e5579-b337-4e99-8321-87eac3b8f01f"},{"version":"CommandV1","origId":3273046978841222,"guid":"758075f9-fd54-4614-b131-3f9c4475c196","subtype":"command","commandType":"auto","position":4.0,"command":"%md\n### Load Shared Utilities","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"348cc433-4d48-4456-bfc1-b097f80de760"},{"version":"CommandV1","origId":3273046978841223,"guid":"7ebf240f-5055-4f6f-9921-07390cf60db1","subtype":"command","commandType":"auto","position":5.0,"command":"%run /DataEngineering/SharedUtils","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"43b32a3b-89b6-46c8-bc25-a2a98014cb98"},{"version":"CommandV1","origId":3273046978841224,"guid":"7cfe9958-f04b-40e3-8a30-1a250117da80","subtype":"command","commandType":"auto","position":6.0,"command":"%md\n### Define Ingestion Engine Class","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"f11cbf7a-135d-4a9e-93b2-c7961ba5de04"},{"version":"CommandV1","origId":3273046978841225,"guid":"3b318d41-a17c-4149-915b-b94be8fb7bfc","subtype":"command","commandType":"auto","position":7.0,"command":"class IngestionEngine:\n    \"\"\"\n    Class, Methods and Properties for Ingestion Engine.\n    Ingests a file landed in the RAW zone into base Delta tables,\n    with Type 2 change capture.\n    \"\"\"\n    \n    def __init__(self, batchId=None, dateFormat='yyyy-MM-dd HH:mm:ss.S', deltaCandidateKey=None,\n               deltaObjectName=None, deltaSchemaName=None, fileColDelimiter='\u0007', fileFormat='ORC', \n               fileHeader=True, filePath=None, fileQuoteCharacter='\u000E', fileSchema='', fileType='FULL',\n               preprocessingNotebook=None, deltaPartitionKey=None, rowNum=0, executeQuery=True):\n        self.batchId = batchId\n        self.dateFormat = dateFormat\n        self.deltaCandidateKey = deltaCandidateKey  \n        self.deltaObjectName = deltaObjectName.lower()\n        self.deltaSchemaName = deltaSchemaName.lower()\n        self.deltaPartitionKey = deltaPartitionKey\n        self.fileColDelimiter = fileColDelimiter\n        self.fileFormat = fileFormat.lower()\n        self.fileHeader = fileHeader\n        self.filePath = filePath\n        self.fileQuoteCharacter = fileQuoteCharacter\n        self.fileSchema = (fileSchema.lower() if fileSchema else '')\n        self.fileType = fileType.upper()\n        self.partitionKeyList = self.get_cleaned_partition_key_list()\n        self.preprocessingNotebook = preprocessingNotebook\n        self.schemaStruct = self.get_file_schema()\n        self.fullFilePath = self.get_full_file_path()\n        self.bad_records_path = f'/mnt/RAW/badRecords/{self.deltaSchemaName}.{self.deltaObjectName}'\n        self.candidateKeyList = self.get_cleaned_candidate_key_list()\n        self.persistedMountPath = f\"/mnt/{DELTA_LOCATION}/{deltaSchemaName}/{deltaObjectName}\"\n        self.errorLog = []\n        self.baseName = os.path.basename(self.filePath)\n        self.rowNum = rowNum\n        self.startTimeUtc = datetime.utcnow()\n        self.endTimeUtc = None\n        self.success = True\n        self.executeQuery = executeQuery\n        self.dataChanged = 0\n        self.tableExists = False\n\n    ###########################################################\n    # logger\n    ###########################################################\n    def logger(self, message, log_message=False, severity_level=INFO):\n        \"\"\"\n        A simple logger that adds timestamp and severity level to print message.\n        Optionally adds message to instance's \"log\".\n        \"\"\"\n        dtNow = datetime.utcnow()\n        \n        if len(message) > 0:\n            # Simple message to console\n            printMsg = f\"{dtNow:%Y-%m-%d %H:%M:%S}::{message}\"\n\n            # \"Thread-safe\" the print function, or some things print on the same line\n            print(f\"{printMsg}\\n\", end =\"\")\n\n        # Log entry for output\n        if log_message:\n            if severity_level== CRITICAL:\n                success = 0\n            else:\n                success = 1\n            # attemt to detect if data has changed\n            try:\n                self.check_data_changed(success)\n            except:\n                pass\n              \n            # Set end time\n            self.endTimeUtc = dtNow\n            # Build output message\n            logEntry = f\"{self.batchId}|{self.startTimeUtc:%Y-%m-%d %H:%M:%S}|{self.endTimeUtc:%Y-%m-%d %H:%M:%S}|{success}|{message}|{self.dataChanged}\"\n            self.errorLog.append(logEntry)\n\n    ###########################################################\n    # Check if Data changed\n    ###########################################################\n    def check_data_changed(self, success):\n        # Using DELTA table metadata, query most recent record to determine if there\n        # were any updates/inserts in the most recent action on the table (e.g. merge)\n        self.data_changed = 0\n        query = f\"\"\"\n        select\n              CASE WHEN operation = 'WRITE' THEN 1\n                 WHEN operation = 'UPDATE' and (operationMetrics.numTargetRowsUpdated + operationMetrics.numTargetRowsInserted) > 0 THEN 1\n                 WHEN operation like 'CREATE%' THEN 1\n                 WHEN (operationMetrics.numTargetRowsUpdated + operationMetrics.numTargetRowsInserted) > 0 THEN 1\n                 ELSE 0\n            END AS DataChanged\n        from (DESCRIBE HISTORY {self.deltaSchemaName}.{self.deltaObjectName} limit 10)\n        where datediff(current_date, cast(timestamp as DATE)) <= 1\n        and (operation in (\"UPDATE\", \"WRITE\", \"MERGE\")\n            OR operation like 'CREATE%')\n        LIMIT 1\n        \"\"\" \n\n        try:\n            df = spark.sql(query)\n            self.dataChanged = df.select(col(\"DataChanged\")).collect()[0][\"DataChanged\"]\n        except Exception as E:\n            errMsg = f\"WARNING! Failed to obtain table change history on {self.deltaSchemaName}.{self.deltaObjectName}.  {str(E)[:100].strip()}!\"\n            self.logger(errMsg)\n            pass\n\n        return self.data_changed\n      \n    ###########################################################\n    # get_cleaned_candidate_key_list\n    ###########################################################\n    def get_cleaned_candidate_key_list(self):\n        \"\"\"\n        Cleans column(s) in Candidate Key\n          - Removes non-alphanumeric characters (except underscore)\n          - Converts column(s) to lower case\n        \"\"\"\n        # If Candidate Key is passed in, clean it up the same way table column names are cleaned\n        candidate_key = [] #List of keys to be used later\n        if self.deltaCandidateKey:\n            newCandidateKey = \"\"\n            ck_list = self.deltaCandidateKey.split(\",\")\n            for k in (ck_list):\n                cname = strip_string(k, \"\", True)\n                candidate_key.append(cname)\n\n        return candidate_key\n\n    ###########################################################\n    # get_cleaned_partition_key_list\n    ###########################################################\n    def get_cleaned_partition_key_list(self):\n        \"\"\"\n        Cleans column(s) in partition Key\n          - Removes non-alphanumeric characters (except underscore)\n          - Converts column(s) to lower case\n        \"\"\"\n        # If Partition Key is passed in, clean it up the same way table column names are cleaned\n        partition_key = [] #List of keys to be used later\n        if self.deltaPartitionKey:\n            newPartitionKey = \"\"\n            pk_list = self.deltaPartitionKey.split(\",\")\n            for pk in (pk_list):\n                cname = strip_string(pk, \"\", True)\n                partition_key.append(cname)\n\n        return partition_key\n      \n    ###########################################################\n    # get_full_file_path\n    ###########################################################\n    def get_full_file_path(self):\n        \"\"\"\n        Appends mount root (mnt) to filepath, ensuring slashes\n        are properly formatted.\n        \"\"\"\n        #Add Databricks/dbfs root mount point to file name\n        args = [\"mnt\", self.filePath]\n        return slash_join(args, True)  # handles existing leading slash (or not)\n      \n    ###########################################################\n    # get_file_schema\n    ###########################################################\n    def get_file_schema(self):\n        \"\"\"\n        Creates schema StructType from schema parameter (string)\n        Ensures column names are cleaned and converted to lower case.\n        \"\"\"\n        if self.fileSchema:\n            # Create StructType of file schema for the read\n            schema_list = self.fileSchema.split(\";\")\n            columns_struct_fields = []\n            \n            # Build field list for StructType\n            for s in schema_list:\n                # Trim leading/trailing spaces\n                s = s.strip()\n                \n                # In case there are spaces in column names, but also a space\n                # between the column and datatype, let's fix that.\n                last_space = s.rfind(\" \")\n                parts = s.split(\" \")\n                if len(parts) > 2:\n                    s = s[:last_space] + \"|\" + s[last_space:]\n                else:\n                    s = s.replace(\" \", \"|\")\n\n                # Get and clean the column name\n                cname = strip_string(s.split(\"|\")[0], \"\", True)\n\n                # Get the Spark datatype\n                spark_dtype = sql_to_pyspark_datatype(s.split(\"|\")[1])\n\n                # Add column to list\n                columns_struct_fields.append(StructField(cname, eval(spark_dtype), True))\n\n            # Create the Struct from column list\n            return StructType(columns_struct_fields)\n\n    ###########################################################\n    # load_file\n    ###########################################################\n    def load_file(self):\n        \"\"\"\n        Loads file.\n         - Reads in file (Supports Parquet, Orc and Csv)\n         - Validates column count in file matches with count in schema parameter\n         - Applies schema parameter to dataframe (overrides loaded file schema)\n         - Removes duplicates.\n        \"\"\"\n        if self.fileFormat in ([\"orc\", \"parquet\"]):\n            options = {\"mode\": \"FAILFAST\"}\n        else: #Assuming CSV\n            options = ({\"mode\": \"FAILFAST\", \n                        \"inferSchema\": \"false\", \n                        \"header\": self.fileHeader, \n                        \"delimiter\": self.fileColDelimiter, \n                        \"quote\": self.fileQuoteCharacter, \n                        \"timestampFormat\": self.dateFormat,\n                        \"ignoreTrailingWhiteSpace\": \"true\",\n                        \"ignoreLeadingWhiteSpace\": \"true\"}\n                      )\n                      #.option(\"badRecordsPath\", self.bad_records_path)\n                      #.option(\"multiline\", \"true\") # Until Spark 3? supports the \"linesep\" option, this greatly slows down performance with large files!\n          \n        # Read the file into a dataframe\n        try:\n            #errMsg = f\"Reading file {self.fullFilePath} ...\"\n            #self.logger(errMsg)\n\n            if self.fileFormat == 'csv':\n                # get schema from input file\n                csv_schema = (spark.read\n                        .options(**options)\n                        .csv(self.fullFilePath).schema\n                   )\n                \n                # Read the file\n                self.df = (spark.read\n                            .format(self.fileFormat)\n                            #.schema(self.schemaStruct) # For CSV we must provide the schema since it's not being inferred\n                            .schema(csv_schema)\n                            .options(**options)\n                            .load(self.fullFilePath)\n                      )\n            else:\n                self.df = (spark.read\n                            .format(self.fileFormat)\n                            #.schema(self.schemaStruct)  This can return an empty dataframe if schema doesn't match 100%\n                            .options(**options)\n                            .load(self.fullFilePath)\n                      )\n                \n            self.logger(f\"File {self.filePath} record count: {self.df.count()}\") # Materialize the dataframe\n        except Exception as E:\n            errMsg = f\"ERROR! Failed reading file {self.filePath}  {str(E)[:100].strip()}!\"\n            self.logger(errMsg, True, CRITICAL)\n            return False\n        \n\n        # Assuming we have read in file:\n        # If schema was passed in, override the dataframe's default schema\n        if self.schemaStruct:\n            # Do a simple comparison of the number of columns, and raise error if they differ\n            #if len(self.schemaStruct) != len(self.df.schema):\n            if len(self.schemaStruct) != len(self.df.schema):\n                errMsg = f\"Mismatched number of columns between source file and schema passed in!\"\n                self.logger(errMsg, True, CRITICAL)\n                return False\n\n            # Override the dataframe's schema (columns have already been cleaned in get_file_schema())\n            i = 0\n            for c in self.df.dtypes:\n                cname_org = c[0]\n                cname_new = self.schemaStruct[i].name\n                ctype_org = c[1]\n                ctype_new = self.schemaStruct[i].dataType\n                \n                self.df = ( self.df\n                    .withColumnRenamed(cname_org, cname_new)\n                    .withColumn(cname_new, col(cname_new).cast(ctype_new))\n                     )\n                i += 1\n        # If no schema was passed in, we have to clean up the column names in dataframe\n        else:\n            new_column_name_list = list(map(lambda x: strip_string(x, \"\", True), self.df.columns))\n\n            # Assign cleaned column names back to existing dataframe\n            self.df = self.df.toDF(*new_column_name_list)\n\n        # Drop Duplicates\n        self.df = self.df.dropDuplicates()\n\n        try:\n            cnt = self.df.count() # Materialize the dataframe\n        except:\n            errMsg = f\"ERROR materializing dataframe from {self.filePath}!\"\n            self.logger(errMsg, True, CRITICAL)\n            return False\n        \n        return self.df\n        \n    ###########################################################\n    # add_metadata_columns\n    ###########################################################\n    def add_metadata_columns(self):\n        \"\"\"\n        Add metadata columns (dl_*) to dataframe\n        Cleans up column names, and applies cleaned columns to dataframe\n        Trims string types (some datasets are \"char\", with padded spaces in data)\n        Metadata columns:\n          - dl_rowhash:             SHA2/256 hash over all original file columns (not dl_*), used for MERGE when FULL load.\n          - dl_partitionkey:        First character of dl_rowhash - used for partitioning (16 potential characters)\n          - dl_iscurrent:           T/F flag for Type 2 history\n          - dl_recordstartdateutc:  Type 2 start date\n          - dl_recordenddateutc:    Type 2 end date\n          - dl_sourcefilename:      Source file\n        \"\"\"\n        try:\n            # FIRST: Add sha2 hash over all source columns for SCD Type 2\n            # https://stackoverflow.com/questions/52292180/pyspark-generate-row-hash-of-specific-columns-and-add-it-as-a-new-column\n            # Note:  concat_ws removes NULLs during the process, so it should work as expected\n            meta_col = META_COLUMN_PREFIX + \"rowhash\"\n            self.df = self.df.withColumn(meta_col, sha2(concat_ws(\"||\", *self.df.columns), 256))\n\n            # Generate a key for partitioning based on first 1 char of rowhash (should be max of 16)\n            # Note: We also partition by Is_Current\n            meta_col = META_COLUMN_PREFIX + \"partitionkey\"\n            self.df = self.df.withColumn(meta_col, (substring(\"dl_rowhash\",0,1)))\n\n            # Add an Active flag, setting it to True\n            # Also used in partitioning\n            meta_col = META_COLUMN_PREFIX + \"iscurrent\"\n            self.df = self.df.withColumn(meta_col, lit(True)) #Add column indicating the pDeltaObjectName #2.24m\n\n            # Add Record dates for SCD Type 2\n            meta_col = META_COLUMN_PREFIX + \"recordstartdateutc\"\n            self.df = self.df.withColumn(meta_col, to_timestamp(lit(BATCH_TIME)))#, \"%Y-%m-%dT%H:%M:%S.%fZ\")) #add timestamp when processed\n            meta_col = META_COLUMN_PREFIX + \"recordenddateutc\"\n            self.df = self.df.withColumn(meta_col, to_timestamp(lit(None), \"yyyy-MM-dd HH:mm:ss\")) #Add column indicating the type 2 end date\n\n            # Add the source file name - traceability\n            # Don: It might be better to have a delta table with all the file name and paths?  This is a lot of extra data.\n            meta_col = META_COLUMN_PREFIX + \"sourcefilename\"\n            self.df = self.df.withColumn(meta_col, lit(os.path.basename(self.filePath))) #Add column indicating the source file name\n            self.df.cache()\n            if self.df:\n                self.df.count() # Materialize the cache\n  \n            return True\n          \n        except Exception as E:\n            errMsg = f\"ERROR! Failed to add metadata columns! {str(E)[:100].strip()}\"\n            self.logger(errMsg, True, CRITICAL)\n            return False\n    \n    ###########################################################\n    # trim_string_columns\n    ###########################################################\n    def trim_string_columns(self):\n        \"\"\"\n        Trims string columns since some data sources send CHAR datatypes.\n        \"\"\"\n        # Make sure string column data is trimmed (some datasets use \"char\" datatypes)\n        for c, t in self.df.dtypes:\n            if t == \"string\":\n                self.df = self.df.withColumn(c, ltrim(rtrim(self.df[c])))\n        \n    ###########################################################\n    # store_delta_metadata\n    ###########################################################\n    def store_delta_metadata(self):\n        \"\"\"\n        Stores Delta metadata for current file/table:\n         - Delta Schema name\n         - Delta Table name\n         - Candidate Key\n         - Table Schema\n        \"\"\"\n        # Create database/schema as required\n        check_if_db_exists(STATE_DB_NAME, True, DELTA_LOCATION)\n\n        # Create table as required\n        check_if_table_exists(db_name=STATE_DB_NAME, table_name=STATE_TBL_NAME)\n        \n        try:\n            sch = self.df.schema\n            str_clean_cols = sch.simpleString().strip('struct').replace(':', ' ').replace(\"<\",\"\").replace(\">\",\"\").lower()\n        except:\n            str_clean_cols = ''\n\n        # Enable cross joins, however this query is only 1 record, so no worries.\n        spark.conf.set(\"spark.sql.crossJoin.enabled\", \"true\")\n\n        try:\n            # Create a dataframe from 4 variables to merge into metadata table\n            meta_df = spark.sql(f\"\"\"select '{self.deltaSchemaName}' as DeltaSchema\n                                         , '{self.deltaObjectName}' as DeltaTable\n                                         , '{\",\".join(self.candidateKeyList)}' as CandidateKey\n                                         , '{str_clean_cols}' as Schema\"\"\")\n\n            # Create a Delta object from the metadata table to merge into\n            metaTable = DeltaTable.forPath(spark, f\"/mnt/DELTA/{STATE_DB_NAME}/{STATE_TBL_NAME}\")\n\n            # MERGE dataframe into table\n            metaTable.alias(\"stats\").merge(\n                  source = meta_df.alias(\"updates\"), \n                  condition = f\"\"\"stats.DeltaSchema = lower(updates.DeltaSchema) \n                             AND stats.DeltaTable = lower(updates.DeltaTable)\n                             AND stats.DeltaSchema = lower('{self.deltaSchemaName}')  \n                             AND stats.DeltaTable = lower('{self.deltaObjectName}')\"\"\" #Last 2 conditions prevent merge concurrency issues\n              ).whenMatchedUpdate(\n                  condition = expr(\"stats.CandidateKey != updates.CandidateKey OR stats.Schema != updates.Schema\"),\n                  set = {\"CandidateKey\": \"updates.CandidateKey\" ,\n                         \"Schema\": \"updates.Schema\"}\n              ).whenNotMatchedInsertAll(\n              ).execute()\n        except Exception as E:\n            errMsg = f\"WARNING! Failed to sync Delta metadata table! {str(E)[:100].strip()}\"\n            self.logger(errMsg, False, WARNING)\n            \n    ###########################################################\n    # preprocess_file\n    ###########################################################\n    def preprocess_file(self):\n        \"\"\"\n        Executes a specific notebook to pre-process the file\n        into the desired format (e.g. pivots/unpivots)\n        \n        Returns global view representing preprocessed data\n        \"\"\"\n        timeout = 600\n        nb = self.preprocessingNotebook.strip()\n        #source_view = \"vw_\" + self.deltaObjectName + \"_\" + self.baseName\n        dtUTC = datetime.utcnow()\n        source_view = \"vw_\" + self.deltaObjectName + \"_\" + dtUTC.strftime('%f')\n        \n        self.logger(f\"Source global view name: {source_view}... notebook: {nb}\")\n       \n        self.df.createOrReplaceGlobalTempView(source_view)\n        \n        args = {\"SourceView\": source_view}\n        \n        if nb:\n            self.logger(f\"Running pre-processing notebook: {nb}...\")\n            \n            # Run the notebook, returning the global view name\n            try:\n                vw_name = run_notebook_with_retry(notebook=nb, params=args, timeout=timeout, retries=0, debug=False)\n                vw_name = \"global_temp.\" + vw_name\n                self.logger(f\"Preprocessed global view name: {vw_name}... \")\n                \n                # Replace the dataframe with the preprocessed data\n                self.df = spark.table(vw_name)\n                \n                # Convert column names to lower case so that's not an issue\n                new_column_name_list = list(map(lambda x: strip_string(x, \"\", True), self.df.columns))\n\n                # Assign cleaned column names back to existing dataframe\n                self.df = self.df.toDF(*new_column_name_list)\n                \n                # Update the schema to reflect new data\n                self.schemaStruct = self.df.schema\n                return True\n              \n            except Exception as E:\n                errMsg = f\"ERROR executing preprocessing notebook {nb}. {str(E)[:100].strip()}\\n{vw_name}\"\n                self.logger(errMsg, True, CRITICAL)\n                return False\n              \n            except:\n                errMsg = f\"ERROR executing preprocessing notebook {nb}.\\n{vw_name}\"\n                self.logger(errMsg, True, CRITICAL)\n                return False\n\n    ###########################################################\n    # create_base_table\n    ###########################################################\n    def create_base_table(self):\n        try:\n            if len(self.partitionKeyList) == 0:\n                partition_by = [\"dl_partitionkey\",\"dl_iscurrent\"]\n            else:\n                partition_by = self.partitionKeyList\n            \n            self.df.write \\\n                .format(\"delta\") \\\n                .partitionBy(partition_by) \\\n                .saveAsTable(f\"{self.deltaSchemaName}.{self.deltaObjectName}\")\n#               .partitionBy(\"dl_partitionkey\", \"dl_iscurrent\") \\\n\n            return True\n          \n        except Exception as E:\n            print(f\"***** ERROR creating base table: {str(E)[:100].strip()}\")\n            pass\n            return False\n            \n    ###########################################################\n    # merge_incremental_file\n    ###########################################################\n    def merge_incremental_file(self):\n        if self.candidateKeyList:\n          \n            # Refresh Delta baseTable object\n            baseTable = DeltaTable.forPath(spark, self.persistedMountPath)\n            basetable_df = baseTable.toDF()\n    \n            try:\n                # This creates dynamic columns consisting of the candidate keys for merging\n                merge_key = \"updates.\" + \", updates.\".join(self.candidateKeyList)\n                updates_merge_key = \"concat_ws('|', \" + merge_key + \")\" \n                merge_key = \"basetable.\" + \", basetable.\".join(self.candidateKeyList)\n                target_merge_key = \"concat_ws('|', \" + merge_key + \")\"\n        \n                # Create the list of columns to join on based on the candidate keys\n                join_condition = [col(\"updates.\" + f) == col(\"basetable.\" + s) for (f, s) in zip(self.candidateKeyList, self.candidateKeyList)]\n                # Add dl_rowhash and dl_iscurrent to the JOIN conditions\n                join_condition.append(col(\"basetable.dl_iscurrent\") == \"True\")\n                join_condition.append(col(\"updates.dl_rowhash\") != col(\"basetable.dl_rowhash\"))\n        \n                # Create a dataframe of new rows to be added\n                newInserts = self.df.alias(\"updates\").join(\n                      basetable_df.alias(\"basetable\"),\n                        join_condition\n                        , \"inner\")\n\n                # Stage the update by unioning two sets of rows\n                # 1. Rows that will be inserted in the `whenNotMatched` clause\n                # 2. Rows that will either UPDATE (\"deactivate\") the current records or INSERT new records \n                stagedUpdates = (newInserts\n                                 .selectExpr(\"NULL as dl_mergeKey\", \"updates.*\")  # Rows for 1\n                                 .union(self.df.alias(\"updates\").selectExpr(f\"{updates_merge_key} as dl_mergeKey\", \"*\"))) # Rows for 2\n\n                # MERGE dataframe into table, using candidate key(s)\n                baseTable.alias(\"basetable\").merge(\n                      source = stagedUpdates.alias(\"staged_updates\"), \n                      condition = expr(f\"staged_updates.dl_mergeKey = {target_merge_key} AND basetable.dl_iscurrent = true\")\n                  ).whenMatchedUpdate(\n                      condition = expr(\"basetable.dl_rowhash <> staged_updates.dl_rowhash\"), \n                      set = {\"basetable.dl_iscurrent\": \"false\" ,\n                             \"basetable.dl_recordenddateutc\": f\"'{BATCH_TIME_STR}'\"}\n                  ).whenNotMatchedInsertAll(\n                  ).execute()\n                \n                basetable_df.unpersist()\n                return True\n            except:\n                basetable_df.unpersist()\n                return False\n            \n        # Incremental, but missing candidate keys\n        else:\n            basetable_df.unpersist()\n            errMsg = f\"ERROR: Missing candidate keys for incremental file {self.baseName} for base table: {self.deltaSchemaName}.{self.deltaObjectName}\"\n            self.logger(errMsg, True, CRITICAL)\n            return False\n \n    ###########################################################\n    # merge_full_file\n    ###########################################################\n    def merge_full_file(self):\n        \n        # Refresh Delta baseTable object\n        baseTable = DeltaTable.forPath(spark, self.persistedMountPath)\n        basetable_df = baseTable.toDF()\n\n        try:\n            ###\n            # 1. Update/expire current rows in base table.  These are\n            #    rows where there is no match on row hash in incoming dataset.\n            ###\n            updateDF = (basetable_df.alias(\"basetable\")\n                    .join(self.df.select(\"dl_rowhash\",\"dl_partitionkey\").alias(\"updates\"),\n                            (col(\"basetable.dl_iscurrent\") == \"True\") \n                          & (col(\"updates.dl_partitionkey\") == col(\"basetable.dl_partitionkey\"))\n                          & (col(\"updates.dl_rowhash\") == col(\"basetable.dl_rowhash\"))\n                          , \"left_outer\")\n                    .filter(\"updates.dl_rowhash IS NULL AND basetable.dl_iscurrent == 'True'\"))\n\n            # Use Merge to update base table, based on row hash\n            baseTable.alias(\"basetable\").merge(\n                  source = updateDF.select(\"basetable.*\").alias(\"expired\"), \n                  condition = expr(\"\"\"expired.dl_partitionkey = basetable.dl_partitionkey \n                                       AND basetable.dl_iscurrent = true \n                                       AND expired.dl_iscurrent = true \n                                       AND expired.dl_rowhash = basetable.dl_rowhash\"\"\")\n              ).whenMatchedUpdate(\n                  set = {\"basetable.dl_iscurrent\": \"false\" ,\n                         \"basetable.dl_recordenddateutc\": f\"'{BATCH_TIME_STR}'\"}\n              ).execute()\n            \n            ###\n            # 2. Insert new records from updates dataset where there \n            #    is no match on the row hash (and is current)\n            ###\n            \n            # Create a list of columns to use in MERGE statement\n            # that has all columns from the base table and incoming dataset\n            diffcols = list(set(self.df.columns)-set(basetable_df.columns)) # Diff incoming dataset with basetable\n            added_cols = basetable_df.columns + diffcols # Add any new columns from dataset to basetable columns\n            diffcols = list(set(basetable_df.columns)-set(self.df.columns)) # Diff basetable with incoming dataset\n            all_columns = [\"updates.\" + col for col in added_cols if not col in diffcols ] # all columns, minus any dropped columns\n\n            # Refresh dataframe (due to previous merge)\n            basetable_df = baseTable.toDF()\n            \n            # Create a dataframe\n            insertDF = (self.df.alias(\"updates\")\n                        .join(basetable_df.alias(\"basetable\"),\n                            (col(\"basetable.dl_iscurrent\") == \"True\") #Must have this for outer join to succeed\n                           &  (col(\"updates.dl_partitionkey\") == col(\"basetable.dl_partitionkey\"))\n                           &  (col(\"updates.dl_rowhash\") == col(\"basetable.dl_rowhash\")) #Note, there can be existing records with this hash\n                           , \"left_outer\")\n                        .filter(\"basetable.dl_rowhash IS NULL\"))\n            \n            # MERGE dataframe into table, using row hash for key\n            baseTable.alias(\"target\").merge(\n                  #source = sdf.select(all_columns).alias(\"updates\"), \n                  source = insertDF.select(all_columns).alias(\"updates\"), \n                  condition = expr(f\"\"\"target.dl_partitionkey == updates.dl_partitionkey \n                                       AND updates.dl_rowhash == target.dl_rowhash \n                                       AND target.dl_iscurrent = true\"\"\") #must have all 3 conditions\n              ).whenNotMatchedInsertAll(\n              ).execute()\n           \n            self.df.unpersist()\n            return True\n          \n        except Exception as E:\n            print(f\"***** ERROR merging file: {str(E)[:100].strip()}\")\n            self.df.unpersist()\n            pass\n            return False\n","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"efe8d2b1-92ea-4766-9646-8ac081128e27"},{"version":"CommandV1","origId":3273046978841226,"guid":"8688a4aa-2b6c-44cf-962f-606acb40c14c","subtype":"command","commandType":"auto","position":8.0,"command":"%md\n### Notebook Functions (not Class methods)","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"5d9b6592-eea0-431d-bf96-0824c1562e48"},{"version":"CommandV1","origId":3273046978841227,"guid":"58ee8492-5306-4f5d-820c-03a51e7d749b","subtype":"command","commandType":"auto","position":9.0,"command":"###########################################################\n# load_batch_config\n# Parse the FileParams parameter to determine if it is JSON \n# or a reference to a CSV file, then load into dictionary\n###########################################################\ndef load_batch_config (config):\n    sdf = None\n  \n    # Try to parse FileParams to see if it's JSON, if not, assume it's a file path\n    if config:\n        try:\n            file_params = json.loads(config)\n        except:\n            file_params = config\n            pass\n    else:\n        #raise ValueError(\"The fileParams parameter must be provided!\") \n        print(\"The fileParams parameter must be provided!\")\n        return None\n      \n    if file_params:\n        # First let's see if this parses as JSON\n        try:\n            # Use Pandas for easy conversion of JSON to dataframe\n            pdf = pd.DataFrame(file_params)\n            #pdf[\"rownum\"] = pdf.reset_index().index\n            # Then convert to Spark Dataframe\n            sdf = spark.createDataFrame(pdf)\n\n        #Not JSON, so assuming a file reference\n        except:\n            #Add Databricks/dbfs root mount point to file name\n            args = [\"mnt\", file_params]\n            theFile = slash_join(args, True)  # handles existing leading slash (or not)\n            params_file = theFile\n\n            if not file_exists(params_file):\n                return None\n\n            print(f\"{datetime.now():%Y-%m-%d %H:%M:%S}::Loading BATCH configuration file from {file_params}...\")\n\n            # Read the file\n            if params_file.endswith(\".csv\"):\n                sdf = (spark \n                            .read \n                            .format('csv') \n                            .option(\"header\", \"true\") \n                            .option(\"inferSchema\", \"true\") \n                            .load(params_file)\n                          )\n\n            if sdf.count() == 0:\n                return None\n\n        sdf = sdf.withColumn(\"rownum\", row_number().over(Window.orderBy(monotonically_increasing_id())))\n\n        # Convert column names to lower case so that's not an issue\n        sdf = sdf.select([col(x).alias(x.lower()) for x in sdf.columns])\n\n        # Add placeholder for pre-processing notebook\n        if \"preprocessingnotebook\" not in sdf.columns:\n            sdf = sdf.withColumn(\"preprocessingnotebook\", lit(None).cast(StringType()))\n\n        # Add placeholder for partition keys\n        if \"deltapartitionkey\" not in sdf.columns:\n            sdf = sdf.withColumn(\"deltapartitionkey\", lit(None).cast(StringType()))\n\n        # Create a dictionary object from dataframe to iterate though\n        params_dict = dataframeToDict(sdf.sort(\"rownum\"))\n\n    else:\n        params_dict = None\n\n    return params_dict\n\n###########################################################\n# process_queue_manager\n###########################################################\ndef process_queue_manager(action, item):\n    l_action = action.lower()\n  \n    if l_action in ['add','append']:\n        if item not in process_queue:\n            # Add table to process queue to avoid concurrency\n            process_queue.append(item)\n\n    elif l_action in ['delete','remove','drop']:\n        # Remove table from processing queue\n        if item in process_queue:\n            process_queue.remove(item)\n    else:\n        raise ValueError(f\"Incorrect action ({action}) passed to process_queue_manager!\") \n        dbutils.notebook.exit(-1)\n          \n  \n###########################################################\n# ingest_file\n###########################################################\ndef ingest_file(file):\n    \"\"\"\n    ***Used by parallel_ingestion function for parallelizing loads***\n    \n    Processes the file (dataframe) into base table.\n     - Uses \"process_queue\" to track tables being written to (to avoid concurrency issues)\n     - Loads file into dataframe\n     - If required, executes preprocessing notebook, returning global view of data\n     - Adds metadata\n     - Saves metadata on Delta table (database, table, schema, candidate keys)\n     - Creates database/schema if it does not exist\n     - First time for table it is simply written out in Delta format (managed table)\n     - Subsequent writes use MERGE logic:\n       https://docs.delta.io/latest/api/python/index.html?highlight=merge#delta.tables.DeltaTable.merge\n       \n        For INCREMENTAL loads:\n         - Uses candidate key for lookup/comparison\n         - Flag/updates rows in Delta table as \"non current\" when matched and row hash differs\n         - Add rows from incoming dataframe where there was a match (and row hash differs), or candidate key is entirely new.\n         - No deletes\n        For FULL loads:\n         - Uses row hash for lookup/comparison\n         - Flags/updates all rows in Delta table as \"IsCurrent = False\" if they don't exist in the incoming dataset\n         - Add new rows from incoming dataset that don't exist in table (based on target iscurrent = True and row hash not exists)\n         - No deletes\n    \"\"\"\n    # Sleep so we don't have concurrency issues writing to tables\n    # This method sleeps files incrementally more, based on their row number in the Params file, to maintain order.\n    wait_sec = file.rowNum * (file.rowNum+2)\n    time.sleep(wait_sec)\n    \n    # Loop until all of the files have processed\n    while file.baseName not in bad_files:\n        \n        # If we're not already processing a file for this table, proceed.\n        if file.deltaObjectName not in process_queue: # and file.baseName not in bad_files: \n\n            # Add table to process queue to avoid concurrency\n            process_queue_manager('add', file.deltaObjectName)\n\n            # Load the file (potentially a preprocessed file), then \n            # make sure the data actually loaded into the dataframe before proceeding.\n            if not file_exists(file.fullFilePath):\n                # Add to \"bad files\" list so it doesn't try to reprocess\n                bad_files.append(file.baseName)\n                errMsg = f\"ERROR! Missing file {file.filePath}!\"\n                file.logger(errMsg, True, CRITICAL)\n                return \n              \n            if file.load_file():\n\n                # If a pre-processing notebook is required, run it here\n                if file.preprocessingNotebook:\n                    if not file.preprocess_file():\n                        file.logger(f\"Failed to run preprocessing for file {file.baseName}...\")\n\n                        # Remove table from processing queue\n                        process_queue_manager('delete', file.deltaObjectName)\n\n                        # Add to \"bad files\" list so it doesn't try to reprocess\n                        bad_files.append(file.baseName)\n\n                        return\n                    else:\n                        # Remove table from processing queue\n                        process_queue_manager('delete', file.deltaObjectName)\n\n                        # Add to \"bad files\" list so it doesn't try to reprocess\n                        bad_files.append(file.baseName)\n                  \n                # Add table to processing queue\n                file.logger(f\"Processing file {file.filePath} for base table: {file.deltaSchemaName}.{file.deltaObjectName}; Batch ID: {file.batchId}\")\n\n                # Add metadata columns, and if successful, proceed.\n                if file.add_metadata_columns():\n  \n                    # Trim unnecessary white space from string columns\n                    file.trim_string_columns()\n\n                    # Save file metadata to Delta state table for Optimization job\n                    file.store_delta_metadata()\n\n                    # Create database/schema as required\n                    check_if_db_exists(file.deltaSchemaName, True, DELTA_LOCATION)\n\n                    file.tableExists = check_if_table_exists(db_name=file.deltaSchemaName, table_name=file.deltaObjectName)\n\n                    # If table doesn't exist, simply write out dataframe as a new table\n                    if not file.tableExists:\n\n                        file.logger(f\"Creating table: {file.deltaSchemaName}.{file.deltaObjectName}\")\n                        #file.logger(f\"\\t Partition key: {file.deltaPartitionKey}\")\n                        #file.logger(f\"\\t{file.partitionKeyList}\")\n\n                        if file.executeQuery:\n                            # Using this method (over Spark SQL) creates Delta tables\n                            # as MANAGED, instead of UNMANAGED, to eliminate need for file system cleanup\n                            if file.create_base_table():\n                                # Remove table from processing queue\n                                process_queue_manager('delete', file.deltaObjectName)\n\n                                # Add to \"bad files\" list so it doesn't try to reprocess\n                                #bad_files.append(file.baseName)\n\n                                file.logger(f\"Created table: {file.deltaSchemaName}.{file.deltaObjectName}\")\n                            else:\n                                errMsg = f\"ERROR: Failure creating table: {file.deltaSchemaName}.{file.deltaObjectName}\"\n                                file.logger(errMsg, True, CRITICAL)\n\n                                # Remove table from processing queue\n                                process_queue_manager('delete', file.deltaObjectName)\n\n                                # Add to \"bad files\" list so it doesn't try to reprocess\n                                bad_files.append(file.baseName)\n\n                                return \n                        else:\n                            errMsg = f\"TESTING:: Created table: {file.deltaSchemaName}.{file.deltaObjectName}.\"\n                            file.logger(errMsg)\n\n                    # Otherwise we need to merge dataframe into existing table\n                    else:\n                        # Create an object from the base table\n                        baseTable = DeltaTable.forPath(spark, file.persistedMountPath)\n\n                        # Sync base table with changes (as appropriate)\n                        #file.sync_base_table_schema()\n\n\n                        if file.executeQuery:\n                            if file.df.count() > 0:\n                                file.logger(f\"Merging {file.fileType} file: {file.baseName} into: {file.deltaSchemaName}.{file.deltaObjectName}\")\n\n                                if file.fileType == 'INCREMENTAL':\n                                    if not file.merge_incremental_file():\n                                        # Remove table from processing queue so another file for same table can process\n                                        process_queue_manager('delete', file.deltaObjectName)\n\n                                        # Add to \"bad files\" list so this file doesn't try to reprocess\n                                        bad_files.append(file.baseName)\n\n                                        errMsg = f\"ERROR: Unable to merge file {file.baseName} into base table: {file.deltaSchemaName}.{file.deltaObjectName}\"\n                                        file.logger(errMsg, True, CRITICAL)\n                                        return \n\n                                # Full Load - Uses RowHash for comparison\n                                elif file.fileType == 'FULL':\n                                    if not file.merge_full_file():\n                                        # Remove table from processing queue so another file for same table can process\n                                        process_queue_manager('delete', file.deltaObjectName)\n\n                                        # Add to \"bad files\" list so this file doesn't try to reprocess\n                                        bad_files.append(file.baseName)\n\n                                        errMsg = f\"ERROR: Unable to merge file {file.baseName} into base table: {file.deltaSchemaName}.{file.deltaObjectName}\"\n                                        file.logger(errMsg, True, CRITICAL)\n                                        return \n\n                                # Unknown Load type\n                                else:\n                                    file.logger(f\"Invalid file type: {file.fileType}\")\n\n                                    # Remove table from processing queue\n                                    process_queue_manager('delete', file.deltaObjectName)\n\n                                    # Add to \"bad files\" list so it doesn't try to reprocess\n                                    bad_files.append(file.baseName)\n\n                                    errMsg = f\"ERROR:  Invalid file type for file {file.baseName} for base table: {file.deltaSchemaName}.{file.deltaObjectName}\"\n                                    file.logger(errMsg, True, CRITICAL)\n                                    return \n\n                                file.logger(f\"Merged {file.baseName} into: {file.deltaSchemaName}.{file.deltaObjectName}\")\n                            else:\n                                process_queue_manager('delete', file.deltaObjectName)\n                                file.logger(f\"File {file.baseName} contained no records for: {file.deltaSchemaName}.{file.deltaObjectName}\")\n                        else:\n                            errMsg = f\"TESTING:: Merged {file.baseName} into: {file.deltaSchemaName}.{file.deltaObjectName}\"\n                            file.logger(errMsg)\n\n                        # Remove table from processing queue\n                        process_queue_manager('delete', file.deltaObjectName)\n\n                    #end if create table or merge\n\n                    errMsg = f\"Successfully ingested {file.baseName} into {file.deltaSchemaName}.{file.deltaObjectName}\"\n                    file.logger(errMsg)\n                    # Log SUCCESS\n                    errMsg = \"\" #no message for success\n                    file.logger(errMsg, True, INFO) #True, so it will log it\n                    return \n                  \n                else:\n                    # Remove table from processing queue\n                    process_queue_manager('delete', file.deltaObjectName)\n                      \n                    # Add to \"bad files\" list so it doesn't try to reprocess\n                    bad_files.append(file.baseName)\n                    \n                    return\n                \n                break\n            else:\n                # Remove table from processing queue\n                process_queue_manager('delete', file.deltaObjectName)\n\n                # Add to \"bad files\" list so it doesn't try to reprocess\n                bad_files.append(file.baseName)\n\n                #errMsg = f\"ERROR processing file {file.baseName} for base table: {file.deltaSchemaName}.{file.deltaObjectName}\"\n                #file.logger(errMsg, True, CRITICAL)\n                return\n\n            break\n        else:\n             # Remove table from processing queue if it's in the bad_files list\n            if file.baseName in bad_files: \n                process_queue_manager('delete', file.deltaObjectName)\n              \n            file.logger(f\"File {file.baseName} is waiting to process into table {file.deltaObjectName}...\")\n            wait_sec = file.rowNum * (file.rowNum+2)\n            time.sleep(wait_sec)\n  \n        #end if file.deltaObjectName not in process_queue:\n    #end while True\n\ndef write_output_file():\n    \"\"\"\n    Generate the output file with an entry for each file processed\n    \"\"\"\n    # Quoting constant for writing CSV with Pandas \n    QUOTE_NONNUMERIC = 2\n\n    # Return batch errors as JSON \n    if batch_errors:\n        pdf = pd.DataFrame(batch_errors, columns = ['batchId','DeltaStartTimeUtc','DeltaEndTimeUtc','DeltaSuccess','DeltaError','DataChanged']).reindex()\n\n        # Convert a couple columns to Ints\n        pdf['batchId'] = pdf['batchId'].astype(int)\n        pdf['DeltaSuccess'] = pdf['DeltaSuccess'].astype(int)\n\n        # Generate error log file name, based on input file\n        outFile = fp.replace(\".csv\", \"_OUTPUTLOG.csv\")\n        # Add dbfs/mnt/ to file path\n        saveLocationParts = [\"dbfs\", \"mnt\", outFile]\n        saveLocation = slash_join(saveLocationParts, True) \n\n        # Write out file using Pandas for a single CSV file (without extra Spark metadata folder/files)\n        #try:\n        if not pdf.empty:\n            pdf[[\"batchId\",\"DeltaStartTimeUtc\",\"DeltaEndTimeUtc\",\"DeltaSuccess\",\"DeltaError\",\"DataChanged\"]].to_csv(saveLocation, sep=\"|\", index=False, quotechar='\"', quoting=QUOTE_NONNUMERIC)\n            outFile = outFile.replace(\"/dbfs/mnt/\", \"\")\n            #print(f\"OUTPUT file written to: {outFile}\")\n    else:\n        outFile = \"\"\n\n    return outFile\n  \n###########################################################\n# parallel_ingestion\n###########################################################\ndef parallel_ingestion(files, numInParallel=CORES):\n    \"\"\"\n    Main logic for parallel execution of file ingestion \n    Ref:  https://docs.python.org/3/library/concurrent.futures.html\n    \"\"\"\n    import operator\n    \n    # This code limits the number of parallel executions.\n    # For each file (class) object, execute \"ingest_file\" function in parallel\n    # but only if it is not already being processed (exists in batch_queue)\n    with ThreadPoolExecutor(max_workers=numInParallel) as executor:\n      return [executor.submit(ingest_file, file) for file in sorted(files, key=operator.attrgetter('rowNum'))] # if file not in process_queue]\n\n###########################################################\n# process_files\n###########################################################\ndef process_files():\n  \n    print(f\"{datetime.now():%Y-%m-%d %H:%M:%S}::Batch ingestion starting...\")\n\n    # Create empty list that will contain class instances of each file to be ingested\n    #files = []\n    file_count = len(params_dict[\"deltaobjectname\"])\n\n    # Create class instances for each file in the dictionary\n    for i in range(0, file_count):\n        files.append(\n                IngestionEngine(\n                  batchId = params_dict[\"batchid\"][i],\n                  dateFormat = params_dict[\"dateformat\"][i],\n                  deltaCandidateKey = params_dict[\"deltacandidatekey\"][i],\n                  deltaObjectName = params_dict[\"deltaobjectname\"][i],\n                  deltaSchemaName = params_dict[\"deltaschemaname\"][i],\n                  fileColDelimiter = params_dict[\"filecoldelimiter\"][i],\n                  fileFormat = params_dict[\"fileformat\"][i],\n                  fileHeader = params_dict[\"fileheader\"][i],\n                  filePath = params_dict[\"filepath\"][i],\n                  fileQuoteCharacter = params_dict[\"filequotecharacter\"][i],\n                  fileSchema = params_dict[\"fileschema\"][i],\n                  fileType = params_dict[\"filetype\"][i],\n                  preprocessingNotebook = params_dict[\"preprocessingnotebook\"][i],\n                  deltaPartitionKey = params_dict[\"deltapartitionkey\"][i],\n                  rowNum = params_dict[\"rownum\"][i],\n                  executeQuery = EXECUTE_QUERY\n                )\n        )\n        \n    # Parallel execution\n    res = parallel_ingestion(files, CORES)\n    result = [f.result(timeout=3600) for f in res] # This is a blocking call.\n\n    print(f\"{datetime.now():%Y-%m-%d %H:%M:%S}::Batch Ingestion completed!\")\n    \n    run_times = []\n    print(\"-----------------------------\")\n    print(\"Execution times per target object:\")\n    \n    for file in files:\n        if file.errorLog:\n            for err in file.errorLog:\n                batch_errors.append(err.split(\"|\"))\n\n        #print(f\"\\t{file.deltaObjectName.upper():30}\\t{((file.endTimeUtc-file.startTimeUtc).total_seconds()):.1f} seconds\")\n        run_times.append(f\"  {file.deltaObjectName.upper():30}\\t{((file.endTimeUtc-file.startTimeUtc).total_seconds()):.1f} seconds\")\n\n    run_times = sorted(run_times)\n    for rt in run_times:\n        print(rt)\n    print(\"-----------------------------\")\n    \n    if fp: #FileParams\n        # Generate output file\n        out = write_output_file()\n        print(f\"Batch Output file has been written to [{out}]:\")\n    else:\n        out = \"\"\n\n    print(\"\")\n\n    #Print out batch log\n    header = [\"BatchId\",\"DeltaStartTimeUtc\",\"DeltaEndTimeUtc\",\"DeltaSuccess\",\"DeltaError\",\"DataChanged\"]\n    \n    #Print dataframe with headers, and no row indexes\n    try:\n        print(pd.DataFrame(batch_errors, None, header).to_string(index=False))\n    except:\n        pass\n      \n    return out","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"cf01eee3-ef37-4396-9e9b-e7415717a62b"},{"version":"CommandV1","origId":3273046978841228,"guid":"32477890-67d7-4e21-a814-2d62d78eb5cf","subtype":"command","commandType":"auto","position":10.0,"command":"%md\n### Execute Notebook","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"6a935fe8-b7a6-4d72-a3cb-15d306ee9e90"},{"version":"CommandV1","origId":3273046978841229,"guid":"8542f493-677c-486f-885c-20f3e0936aef","subtype":"command","commandType":"auto","position":11.0,"command":"files = []\nif __name__ == '__main__':\n    # Get widget/param value\n    fp = dbutils.widgets.get(\"FileParams\")\n    \n    # Load config from param\n    params_dict = load_batch_config(fp)\n    \n    # If we successfully loaded config, ingest files\n    if params_dict:\n        output = process_files()\n    else:\n        raise ValueError(\"Missing or unable to read BATCH configuration/parameters!\") \n        dbutils.notebook.exit(-1)\n","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"977c9fab-c564-4e3c-8136-bb522acc1b15"},{"version":"CommandV1","origId":3273046978841230,"guid":"16264eca-f4d6-4382-be60-e97d2544bdfe","subtype":"command","commandType":"auto","position":12.0,"command":"%md\n### Display Batch Configuration File Contents","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"7c859e32-c7f9-41a3-8b9f-97dda9f4d9b3"},{"version":"CommandV1","origId":3273046978841231,"guid":"d074d913-b1be-4bf8-bb4e-908d3a7f432a","subtype":"command","commandType":"auto","position":13.0,"command":"try:\n    pdf = pd.DataFrame(params_dict)\n    pdf = pdf.fillna(\"\")\n    display(pdf)\nexcept:\n    #print(params_dict)\n    pass","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"64c09858-3344-4481-ab1f-e4a2449b51f7"},{"version":"CommandV1","origId":3273046978841232,"guid":"ea864410-1b45-4d97-afde-056fa4146c17","subtype":"command","commandType":"auto","position":14.0,"command":"%md\n### Exit and Return Output","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"e25cb759-fb2c-4f1e-8625-5f8887728210"},{"version":"CommandV1","origId":3273046978841233,"guid":"bd35dccc-b23b-4194-b60b-dfb649804caf","subtype":"command","commandType":"auto","position":15.0,"command":"# Exit with reference to output file\ndbutils.notebook.exit(outFile)","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"c171be0a-e36a-4d2c-a9fa-7a5141ad1360"}],"dashboards":[],"guid":"246b8c8d-4daf-4be1-9064-44aa5921409a","globalVars":{},"iPythonMetadata":{},"inputWidgets":{"Execute Query":{"nuid":"907f9295-23bf-4628-9fde-a714cceb2263","currentValue":"TRUE","widgetInfo":{"widgetType":"dropdown","name":"Execute Query","defaultValue":"TRUE","label":null,"options":{"widgetType":"dropdown","choices":["TRUE","FALSE"]}}},"FileParams":{"nuid":"72363e89-84d6-4d08-bcdf-4b85dc36a678","currentValue":"","widgetInfo":{"widgetType":"text","name":"FileParams","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}}},"notebookMetadata":{"pythonIndentUnit":2}}
