{"version":"NotebookV1","origId":2064366854103202,"name":"cls_transformation_engine","language":"python","commands":[{"version":"CommandV1","origId":2064366854103203,"guid":"6e141e2b-63d2-47e9-903d-8faf2dadd013","subtype":"command","commandType":"auto","position":2.0,"command":"#\n# Transformation Engine Class\n#","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"bdc8450c-a093-42ca-ada7-ecd9c6ea30f7"},{"version":"CommandV1","origId":2064366854103204,"guid":"b522d644-ff7d-4da6-a7c3-1ad4b4522e71","subtype":"command","commandType":"auto","position":3.0,"command":"from datetime import timedelta, datetime\nimport time\n\nfrom pyspark.sql import Row\nfrom pyspark.sql.types import *\nfrom pyspark.sql.functions import *\nfrom pyspark.sql.window import * #import Window\n\nimport os #for adding filename to dataframe\nimport json #Optionally supports JSON-formatted parameter vs CSV file\nimport pandas as pd #Used to greatly simplify converting JSON payload into a dataframe\nimport ast\n\n# Necessary SESSION-level setting for automatic schema update\nspark.conf.set(\"spark.databricks.delta.schema.autoMerge\", \"true\")\n\n# Necessary SESSION-level setting due to changes in Spark 3.0 date/time parsing\n# The alternative is fixing all transformations with string->date parsing\n#spark.conf.set(\"spark.sql.legacy.timeParserPolicy\", \"legacy\")\n\n# Handle legacy dates - Requires Spark 3+ with Proleptic Gregorian calendar for all reads\nspark.conf.set(\"spark.sql.legacy.parquet.int96RebaseModeInWrite\", \"CORRECTED\")\n\n# Enable optimized writes\nspark.conf.set(\"spark.databricks.delta.optimizeWrite.enabled\",\"true\")\n\n","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"e1b5c593-2872-4ac0-b48b-aa1d5b17e705"},{"version":"CommandV1","origId":2064366854103205,"guid":"11933017-20c6-41ea-9209-f33e2b6e9f1b","subtype":"command","commandType":"auto","position":4.0,"command":"%run /DataEngineering/ClassModules/cls_utils","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Load Shared Utilities Class","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"a0843505-72d0-4af0-a3f1-4ff19d6356be"},{"version":"CommandV1","origId":2064366854103206,"guid":"6667cac8-4e78-4b2f-ba8f-7e4a6189759b","subtype":"command","commandType":"auto","position":5.0,"command":"# Instantiate the Utils library\nutilsLib = Utils()","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"bb970fe8-9a77-42b6-88a9-060ef7beac82"},{"version":"CommandV1","origId":2064366854103207,"guid":"4261ffad-e54f-4c0f-b1b7-aab6df108430","subtype":"command","commandType":"auto","position":6.0,"command":"class TransformationEngine:\n    \"\"\"\n    Represents a table and its associated transformations/code mappings to be transformed.\n    \"\"\"\n\n    DELTA_LOCATION = \"DELTA\"\n    STAGE_LOCATION = \"STAGE\"\n    \n    ADD_METADATA_COLUMNS = True\n    META_COLUMN_PREFIX = \"dl_\"\n    \n    STATE_DB_NAME = 'state'\n    STATE_TBL_NAME = 'tablemetadata' # Case sensitive!!!\n\n    # Current UTC time used for ingestion\n    dtUTC = datetime.utcnow()\n    BATCH_TIME = dtUTC.isoformat() # UTC timestamp for type 2 column\n    BATCH_TIME_STR = dtUTC.strftime('%Y-%m-%dT%H:%M:%S.%fZ') # UTC timestamp for MERGE commands\n\n    UNMAPPED_STRING = \"[UnmappedValue]\"\n    \n    # Error logging\n    CRITICAL = 'CRITICAL'\n    ERROR    = 'ERROR'\n    WARNING  = 'WARNING'\n    INFO     = 'INFO'\n    DEBUG    = 'DEBUG'\n    \n    def __init__(self, configTable = None, codemapTable = None,\n                 operator = None, deltaSchemaName = None, deltaObjectName = None, rowNum=0):\n        self.operator = operator.lower()\n        self.deltaSchemaName = deltaSchemaName.lower()\n        self.deltaObjectName = deltaObjectName # Leave case-sensitive\n        self.configTableRaw = configTable   # Don't lower case this because it might be a file reference\n        self.codemapTableRaw = codemapTable # Don't lower case this because it might be a file reference\n        self.configTableDf = None\n        self.codemapTableDf = None \n        self.persistedMountPath = f\"/mnt/{self.DELTA_LOCATION}/{self.deltaSchemaName}/{self.deltaObjectName}\"\n        self.errorLog = []\n        self.rowNum = rowNum\n        self.customTransformation = False\n        self.configDict = {}\n        self.joinAliases = []\n        self.transformationQuery = None\n        self.codemapQuery = None\n        self.transformedDf = None # Dataframe of transformed data (executed self.transformationQuery)\n        self.startTimeUtc = datetime.utcnow()\n        self.endTimeUtc = None\n        self.success = True\n        self.unmappedCodes = \"\"\n        \n    ###########################################################\n    # logger\n    ###########################################################\n    def logger(self, message, log_message=False, severity_level=INFO):\n        \"\"\"\n        A simple logger that adds timestamp and severity level to print message.\n        Optionally adds message to instance's \"log\".\n        \"\"\"\n        dtNow = datetime.utcnow()\n       \n        if len(message) > 0:\n            # Simple message to console\n            printMsg = f\"{dtNow:%Y-%m-%d %H:%M:%S}::{message}\"\n\n            # \"Thread-safe\" the print function, or some things print on the same line\n            print(f\"{printMsg}\\n\", end =\"\")\n            \n            # Add to global log so it can be used for output, as necessary\n            execution_log.append(printMsg)\n\n        # Log entry for output\n        if log_message:\n            success = 1\n            if severity_level== self.CRITICAL:\n                success = 0\n            # Set end time\n            self.endTimeUtc = dtNow\n            # Build output message\n            logEntry = f\"{self.deltaSchemaName}|{self.deltaObjectName}|{self.startTimeUtc:%Y-%m-%d %H:%M:%S}|{self.endTimeUtc:%Y-%m-%d %H:%M:%S}|{self.unmappedCodes}|{success}|{message}\"\n            self.errorLog.append(logEntry)\n\n    def transform_table(self, execute_query):\n        \"\"\"\n        Primary controlling method to processes transformations.\n        Performs code mapping (if exists)\n        Writes data to 3nfstage\n         - Creates schema if it does not exist\n         - Creates or overwrites table in Delta format (managed table)\n        \"\"\"\n        # Load the transformation config\n        self.configTableDf = self.load_config(self.configTableRaw)\n        \n        if self.configTableDf:\n            config_count = self.configTableDf.count()\n        else:\n            config_count = 0\n        \n        self.logger(f\"{config_count} configuration records were found for target table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}\")\n        \n        if not self.configTableDf or config_count == 0:\n            errMsg = f\"ERROR loading transformation configuration for target table: {self.deltaSchemaName}.{self.deltaObjectName}\"\n            self.logger(errMsg, True, self.CRITICAL)\n            return #errMsg\n\n        # Create dictionary from dataframe to iterate through\n        self.configDict = utilsLib.dict_from_dataframe(self.configTableDf)\n\n        # Load the code mapping config (if exists)\n        if self.codemapTableRaw:\n            self.codemapTableDf = self.load_config(self.codemapTableRaw)\n\n        self.logger(f\"Processing transformations for target table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}\")\n\n        # Add metadata columns, and if successful, proceed.\n        if self.ADD_METADATA_COLUMNS:\n            if not self.add_metadata_columns():\n                errMsg = f\"ERROR adding metadata columns to configuration for target table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}\"\n                self.logger(errMsg, True, self.CRITICAL)\n                return #errMsg\n\n        if not self.create_dataframe_and_dictionary():\n            errMsg = f\"ERROR creating dataframe and dictionary for target table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}\"\n            self.logger(errMsg, True, self.CRITICAL)\n            return #errMsg\n\n        if self.customTransformation:\n            self.logger(f\"Identified CUSTOM transformations for target table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}\")\n            if not self.execute_custom_transformation():\n                errMsg = f\"ERROR executing custom transformation for target table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}\"\n                self.logger(errMsg, True, self.CRITICAL)\n                return #errMsg\n\n        else:  \n            # If this isn't a custom transformation, build the query\n            if not self.build_transformation_query():\n                #errMsg = f\"ERROR building transformation query for target table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}\"\n                #self.logger(errMsg, True, self.CRITICAL)\n                return #errMsg\n\n        if execute_query:\n            #if not self.execute_transformation_query():\n            error = self.execute_transformation_query()\n            if error:\n                errMsg = f\"ERROR executing transformation for target table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}.  {error}\"\n                self.logger(errMsg, True, self.CRITICAL)\n                return #errMsg\n\n        # if we're not just testing transformations, do the code mapping\n        if not ADHOC_MODE:\n            # Don't bother with code mapping if there aren't any\n            if self.codemapTableDf:\n                if not self.get_code_mappings():\n                    errMsg = f\"ERROR getting code mappings for target table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}\"\n                    self.logger(errMsg, True, self.CRITICAL)\n                    return #errMsg\n\n                if not self.execute_codemap_query():\n                    errMsg = f\"ERROR executing query with code mappings for target table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}\"\n                    self.logger(errMsg, True, self.CRITICAL)\n                    return #errMsg\n\n                # Get the unmapped codes file reference (as appropriate)\n                self.unmappedCodesFile = self.get_unmapped_codes()\n\n            # Create database/schema as required\n            utilsLib.check_if_db_exists(self.deltaSchemaName, True, self.DELTA_LOCATION)\n\n            self.logger(f\"Creating table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}\")\n\n            if execute_query:\n                if utilsLib.create_delta_table_from_dataframe(_df = self.transformedDf, targetSchema=self.deltaSchemaName, targetTable=self.deltaObjectName, writeMode=\"overwrite\"):\n                    self.rowCount = self.transformedDf.count()\n                    self.logger(f\"Created table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}\")\n                else:\n                    errMsg = f\"ERROR: Failure creating table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}\"\n                    self.logger(errMsg, True, self.CRITICAL)\n            else:\n                errMsg = f\"TESTING:: Created table: {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}.\"\n                self.logger(errMsg)\n\n###                \n#             # Create database/schema as required\n#             utilsLib.check_if_db_exists(self.deltaSchemaName, True, DELTA_LOCATION)\n\n#             # Create (or overwrite) table\n#             self.logger(f\"Creating table: {self.deltaSchemaName}.{self.deltaObjectName}\")\n\n#             # Using this method (over Spark SQL) lets us use a single command to create or \n#             # replace the table, which allows for schema changes.\n#             # This also creates tables as MANAGED, instead of UNMANAGED so we can\n#             # drop table (if needed) without needing to do file system cleanup\n#             try:\n#                 self.transformedDf.write \\\n#                     .format(\"delta\") \\\n#                     .mode(\"overwrite\") \\\n#                     .option(\"overwriteSchema\", \"true\") \\\n#                     .saveAsTable(f\"{self.deltaSchemaName}.{self.deltaObjectName}\")\n\n#                 self.logger(f\"Successfully created table: {self.deltaSchemaName}.{self.deltaObjectName}\")\n#             except:\n#                 errMsg = f\"ERROR writing table: {self.deltaSchemaName}.{self.deltaObjectName}\"\n#                 self.logger(errMsg, True, CRITICAL)\n#                 return #errMsg\n\n        # If we made it this far, Log SUCCESS\n        errMsg = \"\" #no error message for success\n        self.logger(errMsg, True, self.INFO) #True, so it will log it\n        return self.errorLog\n            \n    ###########################################################\n    # get_full_file_path\n    ###########################################################\n    def get_full_file_path(self, file_path):\n        \"\"\"\n        Appends mount root (mnt) to filepath, ensuring slashes\n        are properly formatted.\n        \"\"\"\n        #Add Databricks/dbfs root mount point to file name\n        args = [\"mnt\", file_path]\n        return utilsLib.slash_join(args, True)  # handles existing leading slash (or not)\n      \n    ###########################################################\n    # load_config\n    ###########################################################\n    def load_config(self, config_table_raw):\n        \"\"\"\n        Loads transformation/code mapping configurations\n        from file/JSON into dataframe.\n        \"\"\"\n        jdf = None\n\n        if config_table_raw:\n\n            # First let's see if this parses as JSON\n            try:\n                # Convert JSON payload into list\n                config_table_load = config_table_raw #.lower()\n                configtbl_list = json.loads(config_table_load)\n\n                pdf = pd.DataFrame(configtbl_list)\n\n                pdf_schema = StructType()\n                for item in pdf.columns:\n                    if item.lower() == 'codemapped':\n                        pdf_schema.add(item, BooleanType())\n                    elif item.lower() == 'stage':\n                        pdf_schema.add(item, StringType())\n                    else:\n                      pdf_schema.add(item, StringType())\n                \n                jdf = spark.createDataFrame(pdf, pdf_schema)        \n            except:\n                #Not JSON, so assuming a file reference\n                if \"filestore/\" in config_table_raw.lower():\n                    configtbl_file = config_table_raw\n                else:\n                    #Add Databricks/dbfs root mount point to file name\n                    configtbl_file = self.get_full_file_path(config_table_raw)\n\n                if not utilsLib.file_exists(configtbl_file):\n                    errMsg = f\"ERROR! Failed to read file {configtbl_file}!\"\n                    self.logger(errMsg, True, self.CRITICAL)\n\n                self.logger(f\"Loading configuration file from {config_table_raw} for table {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}...\")\n\n                # Read the file\n                if configtbl_file.endswith(\".csv\"):\n                    jdf = (spark \n                                .read \n                                .format('csv') \n                                .option(\"header\", \"true\") \n                                .option(\"inferSchema\", \"true\") \n                                .load(configtbl_file)\n                              )\n                elif configtbl_file.endswith(\".orc\"):\n                    jdf = (spark.read\n                                .format('orc')\n                                .load(configtbl_file)\n                             )\n\n            if jdf.count() == 0:\n                errMsg = f\"The configuration file ({config_table_raw}) contains no records for target table {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}!\"\n                self.logger(errMsg)\n                return None\n            else:\n                # Add row number to maintain original order of columns/transformations\n                jdf = jdf.withColumn(\"rownum\", row_number().over(Window.orderBy(monotonically_increasing_id())))\n                pdf_schema = jdf.schema\n\n                # Clean up column names (and convert to lower case)\n                new_column_name_list = list(map(lambda x: utilsLib.clean_string(x, \"\", True), jdf.columns))\n                str_clean_cols = str(new_column_name_list).strip(\"[]\")\n\n                # Assign cleaned column names to existing dataframe\n                jdf = jdf.toDF(*new_column_name_list)\n\n                if 'stage' in jdf.columns:\n                    jdf = jdf.fillna({'stage':'0'})\n\n                if 'uniongroup' in jdf.columns:\n                    jdf = jdf.fillna({'uniongroup':'0'})\n\n                # These 2 conditions will filter the dataframe to the respective table,\n                # for either the code mappings or transformations.  Make sure it's case-insensitive.\n                if 'targettablename' in jdf.columns:\n                    jdf = jdf.filter(lower(jdf[\"targettablename\"]) == self.deltaObjectName.lower())\n                elif 'targetobject' in jdf.columns:\n                    jdf = jdf.filter(lower(jdf[\"targetobject\"]) == self.deltaObjectName.lower())\n                else:\n                    errMsg = f\"ERROR! Configuration file ({configtbl_file}) is missing column headers!\"\n                    self.logger(errMsg, True, self.CRITICAL)\n                    return None\n\n                if jdf.count() == 0:\n                    errMsg = f\"The configuration file ({config_table_raw}) contains no records for target table {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}!\"\n                    self.logger(errMsg)\n                    return None\n\n                # If this config is for transformations:\n                if 'transformation' in jdf.columns:\n                    # Cast columns in Transformation config to ensure proper datatypes\n                    jdf = jdf.withColumn('uniongroup', col('uniongroup').cast(IntegerType()))\n                    jdf = jdf.withColumn('stage', col('stage').cast(IntegerType()))\n                    jdf = jdf.withColumn('codemapped', col('codemapped').cast(BooleanType()))\n                    jdf = jdf.withColumn('rownum', col('rownum').cast(IntegerType()))\n\n                    # Add a few more important columns\n                    jdf = jdf.withColumn('operator', lit(self.operator))\n                    jdf = jdf.withColumn('targetschema', lit(self.deltaSchemaName))\n                    jdf = jdf.withColumn('targetobject', lit(self.deltaObjectName))\n\n                    jdf = jdf.select(\"operationtype\",\"operator\",\"targetschema\",\"targetobject\",\"targetattribute\",\"sourceschema\",\"sourceobject\",\"sourceattribute\",\n                                     \"uniongroup\",\"joingroup\",\"stage\",\"transformationtype\",\"transformation\",\"codemapped\",\"rownum\")\n\n                # Convert contents to lower case\n                check_cols = [\"operationtype\", \"sourceschema\", \"transformationtype\",\"sourceobject\",\"joingroup\"]\n                for c, dt in jdf.dtypes:\n                    if c in check_cols and dt == 'string':\n                        jdf = jdf.withColumn(c, ltrim(rtrim(lower(col(c)))))\n                    \n        #end if config_table_raw\n        return jdf\n\n    ###########################################################\n    # add_metadata_columns\n    ###########################################################\n    def add_metadata_columns(self):\n        \"\"\"\n        Adds metadata (dl_*) columns into transformation configuration\n        \"\"\"\n        dfct = None\n        dfct = self.configTableDf \n        dfct.createOrReplaceTempView(f\"vw_{self.deltaObjectName}\")\n \n        # Create a dataframe containing metadata columns, per STAGE and UNIONGROUP\n        query = f\"\"\"\n        select distinct v.OperationType, v.Operator, '{self.deltaSchemaName}' AS TargetSchema, '{self.deltaObjectName}' as TargetObject, \n            meta_col as TargetAttribute, v.SourceSchema, v.SourceObject, meta_col as SourceAttribute, v.UnionGroup, v.JoinGroup,  \n            v.Stage, NULL as TransformationType, NULL as Transformation, False as CodeMapped, cj.RowNum\n        from vw_{self.deltaObjectName} v\n        cross join (\n            select 'dl_iscurrent' as meta_col, 9991 as RowNum\n            union\n            select 'dl_rowhash' as meta_col, 9992 as RowNum\n            union\n            select 'dl_recordstartdateutc' as meta_col, 9993 as RowNum\n            union\n            select 'dl_recordenddateutc' as meta_col, 9994 as RowNum\n            union\n            select 'dl_sourcefilename' as meta_col, 9995 as RowNum\n          ) cj\n        -- Outer join to eliminate tables that are used in relationships, per stage and uniongroup\n        -- Sometimes the joingroup contains the derived table alias, hence the coalesce below\n        left join vw_{self.deltaObjectName} v2\n            on ifnull(v2.sourceschema,'') = ifnull(v.sourceschema,'') \n            and coalesce(trim(v2.joingroup),trim(v2.sourceobject)) = trim(v.sourceobject)\n            and v2.operationtype = 'relationship'\n            and ifnull(v2.stage,0) = ifnull(v.stage,0) \n            and v2.uniongroup = v.uniongroup \n        where \n            v.operationtype = 'attributemapping'\n            and v2.operationtype is null --outer join filter\n            and not (v.sourceobject is null and v.stage = 0)\n            and not (v.sourceobject is null and v.transformationtype is not null)\n        \"\"\"\n        # Execute query\n        meta_df = spark.sql(query)\n        \n        # Union metadata dataframe with self.configTableDf\n        dfct = dfct.union(meta_df)\n        \n        # Remove dupes\n        dfct = dfct.distinct()\n        dfct = dfct.dropDuplicates(['operator', 'operationType', 'targetschema', 'targetobject', 'targetattribute', 'sourceschema', 'sourceobject', 'sourceattribute', 'uniongroup', 'joingroup', 'stage', 'transformationtype', 'transformation', 'codemapped'])\n        \n        # Store unioned results back to class instance\n        self.configTableDf = dfct\n        # Cleanup\n        spark.catalog.dropTempView(f\"vw_{self.deltaObjectName}\")\n        return True\n      \n    ###########################################################\n    # create_dataframe_and_dictionary\n    ###########################################################\n    def create_dataframe_and_dictionary(self):\n        \"\"\"\n        Create a dataframe and dictionary of transformations, that includes a derived SourceAlias\n        \"\"\"\n        dtUTC = datetime.utcnow()\n        source_view = \"vw_\" + self.deltaObjectName + \"_\" + dtUTC.strftime('%f')\n        self.configTableDf.createOrReplaceTempView(source_view)\n        spark.sql(f\"CACHE TABLE {source_view}\")\n  \n        query = f\"\"\"\n                with t AS (\n                    -- Grab everything for this transformation, \n                    -- PLUS create the basis for a table alias\n                    select OperationType, Operator, TargetSchema, TargetObject, TargetAttribute,\n                        SourceSchema, SourceObject, COALESCE(UnionGroup, 0) AS UnionGroup, COALESCE(JoinGroup,'') AS JoinGroup,\n                        COALESCE(Stage, 0) as Stage, SourceAttribute, TransformationType, Transformation, \n                        CodeMapped, LEFT(SourceObject,2)||LEFT(REVERSE(SourceObject),2) AS regr, RowNum\n                    from {source_view} \n                    where lower(operator) = lower('{self.operator}') \n                    and lower(TargetObject) = lower('{self.deltaObjectName}') \n                    and lower(TargetSchema) = lower('{self.deltaSchemaName}')\n                ),\n                -- Generate a SourceAlias for each Source DB/Table\n                t2 AS (\n                    select t.*,\n                          --coalesce(left(regr,2) || reverse(left(reverse(regr),2)) || ifnull(JoinGroup, UnionGroup), coalesce(left(regr,3))) as SourceAlias\n                          --CONCAT_WS('_', SourceObject, JoinGroup) as SourceAlias\n                          case\n                            when t.SourceObject like '(%' then t.JoinGroup\n                            when t.SourceObject is NULL then ''\n                            --else CONCAT(t.SourceObject, t.JoinGroup)  --, t.UnionGroup) --Might need to include UnionGroup?\n                            else case\n                                    when ifnull(t.JoinGroup,'') != '' then t.JoinGroup\n                                    else CONCAT(t.SourceObject, t.JoinGroup)  --, t.UnionGroup) --Might need to include UnionGroup?\n                                 end\n                          end as SourceAlias\n                    from t\n                )\n                -- Need to create a JoinedAlias column that relates JOINs with the alias they JOIN to \n                SELECT DISTINCT\n                    t2.OperationType, t2.Operator, t2.TargetSchema, t2.TargetObject, t2.TargetAttribute,\n                    t2.SourceSchema, t2.SourceObject, t2.SourceAttribute, t2.CodeMapped, t2.Stage,\n                    COALESCE(t2.UnionGroup, 0) AS UnionGroup, COALESCE(t2.JoinGroup, 0) AS JoinGroup, \n                    t2.TransformationType, t2.Transformation, RowNum, \n                    --t2.SourceAlias, \n                    case when t2.SourceAlias IS NULL then\n                        --(select SourceAlias from t2 where t2.JoinGroup = JoinGroup and lower(OperationType) = 'relationship' LIMIT 1)\n                        (select SourceAlias from t2 where ifnull(t2.JoinGroup,'') = ifnull(JoinGroup,'') and lower(OperationType) = 'relationship' LIMIT 1)\n                      else\n                        t2.SourceAlias\n                      end as SourceAlias,\n                    vw.SourceAlias as JoinedAlias,\n                    case \n                        when lower(t2.OperationType) == 'attributemapping' \n                          then case when lower(t2.TargetAttribute) like 'dl_%' then 3 else 0 end\n                        when lower(t2.OperationType) == 'pivot' then 2\n                        when lower(t2.OperationType) == 'relationship' then 6\n                        when lower(t2.OperationType) == 'filter' then 7\n                        when lower(t2.OperationType) == 'union' then 8\n                        else 4\n                    end AS SortOrder\n                FROM t2\n                LEFT JOIN (select distinct OperationType, UnionGroup, SourceAlias, Stage \n                           from t2 \n                           where lower(OperationType)=='attributemapping'\n                           and SourceAlias not in (select SourceAlias from t2 where lower(OperationType) == 'relationship')\n                           and ifnull(sourcealias,'') != ''\n                           ) AS vw\n                ON vw.UnionGroup = t2.UnionGroup\n                AND vw.SourceAlias != t2.SourceAlias\n                AND lower(t2.OperationType)=='relationship'\n                AND vw.Stage = t2.Stage\n        \"\"\"\n        df = None  \n        # Execute the query\n        df = spark.sql(query)\n        df.cache()\n        #spark.sql(f\"DROP VIEW {source_view}\")\n  \n        # If there are any results, create a temp view for using later\n        if df.count() > 0:\n            self.configTableDf = df\n            self.configTableDf.createOrReplaceTempView(source_view)  # Re-use name from earlier\n            spark.sql(f\"CACHE TABLE {source_view}\")\n        else: # Nothing found, error out\n            errMsg = f\"ERROR! No matching records in configuration for table {self.deltaObjectName}!\"\n            self.logger(errMsg, True, self.CRITICAL)\n            return False\n       \n        join_aliases = []\n        custom_cnt = 0\n\n        #####################################################\n        # Create the main dataframe of attributes and actions\n        #####################################################\n        query = f\"\"\"select distinct OperationType, Operator, TargetSchema, TargetObject, TargetAttribute,\n                      SourceSchema, SourceObject, COALESCE(UnionGroup, 0) AS UnionGroup, COALESCE(Stage, 0) as Stage, SourceAttribute,\n                      TransformationType, Transformation, SourceAlias, SortOrder, CodeMapped, RowNum, JoinGroup,\n                      JoinedAlias\n                   from {source_view} vw\n                   order by Stage, UnionGroup, SortOrder, RowNum --, SourceAlias --, TargetAttribute --MUST INCLUDE TARGETATTRIBUTE SO DATASETS MATCH COLUMN POSITIONS\n                \"\"\"\n        # Execute the query\n        df = spark.sql(query)\n        df.cache()\n\n        self.configDict = utilsLib.dict_from_dataframe(df)\n\n        # See if there are CUSTOM transformations for this set\n        #custom_cnt = df.select(\"OperationType\").where(\"lower(OperationType) == 'custom'\").count()\n        self.customTransformation = (df.select(\"OperationType\").where(\"lower(OperationType) == 'custom'\").count() > 0)\n        \n        ##################################################\n        # Create a dataframe of JOINs\n        ##################################################\n        query = f\"\"\"select distinct concat_ws('_', stage, uniongroup, lower(sourcealias)) as JoinAliases\n                    from {source_view} tc\n                    left join (select distinct JoinedAlias from {source_view}) tc2\n                    on tc.sourcealias = tc2.joinedalias\n                    where ((lower(tc.OperationType) == 'relationship') and tc.sourcealias != '')\n                    and tc2.joinedalias is null\"\"\"\n\n        # Execute the query\n        df_join = spark.sql(query)\n        df_join.cache()\n        \n        if df_join.count() > 0:\n            # Create an iterable list from the dataframe\n            join_list = df_join.collect()\n\n            # Create a list of aliases used as JOINed tables\n            for j in join_list:\n                join_aliases.append(j.JoinAliases)\n\n            # Filter out any duplicates\n            self.joinAliases = list(set(join_aliases))\n            \n        return True\n\n    ###########################################################\n    # build_transformation_query\n    ###########################################################\n    def build_transformation_query(self):\n        \"\"\"\n        Build the transformation query from the configuration dictionary\n        \"\"\"\n        final_query = []\n        stage_list = []\n        stages = self.configTableDf.selectExpr(\"cast(stage as int)\").alias(\"stage\").distinct().sort(\"stage\").collect()\n        \n        for s in stages:\n            stage_list.append(s[\"stage\"])\n\n        if stage_list[0] == None:\n            stage_list[0] = 0\n        \n        # Process transformations by STAGE\n        for stage in stage_list:\n            # Initialize variables\n            insert_clause = \"\"\n            select_clause = \"SELECT \"\n\n            query = []\n            query.append(insert_clause)\n            query.append(select_clause)\n\n            from_clause = where_clause = union_clause = join_clause = selectivity_clause = limit_clause = ''\n            col_counter = 0\n            i = 0\n            last_ug = ''\n\n            inscols = []\n            pivot_clause = []\n\n            # Get a Count of output columns for building query\n            unique_attributes = []\n            for i, val in enumerate(self.configDict[\"TargetAttribute\"]):\n                if self.configDict[\"Stage\"][i] == stage and val != None:\n                    unique_attributes.append(val)\n\n            target_col_cnt = len(set(unique_attributes))\n\n            # We need to know if we're unpivoting because there will be extra columns not included in config\n            pivot_cnt = self.configTableDf.select(\"TransformationType\").where(f\"lower(OperationType) == 'pivot' and stage == {stage}\").count()\n            if pivot_cnt > 0:\n                target_col_cnt += 1\n\n            for i in (i for i in range(0, len(self.configDict['TargetAttribute'])) if self.configDict[\"Stage\"][i] == stage):\n\n                derived = False\n                ot = (self.configDict['OperationType'][i] or '').lower().strip()\n                xft = (self.configDict['TransformationType'][i] or '').lower().strip()\n                xf = (self.configDict['Transformation'][i] or '').strip()\n                ug = (self.configDict['UnionGroup'][i] or '')\n                ta = (self.configDict['TargetAttribute'][i] or '').strip()\n                sa = (self.configDict['SourceAlias'][i] or '').lower().strip()\n                sd = (self.configDict['SourceSchema'][i] or '').lower().strip()\n                st = (self.configDict['SourceObject'][i] or '').lower().strip()\n                sc = (self.configDict['SourceAttribute'][i] or '').strip()\n                ja = (self.configDict['JoinedAlias'][i] or '').strip()\n                op = (self.configDict['Operator'][i] or '').strip()\n                cm = (self.configDict['CodeMapped'][i] or '')\n                if st.startswith(\"(\"):\n                    derived = True\n\n                if ug != last_ug:\n                    col_counter = 0\n\n                #Since we're no longer automatically converting everything to lower case \n                # We need to make sure any substitution variables are in lower case\n                xf = xf.replace(\"SOURCEALIAS\"      , \"sourcealias\")\n                xf = xf.replace(\"JOINEDALIAS\"      , \"joinedalias\")\n                xf = xf.replace(\"SOURCEATTRIBUTE\"  , \"sourceattribute\")\n                xf = xf.replace(\"SOURCEOBJECT\"     , \"sourceobject\")\n                xf = xf.replace(\"SOURCESCHEMA\"     , \"sourceschema\")\n                xf = xf.replace(\"TARGETATTRIBUTE\"  , \"targetattribute\")\n                xf = xf.replace(\"TARGETOBJECT\"     , \"targetobject\")\n                xf = xf.replace(\"TARGETSCHEMA\"     , \"targetschema\")\n                xf = xf.replace(\"OPERATOR\"         , \"operator\")\n                xf = xf.replace(\"'PIVOTCOLUMN'\"    , \"'pivotcolumn'\")\n                xf = xf.replace(\"'PIVOTVALUES'\"    , \"'pivotvalues'\")\n                xf = xf.replace(\"'PIVOTCOLUMNSTO'\" , \"'pivotcolumnsto'\")\n                xf = xf.replace(\"'PIVOTVALUESTO'\"  , \"'pivotvaluesto'\")\n                xf = xf.replace(\"'PIVOTCOLUMNS'\"   , \"'pivotcolumns'\")\n                xf = xf.replace(\"'PIVOTCOLALIASES'\", \"'pivotcolaliases'\")\n                xf = xf.replace(\"'PIVOTCOLUMNS'\"   , \"'pivotcolumns'\")\n\n                #reset variables\n                newselcol = where_clause = join_clause = pivotcolto = pivotvalto = pivot_col = pivot_val = ''\n\n                # Build the current JOIN clause\n                if ot == 'relationship':\n                    # Don't add the source schema if it's a derived query\n                    if derived:\n                          join_clause = xft.upper() + ' ' + st + ' AS ' + sa + '\\n\\t' + 'ON ' + xf.format(sourcealias=sa, joinedalias=ja) #+ '\\n\\t' + f'AND DL_IsCurrent = True '\n                    else:\n                        join_clause = xft.upper() + ' ' + sd + '.' + st + ' AS ' + sa + '\\n\\t' + 'ON ' + xf.format(sourcealias=sa, joinedalias=ja) + '\\n\\t' + f'AND {sa}.DL_IsCurrent = True '\n\n                if ot == 'selectivity':\n                    if 'distinct' in xft:\n                        selectivity_clause = ((xft + ' ' + xf).strip()).upper() #if no transformation/params, strip blank\n                    elif 'limit' in xft:\n                        limit_clause = ((xft + ' ' + xf).strip()).upper() #if no transformation/params, strip blank\n\n                # Build the UNION clause (and store until used)\n                if ot == 'union':\n                    union_clause = xft.upper()\n\n                # If we've moved from the first UnionGroup to the next, add the UNION clause\n                if ug != last_ug and i > target_col_cnt:\n\n                    if len(union_clause) > 0:    \n                        query.append(union_clause)\n\n                    # Then add a new SELECT statement\n                    query.append(select_clause)\n\n                # If there IS a transformation\n                if ot == 'attributemapping' and len(xf) > 0 and len(xft) > 0:\n\n                    if xft == 'ansi':\n                        newselcol = xf\n                    elif xft == 'constant':\n                        newselcol = xf\n                        # If they didn't quote the constant, do it for them\n                        if \"'\" not in newselcol:\n                            newselcol = \"'\" + newselcol + \"'\"\n                        if newselcol.replace(\"'\",\"\").lower() == \"null\":\n                            newselcol = \"NULL\"\n                        else:\n                            if not newselcol.startswith(\"'\"):\n                                newselcol = \"'\" + newselcol\n                    elif xft == 'aggregate':\n                        newselcol = xf\n                    else:\n                        newselcol = \"\"\n                        errMsg = f\"ERROR! Unsupported Transformation Type: {xft} {xf} for {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}!\"\n                        self.logger(errMsg, True, self.CRITICAL)\n                        return False\n            \n                # No Transformation - simply maps source attribute to target attribute\n                else:\n                    if sc != '': #There's a source column\n                        if sa:\n                            newselcol = sa + '.' + sc\n                        else:\n                            newselcol = sc\n                        if sc == 'dl_iscurrent':\n                            newselcol = f\"IFNULL({newselcol}, True)\"\n                    else: # No source column provided, assume NULL\n                        newselcol = 'NULL'\n      \n                # Variable substitution\n                newselcol = newselcol.format(sourcealias=sa, sourceschema=sd, sourceobject=st, sourceattribute=sc, operator=op, joinedalias=ja)\n\n                # Filter Operation\n                if ot == 'filter':\n                    where_clause = 'WHERE {}'.format(xf)\n                    if sa != \"\":\n                        where_clause = where_clause.format(sourcealias=sa)\n                    else:\n                        where_clause = where_clause.replace(\"{sourcealias}.\", \"\")\n\n                # Pivot (or Unpivot) Operation\n                if ot == 'pivot':\n                    if xft == 'pivot':\n                        pivot_dict = eval(xf) #Convert dictionary string to actual dictionary type\n                        pivot_col = pivot_dict[\"pivotcolumn\"]\n                        pivot_val = pivot_dict[\"pivotvalues\"]\n                        pivot_clause = []\n                        tbl = sd + '.' + st\n                        pivoted_aliases = []\n                        pivoted_data = spark.sql(f'select distinct {pivot_col} from {tbl}').collect()\n                        for f in pivoted_data:\n                            pivoted_aliases.append(f.LeaseEventName)\n                        pivoted_aliases = \"'\" + \"','\".join(pivoted_aliases) + \"'\"\n                        pivot_clause.append('PIVOT (')\n                        pivot_clause.append(f'\\tFIRST({pivot_val})')\n                        pivot_clause.append(f'\\tFOR {pivot_col} IN ({pivoted_aliases}) \\n)')\n                    elif xft == 'unpivot':\n                        pivot_dict = eval(xf) #Convert dictionary string to actual dictionary type\n                        pivotcolto = pivot_dict[\"pivotcolumnsto\"]\n                        pivotvalto = pivot_dict[\"pivotvaluesto\"]\n                        stack_list = []\n                        cols = pivot_dict['pivotcolumns']\n                        # Pivotcolaliases aren't mandatory, so if it's missing, use column names as default values\n                        try:\n                            aliases = pivot_dict['pivotcolaliases']\n                        except:\n                            aliases = cols\n                        n = len(cols)\n                        for c in range(n):\n                            stack_list.append(\"'{}'\".format(aliases[c]) + \",\" + cols[c])\n\n                        stack_str = ','.join(stack_list)\n                        newselcol = \"stack({},{}) as ({},{})\".format(n, stack_str, pivotcolto, pivotvalto)\n\n                # If we're adding a column, append it to list\n                if len(newselcol) > 0 and len(ta) > 0 and ot == 'attributemapping':\n\n                      # Provide an alias for the column to map to Target Table\n                      newselcol = newselcol + ' AS {}'.format(ta)\n\n                      # Add column to INSERT list, as long as it's not a Transform\n                      if ta not in inscols and len(ta) > 0:\n                          inscols.append(ta)\n\n                      # Add column/transformation to the SELECT list, assuming it's not a duplicate of previous line\n                      line_to_add = '\\t' + newselcol + ','\n                      last_line = query[-1]\n                      if last_line != line_to_add:\n                          query.append(line_to_add)\n\n                      # Increment column counter so we know where we're at\n                      col_counter += 1 \n                            \n                elif ot == 'pivot' and pivotcolto != '':\n                    # Add pivot column and values column to INSERT and SELECT \n                    inscols.append(pivotcolto)\n                    inscols.append(pivotvalto)\n                    newselcol = f'\\t{newselcol},'  # Don't alias since this is a stack function\n                    query.append(newselcol)\n                    col_counter += 1\n\n                elif ot == 'pivot' and pivot_col != '':\n                    # Add pivot column and values column to INSERT and SELECT \n                    inscols.append(pivot_col)\n                    inscols.append(pivot_val)\n                    newselcol = f'\\t{sa}.{pivot_col},'  \n                    query.append(newselcol)\n                    newselcol = f'\\t{sa}.{pivot_val},'  \n                    query.append(newselcol)\n                    col_counter += 1\n\n                # If this isn't one of the JOINed tables, build the FROM clause (used either at the end of a single query, or each query if UNIONed)\n                if (str(stage)+\"_\"+str(ug or '0')+\"_\"+sa).lower() not in self.joinAliases and st != \"\":\n                    from_clause = f'FROM {sd}.{st} AS {sa}'\n\n                # Add the FROM clause, if not already present\n                if col_counter == target_col_cnt and from_clause not in query[-10:]:  #not in last 10 lines of query... allow for other joins/union groups/etc\n                    query.append(from_clause)\n\n                # Add the JOIN clause as appropriate\n                if col_counter == target_col_cnt and ot == 'relationship':\n                    query.append(join_clause)\n\n                # Add the WHERE clause as appropriate\n                if col_counter == target_col_cnt and ot == 'filter' and st != \"\":\n                    query.append(where_clause)\n\n                if ug != last_ug:\n                    last_ug = ug \n\n            # Create lists of lower case items\n            tt_list = []\n            tt_list[:] = [item.lower() for item in self.configDict['TransformationType'] if item is not None]\n            tf_list = []\n            tf_list[:] = [item.lower() for item in self.configDict['Transformation'] if item is not None]\n\n            # Check to see if we need to add a GROUP BY (case sensitive)\n            if 'aggregate' in tt_list and 'groupby' not in tt_list and ' over' not in ''.join(tf_list): #not part of a derived column windowing function\n                groupby_cols = []\n                for n in range(0, len(self.configDict['TargetAttribute'])):\n                    tt = (self.configDict['TransformationType'][n] or '').lower()\n                    ta = (self.configDict['TargetAttribute'][n] or '')\n                    if tt != 'aggregate' and tt != '' and ta != '':\n                        groupby_cols.append(ta)\n\n                # Add GROUP BY and applicable columns to query\n                if len(groupby_cols) > 0:\n                    query.append('GROUP BY \\n\\t' + ', \\n\\t'.join(list(set(groupby_cols))))\n\n            if len(pivot_clause) > 0:\n                query.insert(1, 'SELECT * FROM (') # Add to query, after INSERT clause\n                query.append(')')\n                for p in pivot_clause:\n                    query.append(p)\n\n            # Remove any blank lines\n            query[:] = [item for item in query if item != '']\n\n            # Format list of INSERT columns\n            inscols = '\\n\\t' + ',\\n\\t'.join(inscols) + '\\n'\n\n            # Add DISTINCT, as appropriate\n            if len(selectivity_clause) > 0:\n                for n, q in enumerate(query):  #Look for SELECTs to replace with SELECT DISTINCT\n                    if q.startswith(\"SELECT\"):\n                        new_select = \"SELECT \" + selectivity_clause\n                        if query[n-1] != new_select:\n                            query[n] = new_select\n                        else:\n                            query.pop(n)\n\n            # Add LIMIT clause, as appropriate\n            if len(limit_clause) > 0:\n                query.append(limit_clause)\n\n            # Create query string from list\n            sqlquery = ''\n            for q in query:\n                sqlquery += q + '\\n'\n\n            # String replacement for list of INSERT columns\n            sqlquery = sqlquery.format(inscols=inscols)\n\n            # Remove trailing commas on last select column\n            sqlquery = sqlquery.replace(',\\nFROM', '\\nFROM')\n\n            if st != \"\":\n                for q in query:\n                    final_query.append(q)\n\n            if stage > 1:\n                # Set up the final query to select from the derived table (inner query/state)\n                final_query.insert(0, 'SELECT')\n                i = 1\n                for q in query:\n                    if q != 'SELECT ':\n                        final_query.insert(i, q)\n                        i += 1\n                final_query.insert(i, 'FROM (')\n                final_query.append(')')\n\n                # Add the WHERE clause as appropriate\n                if where_clause:\n                    final_query.append(where_clause)\n\n                if final_query[0].startswith(\"SELECT\") and final_query[1].startswith(\"SELECT\"):\n                    final_query.pop(0)\n\n                sql_out = \"\"\n                for q in final_query:\n                    sql_out += q + '\\n'\n\n                # Remove trailing commas on last select column\n                sql_out = sql_out.replace(',\\nFROM', '\\nFROM')\n            else:\n                sql_out = sqlquery\n\n        self.transformationQuery = sql_out\n\n        return True\n      \n    ###########################################################\n    # execute_transformation_query\n    ###########################################################\n    def execute_transformation_query(self):\n        \"\"\"\n        Execute the transformation query\n        \"\"\"\n        try:\n            self.logger(f\"Testing query for table {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}...\")\n            self.transformedDf = spark.sql(self.transformationQuery) #see if it executes without throwing error\n            self.logger(f\"Query tested successfully for table {self.deltaSchemaName.upper()}.{self.deltaObjectName.upper()}!\")\n            return \"\"\n        except Exception as E:\n#             print(\"==============================================================\")\n#             print(f\"Query for {self.deltaObjectName}\")\n#             print(\"==============================================================\")\n#             print(self.transformationQuery)\n#             print(\"==============================================================\")\n            if hasattr(E, 'message'):\n                error = E.message[:200].replace('\\n','')\n            else:\n                error = str(E)[:200].replace('\\n','')\n            #errMsg = f\"ERROR with query: {error}\"\n            #self.logger(errMsg, True, CRITICAL)\n            return error\n        \n    ###########################################################\n    # get_code_mappings\n    ###########################################################\n    def get_code_mappings(self):\n        \"\"\"\n        Gets code mappings on transformed data:\n        1. Renames original code mapped column to <col>__RAW\n        2. Adds the following code mapped columns to existing dataframe:\n              <col>__PC  (Portfolio Company description)\n              <col>__AC  (Asset Class description)\n              <col>__ENT (Enterprise description)\n        \"\"\"\n        # If this table has Code Mappings, do it here.\n        if self.codemapTableDf: \n \n            # Get a list of columns from the Transformed dataframe that will be written out\n            df_cols = self.transformedDf.columns\n\n            # Find the position of the first \"dl_\" column so we can insert generated columns here\n            try:\n                df_insert_index = df_cols.index(\"dl_rowhash\")\n            except:\n                df_insert_index = len(df_cols)\n\n            # Get a list of code mapped columns\n            dtUTC = datetime.utcnow()\n            source_view = \"vw_\" + self.deltaObjectName + \"_\" + dtUTC.strftime('%f')\n            self.configTableDf.createOrReplaceTempView(source_view)\n            cm_cols = spark.sql(f\"select distinct targetattribute from {source_view} where codemapped = True\").collect()\n\n            join_list = []\n            cm_col_list = []\n\n            dtUTC = datetime.utcnow()\n            codemap_view = \"vw_\" + self.deltaObjectName + \"_cm_\" + dtUTC.strftime('%f')\n            self.codemapTableDf.createOrReplaceTempView(codemap_view)\n            spark.sql(f\"CACHE TABLE {codemap_view}\")\n            \n            for cm in cm_cols:\n                #Grab the column name being mapped\n                c = cm[\"targetattribute\"]\n\n                # Generate new columns to be added to output\n                df_cols.insert(df_insert_index,   f\"IF({c}__PC IS NULL,  IF({c}__RAW IS NULL, NULL, '{self.UNMAPPED_STRING}'), {c}__PC)  as {c}__PC\"),\n                df_cols.insert(df_insert_index+1, f\"IF({c}__AC IS NULL,  IF({c}__RAW IS NULL, NULL, '{self.UNMAPPED_STRING}'), {c}__AC)  as {c}__AC\"),\n                df_cols.insert(df_insert_index+2, f\"IF({c}__ENT IS NULL, IF({c}__RAW IS NULL, NULL, '{self.UNMAPPED_STRING}'), {c}__ENT) as {c}__ENT\"),\n\n                # Create a table alias for this query\n                c_alias = f\"t{c}\"\n\n                # Generate the JOIN to be used with the transformed data\n                # Each JOIN is to the Code Map table, for each code mapped column\n                # Each of these additional columns are added to the final output\n                query = f\"\"\"\n                  LEFT JOIN (\n                    select DISTINCT\n                        RawValue \n                      , PortCoDescription AS {c}__PC\n                      , AssetClassDescription AS {c}__AC\n                      , EnterpriseDescription AS {c}__ENT\n                    FROM {codemap_view} cm\n                    WHERE cm.TargetColumnName = '{c}'\n                  ) {c_alias} ON LOWER(TRIM({c}__RAW)) = LOWER(TRIM({c_alias}.RawValue))\"\"\"\n                join_list.append(query)\n                df_insert_index += 3\n\n            for cm in cm_cols:\n                #Grab the column name being mapped\n                c = cm[\"targetattribute\"]\n                for idx, elem in enumerate(df_cols):\n                    if elem == c:\n                        df_cols[idx] = f\"{c}__RAW\"\n                self.transformedDf = self.transformedDf.withColumnRenamed(c, f\"{c}__RAW\")       \n\n            # Replace the view since it has new column names now\n            dtUTC = datetime.utcnow()\n            source_view = \"vw_\" + self.deltaObjectName + \"_\" + dtUTC.strftime('%f')\n            self.transformedDf.createOrReplaceTempView(source_view)\n            spark.sql(f\"CACHE TABLE {source_view}\")\n\n            # Make strings from lists\n            joins_str = ''.join(join_list)\n            col_str = ', '.join(df_cols)\n\n            # Prepare final query, with all of the columns (including newly generated ones)\n            # and JOINs for each codemapped column\n            self.codemapQuery = f\"\"\"\n                  SELECT {col_str}\n                  FROM {source_view} AS target {joins_str}\n            \"\"\"\n            \n        return True\n\n    ###########################################################\n    # execute_codemap_query\n    ###########################################################\n    def execute_codemap_query(self):\n        \"\"\"\n        Execute the transformation query\n        \"\"\"\n        # Execute the query, storing to df_out Dataframe\n        self.transformedDf = spark.sql(self.codemapQuery)\n\n        # If we want to update the row hash\n        #self.transformedDf = self.transformedDf.withColumn(\"dl_rowhash\", sha2(concat_ws(\"||\", *self.transformedDf.columns), 256))\n\n        # In the event there are no values in column (typically for a placeholder), set datatype to string\n        # Otherwise the datatype is \"null\", which causes problems downstream\n        # Also, ensure the code mapped columns are all strings\n        for c, dt in self.transformedDf.dtypes:\n            if dt == 'null' or (dt != 'string' and (c.endswith(\"__RAW\") or c.endswith(\"__PC\") or c.endswith(\"__AC\") or c.endswith(\"__ENT\"))):\n                self.transformedDf = self.transformedDf.withColumn(c, self.transformedDf[c].cast(StringType()))\n\n        return True\n        \n    ###########################################################\n    # get_unmapped_codes\n    ###########################################################\n    def get_unmapped_codes(self):\n        \"\"\"\n        Here we need to find all the code values that don't have a mapping in ColumnCodeMap.  \n        To do this we're looking in the final transformed dataframe for any code-mapped columns, with raw values, but no __PC description.\n        \"\"\"\n        # If this table has Code Mappings, do it here.\n\n        #dtUTC = datetime.utcnow()\n        #source_view = \"vw_\" + self.deltaObjectName + \"_\" + dtUTC.strftime('%f')\n        #self.configTableDf.createOrReplaceTempView(source_view)\n        \n        # Get a list of code-mapped columns \n        #cm_cols = spark.sql(f\"select distinct targetattribute from {source_view} where codemapped = True\").collect()\n        cm_cols = self.configTableDf.select(\"targetattribute\").filter(self.configTableDf[\"codemapped\"] == True).distinct().collect()\n        #spark.sql(f\"DROP VIEW {source_view}\")\n\n        # It's possible there are code mappings, yet no transformations exist marked as codemapped\n        if cm_cols:\n            # Create a view to use below\n            dtUTC = datetime.utcnow()\n            source_view = \"vw_\" + self.deltaObjectName + \"_\" + dtUTC.strftime('%f')\n            self.transformedDf.createOrReplaceTempView(source_view)\n\n            qry_list = []\n\n            # Iterate through the list of code-mapped columns.\n            for cm in cm_cols:\n                c = cm[\"targetattribute\"]\n\n                query = f\"SELECT DISTINCT '{c}' AS TargetAttribute, {c}__RAW AS RawValue FROM {source_view} v WHERE {c}__RAW IS NOT NULL AND {c}__PC = '{self.UNMAPPED_STRING}'\"\n\n                if qry_list:\n                    qry_list.append(\"UNION\")\n                qry_list.append(query)\n\n            final_query = '\\n'.join(qry_list)\n\n            # Execute the query, store results in Dataframe\n            df_unmapped = spark.sql(final_query)\n\n            saveLocationParts = [\"dbfs\", \"mnt\", self.STAGE_LOCATION, \"unmapped_codes\", self.deltaSchemaName]\n            saveLocation = utilsLib.slash_join(saveLocationParts, True) \n            unmapped_codes_file = f\"{saveLocation}/{self.deltaObjectName}.csv\"\n\n            try:\n                if df_unmapped.count() > 0:\n                    # Create folders if they don't exist\n                    if not os.path.exists(saveLocation):\n                        os.makedirs(saveLocation)\n                    \n                    # Write out file with unmapped codes\n                    df_unmapped.toPandas().to_csv(unmapped_codes_file, index=False, quotechar='\"')\n                    \n                    # Clean up name for output\n                    unmapped_codes_file = unmapped_codes_file.replace(\"/dbfs/mnt/\", \"\")\n                    self.unmappedCodes = unmapped_codes_file\n                    \n                    self.logger(f\"Unmapped records for table {self.deltaObjectName} written to: {unmapped_codes_file}.\")\n            except:\n                pass\n                self.logger(f\"INFO: There are no unmapped records to output for table: {self.deltaObjectName}!\")\n                unmapped_codes_file = \"\"\n        else:\n            unmapped_codes_file = \"\"\n\n        return unmapped_codes_file\n      \n    ###########################################################\n    # execute_custom_transformation\n    ###########################################################\n    def execute_custom_transformation(self):\n        \"\"\"\n        Executes a specific notebook to transform data\n        into the desired format when transformation is too\n        complicated for standard transformation config.\n        \n        Returns global view representing preprocessed data\n        \"\"\"\n        timeout = 600\n        nb = self.configDict[\"Transformation\"][0]\n        args = {}\n        \n        if nb:\n            self.logger(f\"Running custom transformation notebook: {nb}...\")\n            \n            # Run the notebook, returning the global view name\n            try:\n                output = run_notebook_with_retry(notebook=nb, params=args, timeout=timeout, retries=0, debug=False)\n                # Extract query and log from output\n                output_dict = ast.literal_eval(output)\n                query = output_dict[\"SQLQuery\"]\n                error = output_dict[\"Log\"]\n\n                if query:\n                    self.logger(f\"Query: {query}... \")\n                    self.transformationQuery = query\n                \n                    # Replace the dataframe with the preprocessed data\n                    self.transformedDf = spark.sql(self.transformationQuery)\n                    return True\n                else:\n                    errMsg = f\"ERROR with custom query. {error}\"\n                    self.logger(errMsg, True, self.CRITICAL)\n                    return False\n            except:\n                errMsg = f\"ERROR executing custom transformation notebook {nb}.\"\n                self.logger(errMsg, True, ERROR)\n                vw_name\n                return False\n","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Transformation Engine Class","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"2fd0b807-b01d-4b16-872d-b6575bb023d2"}],"dashboards":[],"guid":"a9fb6fba-f13d-4d66-8a73-79eb928c3e91","globalVars":{},"iPythonMetadata":null,"inputWidgets":{},"notebookMetadata":{"pythonIndentUnit":2}}