{"version":"NotebookV1","origId":3273046978841423,"name":"IngestRAWandWriteDelta","language":"python","commands":[{"version":"CommandV1","origId":2986605948003623,"guid":"14c3d7a1-848d-49de-8ecb-3a6bd17aacbc","subtype":"command","commandType":"auto","position":1.0,"command":"%md\n<img align=\"left\" src=\"https://global.hitachi-solutions.com/-/media/Images/Logos/hitachi-solutions-logo.svg\" alt=\"Hitachi Logo\" width=\"300\"/>\n### Notebook Info\nDescription | Comment\n-------------- | -----------\nName | **IngestRAWandWriteDelta**\nPurpose | This notebook ingests single files from RAW to DELTA Zone, aka \"Base Tables\".\n|This notebook requires Spark 3 to function properly for schema evolution.\nAuthor |  Larry Edlin, ledlin@hitachisolutions.com<br>\nLast Updated| 1/27/2021<br>\nLast Updated By | LE\n\n### Input Parameters\nParameter Name | Description\n-------------- | -----------\nbatchId | batchId from ADF\ndateFormat | Format of dates in source file\ndeltaCandidateKey | Candidate/business key for source data.  Mandatory for INCREMENTAL loads.\ndeltaObjectName | Name of output Delta table\ndeltaSchemaName | Name of output Delta schema/database\ndeltaPartitionKey | User-defined partition key(s)\nfileFormat | ORC, PARQUET or CSV\nfileHeader | For CSV - whether file header exists in source file\nfilePath | Full path in ADLS to source file\nfileColDelimiter | Column delimiter used in CSV files\nfileQuoteCharacter | Quote character used in CSV files\nfileSchema | Schema to use for file load (overrides source file column names). Number of columns must match source.\nfileType | FULL or INCREMENTAL \npreprocessingParams | Parameters used for calling an optional preprocessing notebook\n\n### Output Parameters\nReturns JSON string with following key-value pairs:\n\nColumn Name | Description\n-------------- | -----------\nSuccess| 0 (failure) or 1 (success)\nMessage| Error message, as appropriate\nDeltaTable| Base table name written to Delta\nDataChanged| 0 (no data changed) or 1 (data changed) during Merge operation\n\n###TO DO:\nNr|Description\n----|----\n1|Do some basic sanity comparisons between incoming file and existing base table schema.\n|-If incoming file only has 1 column, abort, as it's probably corrupt (this has happened a couple times)\n|-Fail if column count difference between source/target is less/greater some defined \n|&nbsp;amount, as we don't want schema evolution occuring for a \"bad\" file.\n2|","commandVersion":2,"state":"finished","results":null,"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"512a55b5-1e91-4ae8-9b4a-2fd6a7600104"},{"version":"CommandV1","origId":2986605948003624,"guid":"7c6b71ca-9c9e-4a73-b477-f85c366de2bb","subtype":"command","commandType":"auto","position":2.0,"command":"from datetime import timedelta, datetime\nfrom pyspark.sql import Row\nfrom pyspark.sql.types import *\nfrom pyspark.sql.functions import *\nfrom pyspark.sql.window import Window as W\nfrom pyspark.storagelevel import StorageLevel\nfrom delta.tables import *\n\nimport os #for adding filename to dataframe\nimport re #regular expressions\n\ndelta_location = \"DELTA\"\n\n# DB and Table for Optimize stats metadata\nstate_db_name = 'state'\nstate_tbl_name = 'tablemetadata'\n\n# Get current UTC time to be used for entire batch of files to be ingested\ndtUTC = datetime.utcnow()\nBATCH_TIME = dtUTC.isoformat() # UTC timestamp for type 2 column\nBATCH_TIME_STR = dtUTC.strftime('%Y-%m-%dT%H:%M:%S.%fZ') # UTC timestamp for MERGE commands\n\n# Output variables\nsuccess = True\ndata_changed = 0\noutput_message = \"\"\ntable_exists = False\n\n# Necessary SESSION-level setting for automatic schema update\nspark.conf.set(\"spark.databricks.delta.schema.autoMerge.enabled\",\"true\")\n# Enable optimized writes\nspark.conf.set(\"spark.databricks.delta.optimizeWrite.enabled\",\"true\")\n# Repartition before write:  https://databricks.com/blog/2020/05/19/schema-evolution-in-merge-operations-and-operational-metrics-in-delta-lake.html\nspark.conf.set(\"spark.databricks.delta.merge.repartitionBeforeWrite.enabled\",\"true\")\n","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822759,"submitTime":1622642822447,"finishTime":1622642822782,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Imports and Session Settings","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"f8512130-2cc9-4fbd-b97f-3347e2fd5a52"},{"version":"CommandV1","origId":2986605948003625,"guid":"bf6e9cdb-24e4-4751-bbf3-8f98e4bce556","subtype":"command","commandType":"auto","position":3.0,"command":"%run /DataEngineering/SharedUtils","commandVersion":1,"state":"finished","results":null,"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822452,"submitTime":1622642822452,"finishTime":1622642823901,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Load shared functions module","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"6b301c80-38f4-4b7b-bc06-157920040f3a"},{"version":"CommandV1","origId":2986605948003626,"guid":"b952a4ad-c892-4ce9-a5bd-ce406686a2eb","subtype":"script","commandType":"auto","position":3.033333333333333,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661947888,"submitTime":0,"finishTime":1620661947910,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"d62311a7-be3c-498c-b12e-ca94329db284"},{"version":"CommandV1","origId":2986605948003627,"guid":"4eee0772-5d04-426f-af3b-2492df916eb7","subtype":"script","commandType":"auto","position":3.066666666666667,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661947914,"submitTime":0,"finishTime":1620661947934,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"6a9eb0d7-50a9-44e8-90e8-59a7369c43bf"},{"version":"CommandV1","origId":2986605948003628,"guid":"e070071d-8d69-4555-8fad-0d95f58b4ac4","subtype":"script","commandType":"auto","position":3.1,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661947938,"submitTime":0,"finishTime":1620661947971,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"98848bd5-ca5c-4049-9c09-249a58ffd562"},{"version":"CommandV1","origId":2986605948003629,"guid":"46d99c05-65de-4d10-a4a4-d8fb3b18579d","subtype":"script","commandType":"auto","position":3.1333333333333333,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661947976,"submitTime":0,"finishTime":1620661947999,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"16b69105-4b11-4b61-b3da-aa8772c529c7"},{"version":"CommandV1","origId":2986605948003630,"guid":"e8a4c1cb-ca85-46d5-8f14-d75db9e63003","subtype":"script","commandType":"auto","position":3.1666666666666665,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948003,"submitTime":0,"finishTime":1620661948023,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b26ae44c-4b25-45c1-81f2-d7545c4eec8a"},{"version":"CommandV1","origId":2986605948003631,"guid":"77b2c078-880a-4cd3-b43c-81b3a8d26903","subtype":"script","commandType":"auto","position":3.2,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948029,"submitTime":0,"finishTime":1620661948050,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"cce1a3d1-b0dc-4512-811f-294e7aa2af0b"},{"version":"CommandV1","origId":2986605948003632,"guid":"8e6187fd-2400-4839-8e17-1abc0bdcd6c5","subtype":"script","commandType":"auto","position":3.2333333333333334,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948061,"submitTime":0,"finishTime":1620661948080,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"abdc8deb-2f27-4cd3-9e0b-e69ef54a874c"},{"version":"CommandV1","origId":2986605948003633,"guid":"1da30b65-2fa6-4c5a-bb4b-42b1d4277fbd","subtype":"script","commandType":"auto","position":3.2666666666666666,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948084,"submitTime":0,"finishTime":1620661948103,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"7f528320-e891-4403-9e60-bb5b532b7384"},{"version":"CommandV1","origId":2986605948003634,"guid":"53807010-4ce7-4dc0-aef5-1d6843c337d9","subtype":"script","commandType":"auto","position":3.3,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948106,"submitTime":0,"finishTime":1620661948142,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"6c5783ec-7193-497c-ae48-8f359dd15d79"},{"version":"CommandV1","origId":2986605948003635,"guid":"0902a267-1446-42ca-9045-6aeeb48f89f6","subtype":"script","commandType":"auto","position":3.3333333333333335,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948149,"submitTime":0,"finishTime":1620661948169,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b6ecd665-7a6f-4c6b-bb5d-2e7cafaa1ece"},{"version":"CommandV1","origId":2986605948003636,"guid":"41ae7034-1367-4bc6-9074-4c00e19ca6c4","subtype":"script","commandType":"auto","position":3.3666666666666667,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948173,"submitTime":0,"finishTime":1620661948195,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b4d11207-d953-4fea-80bd-48307d1f101e"},{"version":"CommandV1","origId":2986605948003637,"guid":"419a8915-2fd2-4c52-b9cd-c9700c4e5433","subtype":"script","commandType":"auto","position":3.4,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948199,"submitTime":0,"finishTime":1620661948221,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b7b1e43b-7474-4cf0-ae05-b94893f23300"},{"version":"CommandV1","origId":2986605948003638,"guid":"86c52986-9a05-4819-9c21-229c7ae68cf3","subtype":"script","commandType":"auto","position":3.4333333333333336,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948224,"submitTime":0,"finishTime":1620661948246,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"67be1ec5-042a-4820-91b5-546c62afa16b"},{"version":"CommandV1","origId":2986605948003639,"guid":"0eca9019-dbdb-442b-afdd-2020824db059","subtype":"script","commandType":"auto","position":3.466666666666667,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948252,"submitTime":0,"finishTime":1620661948278,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"c9ba476d-bc46-4cb5-8273-442ad0c13aa1"},{"version":"CommandV1","origId":2986605948003640,"guid":"f50fa337-38de-4df2-bd24-931a143dbe3d","subtype":"script","commandType":"auto","position":3.5,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948282,"submitTime":0,"finishTime":1620661948301,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"61923e5c-f069-4991-bce1-d1998f03ca50"},{"version":"CommandV1","origId":2986605948003641,"guid":"9c9ddcba-e49d-4c87-89f9-529fc36663ce","subtype":"script","commandType":"auto","position":3.533333333333333,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948305,"submitTime":0,"finishTime":1620661948323,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"6a2386a8-3585-4f5b-8d1a-6a221008a3a8"},{"version":"CommandV1","origId":2986605948003642,"guid":"7913c336-5a1d-48bb-8b6f-05b382a2fa03","subtype":"script","commandType":"auto","position":3.5666666666666664,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948328,"submitTime":0,"finishTime":1620661948348,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"2034a836-ca3b-4e74-9c99-9c5406fae914"},{"version":"CommandV1","origId":2986605948003643,"guid":"d7098bda-8eb8-4cde-bf23-6e439955c2b2","subtype":"script","commandType":"auto","position":3.6,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948353,"submitTime":0,"finishTime":1620661948374,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"fa7038b8-eeae-4faf-b67c-3533f592c4be"},{"version":"CommandV1","origId":2986605948003644,"guid":"3096fc4b-8568-43cf-91a2-a23cafa533eb","subtype":"script","commandType":"auto","position":3.6333333333333333,"command":"","commandVersion":9,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948377,"submitTime":0,"finishTime":1620661948404,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"f7e130f3-a10c-4be5-b766-bf22f0eaa005"},{"version":"CommandV1","origId":2986605948003645,"guid":"54b7ef64-f2e0-45fe-b0de-a8b0438c4c3e","subtype":"script","commandType":"auto","position":3.6666666666666665,"command":"","commandVersion":19,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948408,"submitTime":0,"finishTime":1620661948427,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"cd3b7f3c-4cff-4081-8f26-8fee0f0b4dd6"},{"version":"CommandV1","origId":2986605948003646,"guid":"af1d0497-c0dd-4fcc-a7df-b05db0385520","subtype":"script","commandType":"auto","position":3.7,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948432,"submitTime":0,"finishTime":1620661948455,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"4a226d2a-a3e4-4c6d-98fa-85f8ba9827c9"},{"version":"CommandV1","origId":2986605948003647,"guid":"e3b1fb97-c4b4-4b5d-a13d-20175e9d44e7","subtype":"script","commandType":"auto","position":3.7333333333333334,"command":"","commandVersion":5,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948459,"submitTime":0,"finishTime":1620661948482,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"e1c71d16-34f1-47f0-bf92-ee4ac052e889"},{"version":"CommandV1","origId":2986605948003648,"guid":"2c383db4-6398-48c9-b40d-5af0b052e922","subtype":"script","commandType":"auto","position":3.7666666666666666,"command":"","commandVersion":4,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948485,"submitTime":0,"finishTime":1620661948511,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"0b9ccd37-947a-47e6-ae42-d1e165f3c085"},{"version":"CommandV1","origId":2986605948003649,"guid":"5261c4b3-53fc-418e-8711-6eaf89944b81","subtype":"script","commandType":"auto","position":3.8,"command":"","commandVersion":11,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948514,"submitTime":0,"finishTime":1620661948536,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":null,"yColumns":null,"pivotColumns":null,"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b0e73abd-c26a-4354-ba36-0b5f8d95caee"},{"version":"CommandV1","origId":2986605948003650,"guid":"c2809b96-b349-4688-988b-c6a30f5fea2a","subtype":"script","commandType":"auto","position":3.8333333333333335,"command":"","commandVersion":4,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948540,"submitTime":0,"finishTime":1620661948559,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"9e4e7bf9-f126-472c-9fbc-f4e8613533fb"},{"version":"CommandV1","origId":2986605948003651,"guid":"3fbc99af-04a3-4036-9c51-ef0a9e145aad","subtype":"script","commandType":"auto","position":3.8666666666666667,"command":"","commandVersion":42,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948562,"submitTime":0,"finishTime":1620661948583,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"2ebbc8ec-3d9a-4199-8649-df651f2387b6"},{"version":"CommandV1","origId":2986605948003652,"guid":"cee17933-1675-4325-bbaf-0c9b3c07ab15","subtype":"script","commandType":"auto","position":3.9,"command":"","commandVersion":3,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948587,"submitTime":0,"finishTime":1620661948607,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":null,"yColumns":null,"pivotColumns":null,"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"966dd1bf-77f9-4969-87ce-57a3dc32afb7"},{"version":"CommandV1","origId":2986605948003653,"guid":"71aab2ab-46c6-4a37-85ec-9f7d35544abb","subtype":"script","commandType":"auto","position":3.9333333333333336,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948611,"submitTime":0,"finishTime":1620661948784,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"5da50073-429f-401d-b40f-6379eeead438"},{"version":"CommandV1","origId":2986605948003654,"guid":"b54df0bf-f025-4e85-9645-7b5d106468c9","subtype":"script","commandType":"auto","position":3.966666666666667,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1620661948789,"submitTime":0,"finishTime":1620661948813,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["f77a3e84-875d-43f3-ad90-1f63da01fbb6"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"7a379d31-5ff4-4887-8d39-12f09f6645d0"},{"version":"CommandV1","origId":2986605948003655,"guid":"1371f506-0cd3-455a-b800-1b79dd72c0c5","subtype":"command","commandType":"auto","position":4.0,"command":"#dbutils.widgets.removeAll()\n\ndbutils.widgets.text(\"filePath\", \"\")\ndbutils.widgets.text(\"fileSchema\", \"\")\ndbutils.widgets.text(\"excludedColumns\", \"\")\ndbutils.widgets.text(\"CandidateKey\", \"\")\ndbutils.widgets.dropdown(\"fileType\", \"FULL\", [\"FULL\", \"INCREMENTAL\"])\ndbutils.widgets.dropdown(\"fileFormat\", \"ORC\", [\"PARQUET\", \"CSV\", \"ORC\"])\ndbutils.widgets.dropdown(\"fileHeader\", \"FALSE\", [\"TRUE\", \"FALSE\"])\ndbutils.widgets.text(\"dateFormat\", \"yyyy-MM-dd HH:mm:ss.S\")  #\"MM/dd/yyyy HH:mm:ss a\"\ndbutils.widgets.text(\"SchemaName\", \"\")\ndbutils.widgets.text(\"ObjectName\", \"\")\ndbutils.widgets.text(\"batchId\", \"\")\ndbutils.widgets.text(\"preprocessingParams\",\"\")\ndbutils.widgets.text(\"PartitionKey\", \"\") # Empty will default to dl_partitionkey, dl_iscurrent\n\n# Note: These defaults are represented in hex format for python, since they're not a typable character\n# For interactively testing other characters, you can open a file with those characters\n# in Notepad++ and copy the \"symbols\" from there into the widgets above.\ndbutils.widgets.text(\"fileColDelimiter\", \"\\x07\")  # Unicode character:  \\u0007   Alt+007\ndbutils.widgets.text(\"fileQuoteCharacter\", \"\\x0e\")  # Unicode character: \\u000e  Alt+014\n","commandVersion":3,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{"CandidateKey":{"widgetType":"text","name":"CandidateKey","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}},"PartitionKey":{"widgetType":"text","name":"PartitionKey","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}},"preprocessingParams":{"widgetType":"text","name":"preprocessingParams","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}},"fileType":{"widgetType":"dropdown","name":"fileType","defaultValue":"FULL","label":null,"options":{"widgetType":"dropdown","choices":["FULL","INCREMENTAL"]}},"fileColDelimiter":{"widgetType":"text","name":"fileColDelimiter","defaultValue":"\u0007","label":null,"options":{"widgetType":"text","validationRegex":null}},"fileQuoteCharacter":{"widgetType":"text","name":"fileQuoteCharacter","defaultValue":"\u000E","label":null,"options":{"widgetType":"text","validationRegex":null}},"ObjectName":{"widgetType":"text","name":"ObjectName","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}},"fileHeader":{"widgetType":"dropdown","name":"fileHeader","defaultValue":"FALSE","label":null,"options":{"widgetType":"dropdown","choices":["TRUE","FALSE"]}},"filePath":{"widgetType":"text","name":"filePath","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}},"SchemaName":{"widgetType":"text","name":"SchemaName","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}},"excludedColumns":{"widgetType":"text","name":"excludedColumns","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}},"dateFormat":{"widgetType":"text","name":"dateFormat","defaultValue":"yyyy-MM-dd HH:mm:ss.S","label":null,"options":{"widgetType":"text","validationRegex":null}},"batchId":{"widgetType":"text","name":"batchId","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}},"fileSchema":{"widgetType":"text","name":"fileSchema","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}},"fileFormat":{"widgetType":"dropdown","name":"fileFormat","defaultValue":"ORC","label":null,"options":{"widgetType":"dropdown","choices":["PARQUET","CSV","ORC"]}}},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823481,"submitTime":1622642822488,"finishTime":1622642823519,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Create Widgets","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"ed6e0f89-10e2-41e9-9f4b-c34c3464b8a5"},{"version":"CommandV1","origId":2986605948003656,"guid":"c19dc83d-d987-47fe-ba47-7967bedebab4","subtype":"command","commandType":"auto","position":5.0,"command":"# Get widget values\npFilePath = dbutils.widgets.get(\"filePath\")\npDeltaSchemaName = dbutils.widgets.get(\"SchemaName\").strip().lower()\npDeltaSchemaSilverName = pDeltaSchemaName + 'Silver'\npDeltaObjectName = dbutils.widgets.get(\"ObjectName\").strip().lower()\npCandidateKey = dbutils.widgets.get(\"CandidateKey\").replace(\" \", \"\")\npDelimiter = dbutils.widgets.get(\"fileColDelimiter\").strip()\npQuoteChar = dbutils.widgets.get(\"fileQuoteCharacter\").strip()\npExcludedColumns = dbutils.widgets.get(\"excludedColumns\")\npFileSchema = dbutils.widgets.get(\"fileSchema\").replace(\"; \", \";\")\npFileType = dbutils.widgets.get(\"fileType\")\npFileFormat = dbutils.widgets.get(\"fileFormat\").strip().upper()\npFileHeader = dbutils.widgets.get(\"fileHeader\")\npFileHeader = pFileHeader.lower()\npDateFormat = dbutils.widgets.get(\"dateFormat\")\npBatchId = dbutils.widgets.get(\"batchId\")\npPartitionKeys = dbutils.widgets.get(\"PartitionKey\").strip().lower()\nbatchTime = datetime.utcnow().isoformat() # We need a static time for multi-query operations\nbatchTimeStr = '\\''+str(batchTime)+'Z\\''\n\npartition_keys = []\nif pPartitionKeys:\n    partition_keys = pPartitionKeys.replace(\" \",\"\").split(\",\")\n\npPreprocessingParams = dbutils.widgets.get(\"preprocessingParams\")\nif pPreprocessingParams:\n    preProcessingParams_dict = ast.literal_eval(pPreprocessingParams)\n    pPreprocessingNotebook = preProcessingParams_dict[\"Notebook\"] \nelse:\n    preProcessingParams_dict = {}\n    pPreprocessingNotebook = \"\"\n    \n# If delimiter is blank, set it to the default\nif pDelimiter == \"\":\n    pDelimiter = \"\\x07\"\n\n# If quote character is blank, set it to the default\nif pQuoteChar == \"\":\n    pQuoteChar = \"\\x0e\"\n\n#Add Databricks/dbfs root mount point to file name\nargs = [\"mnt\", pFilePath]\ntheFile = slash_join(args, True)  # handles existing leading slash (or not)\n\n# If Candidate Key is passed in, clean it up the same way table column names are cleaned\ncandidate_key = [] #List of keys to be used later\nif pCandidateKey:\n    newCandidateKey = \"\"\n    ck_list = pCandidateKey.split(\",\")\n    for k in (ck_list):\n        cname = strip_string(k, \"\", False)\n        candidate_key.append(cname)\n        if newCandidateKey:\n            newCandidateKey = newCandidateKey + \",\" + cname\n        else:\n            newCandidateKey = cname\n            \n    pCandidateKey = newCandidateKey\n    \n# Get the file name from the filePath passed in\nfilename = os.path.basename(pFilePath)\n\npath = pFilePath.replace(filename, '') #strip off filename from path\nregex = re.compile('[^0-9-]') #get numeric (date) data from path\nfiledate = regex.sub('', path)\n\nbad_records_path = f'/mnt/RAW/badRecords/{pDeltaSchemaName}.{pDeltaObjectName}'\n\n# Delta path of table (used in Merge)\nmount_path = f\"/mnt/{delta_location}/{pDeltaSchemaName}/{pDeltaObjectName}\"\n\nprint(\"----- NOTEBOOK PARAMETERS -----\")\nprint(\"Input file\\t:\", pFilePath)\nprint(\"File Date:\\t:\", filedate)\nprint(\"Database Name\\t:\", pDeltaSchemaName)\nprint(\"Object Name\\t:\", pDeltaObjectName)\nprint(\"Batch ID\\t:\", pBatchId)\nprint(\"Col Delimiter\\t:\", pDelimiter)\nprint(\"Quote Character\\t:\", pQuoteChar)\nprint(\"Date Format\\t:\", pDateFormat)\nprint(\"File Type\\t:\", pFileType)\nprint(\"File Format\\t:\", pFileFormat)\nprint(\"File Header\\t:\", pFileHeader)\nprint(\"Candidate Key\\t:\", pCandidateKey)\n#print(\"Bad Record Path\\t:\", bad_records_path)\nprint(\"Partition Key\\t:\", pPartitionKeys)\nprint(\"Preprocessing Nb:\", pPreprocessingNotebook)\nprint(\"Preprocessing Params:\", preProcessingParams_dict)\nprint(\"\\n\")\nprint(\"Excluded Columns\\t:\", pExcludedColumns)\nprint(\"File Schema\\t:\", pFileSchema)\n","commandVersion":6,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\">----- NOTEBOOK PARAMETERS -----\nInput file\t: /RAW/classicbrands/byod/EcoResReleasedProductV2Staging/2021/05/17/EcoResReleasedProductV2Staging_Full_2021-05-17T13-30-55.parquet\nFile Date:\t: 220210517\nDatabase Name\t: classicbrands\nObject Name\t: ecoresreleasedproductv2staging\nBatch ID\t: 15168\nCol Delimiter\t: \u0007\nQuote Character\t: \u000E\nDate Format\t: yyyy-MM-dd HH:mm:ss.S\nFile Type\t: FULL\nFile Format\t: PARQUET\nFile Header\t: false\nCandidate Key\t: \nPartition Key\t: \nPreprocessing Nb: \nPreprocessing Params: {}\n\n\nExcluded Columns\t: \nFile Schema\t: \n</div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{"CandidateKey":"","PartitionKey":"","preprocessingParams":"","fileType":"FULL","fileColDelimiter":"\u0007","fileQuoteCharacter":"\u000E","ObjectName":"EcoResReleasedProductV2Staging","fileHeader":"FALSE","filePath":"/RAW/classicbrands/byod/EcoResReleasedProductV2Staging/2021/05/17/EcoResReleasedProductV2Staging_Full_2021-05-17T13-30-55.parquet","SchemaName":"classicbrands","excludedColumns":"","dateFormat":"yyyy-MM-dd HH:mm:ss.S","batchId":"15168","fileSchema":"","fileFormat":"PARQUET"},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823522,"submitTime":1622642822493,"finishTime":1622642823553,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Read Parameters (Widgets) and Initialize Notebook Variables","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"bc45987b-2cfe-4c48-8df6-a6562cada3e2"},{"version":"CommandV1","origId":2986605948003657,"guid":"7c3ba48e-4a2a-4602-a72c-f923f0528f6f","subtype":"command","commandType":"auto","position":6.0,"command":"# Raise error and exit if required params/logic aren't met\n\n# Check to ensure file exists, otherwise raise exception\nif not file_exists(theFile):\n    output_message = f\"ERROR: Missing file {theFile}!\"\n    success = False\n\nif pFileType == \"INCREMENTAL\" and pCandidateKey == \"\":  # We must have a Candidate key for incremental loads\n    output_message = f\"ERROR: A Candidate Key must be provided for incremental loads!\"\n    success = False\n    \nif pFilePath == \"\":\n    output_message = f\"ERROR: The filePath parameter must be provided!\"\n    success = False\n\n# If we're ingesting CSV, then we must have the schema passed in\nif pFileFormat == \"CSV\":\n    if pFileSchema == \"\":\n        output_message = f\"ERROR: A fileSchema must be provided!\"\n        success = False\n    \nif pDeltaObjectName == \"\":\n    output_message = f\"ERROR: The deltaObjectName parameter must be provided!\"\n    success = False\n\nif pDeltaSchemaName == \"\":\n    output_message = f\"ERROR: The deltaSchemaName parameter must be provided!\"\n    success = False\n\nprint(output_message)","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\">\n</div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823556,"submitTime":1622642822497,"finishTime":1622642823639,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Check for Required Parameters","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"c7d918a8-5657-4a98-96a7-ef697842625d"},{"version":"CommandV1","origId":2986605948003658,"guid":"49394ba3-1506-4d59-bd74-f9544bee857c","subtype":"command","commandType":"auto","position":7.0,"command":"#schemaStruct = None\nschemaStruct = StructType()\n\nif success and pFileSchema:\n    # Create StructType of file schema for the read\n    schema_list = pFileSchema.split(\";\")\n    columns_struct_fields = []\n\n    for s in schema_list:\n        # Get just the column name\n        cname = s.split(\" \")[0]\n        # Clean up column name\n        cname = strip_string(cname, \"\", True)\n        # Get the Spark datatype\n        spark_dtype = sql_to_pyspark_datatype(s.split(\" \")[1])\n        # Add column to list\n        columns_struct_fields.append(StructField(cname, eval(spark_dtype), True))\n    \n    # Create the Struct from column list\n    schemaStruct = StructType(columns_struct_fields)\n    for sf in schemaStruct:\n        print(sf.name,  sf.dataType, end='; ')\n","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823642,"submitTime":1622642822502,"finishTime":1622642823660,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Create Struct of source file schema AND clean up column names","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"95321326-0943-4dad-b812-382f0612db5e"},{"version":"CommandV1","origId":2986605948003659,"guid":"2fb91ea1-c3da-4954-8ab3-a180be53f3f9","subtype":"command","commandType":"auto","position":8.0,"command":"if success:\n    if pPreprocessingNotebook:\n        try:\n            print(f\"Loading and pre-processing file: {theFile}\")\n            pp_args = {}\n            pp_args[\"PreprocessingParams\"] = pPreprocessingParams\n            pp_args[\"SourceView\"] = pDeltaObjectName\n            pp_args[\"FilePath\"] = theFile\n\n            print(\"Notebook parameters:\", pp_args)\n            \n            # Run the notebook, returning the global view name\n            vw_name = run_notebook_with_retry(notebook=pPreprocessingNotebook, params=pp_args, timeout=3600, retries=0, debug=False)\n            vw_name = \"global_temp.\" + vw_name\n\n            # Create initial dataframe\n            df = spark.sql(f\"select * from {vw_name}\")\n        except Exception as E:\n            print(E)\n            output_message = f\"ERROR Loading and pre-processing file: {theFile}\"\n            print(output_message)\n            success = False\n    else:\n        try:\n            if pFileFormat == \"PARQUET\":\n                print('Loading parquet file {}'.format(theFile))\n\n                # Read the file\n                df = (spark.read\n                        .format('parquet')\n                        #.schema(schemaStruct)  #applying a schema here can result in an empty or corrupt dataframe.\n                        .option(\"mode\", \"FAILFAST\")\n                        .load(theFile)\n                    )\n\n            elif pFileFormat == \"ORC\":\n                print('Loading orc file {}'.format(theFile))\n\n                # Read the file\n                df = (spark.read\n                        .format('orc')\n                        #.schema(schemaStruct)  #applying a schema here can result in an empty or corrupt dataframe.\n                        .option(\"mode\", \"FAILFAST\")\n                        .load(theFile))\n\n                schema_cols = len(schemaStruct)\n                file_cols = len(df.columns)\n\n\n            elif pFileFormat == \"CSV\":\n                # For CSV there must be a schema passed in (earlier check throws error if not).\n                print('Loading csv file schema...')\n\n                # Read the file's schema to determine if the number of columns match the schema that was passed in\n                file_schema = (spark.read\n                        .option(\"inferSchema\", \"false\")\n                        .option(\"header\", pFileHeader)\n                        .option(\"delimiter\", pDelimiter)\n                        .option(\"quote\", pQuoteChar)\n                        .csv(theFile).schema\n                   )\n\n                schema_cols = len(schemaStruct)\n                file_cols = len(file_schema)\n\n                if schema_cols != file_cols:\n                    output_message = f\"Mismatched number of columns between source file ({file_cols}) and passed in schema ({schema_cols})!\"\n                    print(output_message)\n                    success = False\n\n                # Read the file\n                df = (spark.read\n                        .format('csv')\n                        .schema(schemaStruct)\n                        #.option(\"badRecordsPath\", bad_records_path) #Cannot have this and \"mode\" option together.\n                        .option(\"mode\", \"FAILFAST\")\n                        .option(\"inferSchema\", \"false\")\n                        .option(\"header\", pFileHeader)\n                        .option(\"delimiter\", pDelimiter)\n                        .option(\"quote\", pQuoteChar)\n                        #.option(\"multiline\", \"true\") # Until Spark 3? supports the \"linesep\" option, this greatly slows down performance with large files!\n                        .option(\"timestampFormat\", pDateFormat)\n                        .option(\"ignoreTrailingWhiteSpace\", \"true\")\n                        .option(\"ignoreLeadingWhiteSpace\", \"true\")\n                        .load(theFile)\n                    )\n\n        except Exception as E:\n            print(E)\n            output_message = f\"ERROR loading source file! {str(E)[:100].strip()}\"\n            print(output_message)\n            success = False\n        except:\n            output_message = f\"ERROR loading source file!\"\n            print(output_message)\n            success = False\n        \n            \n    if success:\n        # Override schema if one was passed in (N/A for CSV since that is already required)\n        if schemaStruct and pFileFormat != \"CSV\":\n            # Do a simple comparison of the number of columns, and raise error if they differ\n            schema_cols = len(schemaStruct)\n            file_cols = len(df.columns)\n\n            if file_cols != schema_cols:\n                output_message = f\"Mismatched number of columns between source file ({file_cols}) and passed in schema ({schema_cols})!\"\n                print(output_message)\n                success = False\n\n            if success:\n                # Override the dataframe's schema\n                i = 0\n                for c in df.dtypes:\n                    cname_org = c[0]\n                    cname_new = schemaStruct[i].name\n                    ctype_org = c[1]\n                    ctype_new = schemaStruct[i].dataType\n                    df = ( df\n                        .withColumnRenamed(cname_org, cname_new)\n                        .withColumn(cname_new, col(cname_new).cast(ctype_new))\n                         )\n                    i += 1\n        # No schema passed in, also not a CSV, clean up column names\n        elif pFileFormat != \"CSV\":\n            for c in df.columns:\n                c_new = strip_string(c, \"\", True)\n                df = df.withColumnRenamed(c, c_new)\n                    \n        # Create a view to use in SQL statements      \n        vw_name_raw = 'tmpvw_raw_' + pDeltaSchemaName + '_' + pDeltaObjectName\n\n        # Drop Duplicates\n        df = df.dropDuplicates()\n        \n        # Exclude Columns\n        excluded_columns = pExcludedColumns.split(\",\")\n        for ec in excluded_columns:\n          df = df.drop(ec)\n\n        df.createOrReplaceTempView(vw_name_raw)\n","commandVersion":12,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\">Loading parquet file /mnt/RAW/classicbrands/byod/EcoResReleasedProductV2Staging/2021/05/17/EcoResReleasedProductV2Staging_Full_2021-05-17T13-30-55.parquet\n</div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[{"name":"df","typeStr":"pyspark.sql.dataframe.DataFrame","schema":{"fields":[{"metadata":{},"name":"productsubtype","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultledgerdimensiondisplayvalue","nullable":true,"type":"string"},{"metadata":{},"name":"aretransportationmanagementprocessesenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"costchargesquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"saleschargesquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasechargesquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"pkwiucode","nullable":true,"type":"string"},{"metadata":{},"name":"definitiongroup","nullable":true,"type":"string"},{"metadata":{},"name":"executionid","nullable":true,"type":"string"},{"metadata":{},"name":"isselected","nullable":true,"type":"integer"},{"metadata":{},"name":"transferstatus","nullable":true,"type":"integer"},{"metadata":{},"name":"producttype","nullable":true,"type":"integer"},{"metadata":{},"name":"productnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productsearchname","nullable":true,"type":"string"},{"metadata":{},"name":"itemnumber","nullable":true,"type":"string"},{"metadata":{},"name":"searchname","nullable":true,"type":"string"},{"metadata":{},"name":"productgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"itemmodelgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"storagedimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"trackingdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"inventoryreservationhierarchyname","nullable":true,"type":"string"},{"metadata":{},"name":"defaultordertype","nullable":true,"type":"integer"},{"metadata":{},"name":"marginabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"revenueabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"carryingcostabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"valueabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"alternativeproductusagecondition","nullable":true,"type":"integer"},{"metadata":{},"name":"alternativeproductconfigurationid","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeproductcolorid","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeproductsizeid","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeproductstyleid","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeitemnumber","nullable":true,"type":"string"},{"metadata":{},"name":"willinventoryissueautomaticallyreportasfinished","nullable":true,"type":"integer"},{"metadata":{},"name":"batchmergedatecalculationmethod","nullable":true,"type":"integer"},{"metadata":{},"name":"batchnumbergroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"costcalculationgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"bomlevel","nullable":true,"type":"integer"},{"metadata":{},"name":"bomunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"flushingprinciple","nullable":true,"type":"integer"},{"metadata":{},"name":"willinventoryreceiptignoreflushingprinciple","nullable":true,"type":"integer"},{"metadata":{},"name":"commissionproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"costgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"isunitcostautomaticallyupdated","nullable":true,"type":"integer"},{"metadata":{},"name":"productionconsumptiondensityconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productionconsumptiondepthconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productionconsumptionwidthconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productionconsumptionheightconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"grossdepth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"grossproductheight","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"grossproductwidth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productvolume","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"netproductweight","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"buyergroupid","nullable":true,"type":"string"},{"metadata":{},"name":"isunitcostproductvariantspecific","nullable":true,"type":"integer"},{"metadata":{},"name":"purchasepricetolerancegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"vendorinvoicelinematchingpolicy","nullable":true,"type":"integer"},{"metadata":{},"name":"packingmaterialgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"bestbeforeperioddays","nullable":true,"type":"integer"},{"metadata":{},"name":"freightallocationgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"salesrebateproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"potencybaseattributevalueentryevent","nullable":true,"type":"integer"},{"metadata":{},"name":"shelfadviceperioddays","nullable":true,"type":"integer"},{"metadata":{},"name":"shelflifeperioddays","nullable":true,"type":"integer"},{"metadata":{},"name":"potencybaseattibutetargetvalue","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"approvedvendorcheckmethod","nullable":true,"type":"integer"},{"metadata":{},"name":"isphantom","nullable":true,"type":"integer"},{"metadata":{},"name":"planningformulaitemnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productiontype","nullable":true,"type":"integer"},{"metadata":{},"name":"yieldpercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"primaryvendoraccountnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productiongroupid","nullable":true,"type":"string"},{"metadata":{},"name":"productionpoolid","nullable":true,"type":"string"},{"metadata":{},"name":"projectcategoryid","nullable":true,"type":"string"},{"metadata":{},"name":"necessaryproductionworkingtimeschedulingpropertyid","nullable":true,"type":"string"},{"metadata":{},"name":"ispurchasepriceautomaticallyupdated","nullable":true,"type":"integer"},{"metadata":{},"name":"productcoveragegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"salespricecalculationmodel","nullable":true,"type":"integer"},{"metadata":{},"name":"basesalespricesource","nullable":true,"type":"integer"},{"metadata":{},"name":"salespricecalculationcontributionratio","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salespricecalculationchargespercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"constantscrapquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"variablescrappercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"serialnumbergroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"shippingandreceivingsortordercode","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultproductconfigurationid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductcolorid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductsizeid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductstyleid","nullable":true,"type":"string"},{"metadata":{},"name":"tareproductweight","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"packingdutyquantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"arrivalhandlingtime","nullable":true,"type":"integer"},{"metadata":{},"name":"warehousefiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"warehousefiltergroup","nullable":true,"type":"string"},{"metadata":{},"name":"maximumpickquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"packageclassid","nullable":true,"type":"string"},{"metadata":{},"name":"packsizecategoryid","nullable":true,"type":"string"},{"metadata":{},"name":"physicaldimensiongroupid","nullable":true,"type":"string"},{"metadata":{},"name":"willworkcenterpickingallownegativeinventory","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultreceivingquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"warehousemobiledevicedescriptionline1","nullable":true,"type":"string"},{"metadata":{},"name":"warehousemobiledevicedescriptionline2","nullable":true,"type":"string"},{"metadata":{},"name":"rawmaterialpickingprinciple","nullable":true,"type":"integer"},{"metadata":{},"name":"unitconversionsequencegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"catchweightunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"maximumcatchweightquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"minimumcatchweightquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"issalespriceadjustmentallowed","nullable":true,"type":"integer"},{"metadata":{},"name":"continuityeventduration","nullable":true,"type":"integer"},{"metadata":{},"name":"continuityscheduleid","nullable":true,"type":"string"},{"metadata":{},"name":"isrestrictedforcoupons","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultdirectdeliverywarehouse","nullable":true,"type":"string"},{"metadata":{},"name":"isdelivereddirectly","nullable":true,"type":"integer"},{"metadata":{},"name":"isexemptfromautomaticnotificationandcancellation","nullable":true,"type":"integer"},{"metadata":{},"name":"isinstallmenteligible","nullable":true,"type":"integer"},{"metadata":{},"name":"willpickingworkbenchapplyboxinglogic","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaserebateproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"sellenddate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"sellstartdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"isshipaloneenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"shipstartdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"barcodesetupid","nullable":true,"type":"string"},{"metadata":{},"name":"comparisonpricebaseunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"isscaleproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"isposregistrationblocked","nullable":true,"type":"integer"},{"metadata":{},"name":"posregistrationblockeddate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"posregistrationplannedblockeddate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"posregistrationactivationdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"keyinpricerequirementsatposregister","nullable":true,"type":"integer"},{"metadata":{},"name":"keyinquantityrequirementsatposregister","nullable":true,"type":"integer"},{"metadata":{},"name":"mustkeyincommentatposregister","nullable":true,"type":"integer"},{"metadata":{},"name":"iszeropriceposregistrationallowed","nullable":true,"type":"integer"},{"metadata":{},"name":"isdiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"isposregistrationquantitynegative","nullable":true,"type":"integer"},{"metadata":{},"name":"productlifecycleseasoncode","nullable":true,"type":"string"},{"metadata":{},"name":"productlifecyclevalidfromdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"productlifecyclevalidtodate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"isvariantshelflabelsprintingenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"transferorderoverdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"transferorderunderdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"unitcost","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"unitcostdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"unitcostquantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"isunitcostincludingcharges","nullable":true,"type":"integer"},{"metadata":{},"name":"fixedcostcharges","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"inventoryunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"isintercompanysalesusageblocked","nullable":true,"type":"integer"},{"metadata":{},"name":"salessalestaxitemgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"willtotalsalesdiscountcalculationincludeproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"saleslinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"salesmultilinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"salessupplementaryproductproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"saleschargeproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"salesoverdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salesunderdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salespricingprecision","nullable":true,"type":"integer"},{"metadata":{},"name":"salesprice","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salespricedate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"salespricequantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"salesunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"issalespriceincludingcharges","nullable":true,"type":"integer"},{"metadata":{},"name":"fixedsalespricecharges","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasesalestaxitemgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"isintercompanypurchaseusageblocked","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaselinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"purchasemultilinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"willtotalpurchasediscountcalculationincludeproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"purchasesupplementaryproductproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"purchaseoverdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchaseunderdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasepricingprecision","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaseprice","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasepricedate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"purchasepricequantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"purchaseunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"purchasechargeproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"ispurchasepriceincludingcharges","nullable":true,"type":"integer"},{"metadata":{},"name":"fixedpurchasepricecharges","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"intrastatcommoditycode","nullable":true,"type":"string"},{"metadata":{},"name":"intrastatchargepercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"origincountryregionid","nullable":true,"type":"string"},{"metadata":{},"name":"originstateid","nullable":true,"type":"string"},{"metadata":{},"name":"approximatesalestaxpercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"itemfiscalclassificationexceptioncode","nullable":true,"type":"string"},{"metadata":{},"name":"productfiscalinformationtype","nullable":true,"type":"string"},{"metadata":{},"name":"isicmstaxappliedonservice","nullable":true,"type":"integer"},{"metadata":{},"name":"producttaxationorigin","nullable":true,"type":"integer"},{"metadata":{},"name":"itemfiscalclassificationcode","nullable":true,"type":"string"},{"metadata":{},"name":"servicefiscalinformationcode","nullable":true,"type":"string"},{"metadata":{},"name":"issaleswithholdingtaxcalculated","nullable":true,"type":"integer"},{"metadata":{},"name":"salesitemwithholdingtaxgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"ispurchasewithholdingtaxcalculated","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaseitemwithholdingtaxgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"salesgstreliefcategorycode","nullable":true,"type":"string"},{"metadata":{},"name":"purchasegstreliefcategorycode","nullable":true,"type":"string"},{"metadata":{},"name":"inventorygstreliefcategorycode","nullable":true,"type":"string"},{"metadata":{},"name":"ngpcode","nullable":true,"type":"integer"},{"metadata":{},"name":"sadgroup","nullable":true,"type":"string"},{"metadata":{},"name":"ismanualdiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"inventorycountingreasoncodepolicyname","nullable":true,"type":"string"},{"metadata":{},"name":"hsncode","nullable":true,"type":"string"},{"metadata":{},"name":"serviceaccountingcode","nullable":true,"type":"string"},{"metadata":{},"name":"exempt","nullable":true,"type":"integer"},{"metadata":{},"name":"customsimporttariffcode","nullable":true,"type":"string"},{"metadata":{},"name":"customsexporttariffcode","nullable":true,"type":"string"},{"metadata":{},"name":"satcode","nullable":true,"type":"string"},{"metadata":{},"name":"scaleindicator","nullable":true,"type":"integer"},{"metadata":{},"name":"cnpj","nullable":true,"type":"string"},{"metadata":{},"name":"taxsubstitutioncode","nullable":true,"type":"string"},{"metadata":{},"name":"partition","nullable":true,"type":"string"},{"metadata":{},"name":"updateproductvariantlifecyclestate","nullable":true,"type":"integer"},{"metadata":{},"name":"catchweightitemhandlingpolicyname","nullable":true,"type":"string"},{"metadata":{},"name":"firstproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"secondproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"thirdproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"fourthproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"firstproductfiltergroupid","nullable":true,"type":"string"},{"metadata":{},"name":"secondproductfiltergroupid","nullable":true,"type":"string"},{"metadata":{},"name":"taxratetype","nullable":true,"type":"string"},{"metadata":{},"name":"productlifecyclestateid","nullable":true,"type":"string"},{"metadata":{},"name":"nongst","nullable":true,"type":"integer"},{"metadata":{},"name":"productdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"iscatchweightproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecrevenuetype","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecbundle","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecexcludefromcarveout","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecmedianprice","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecmedianpricemaximumtolerance","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"revrecmedianpriceminimumtolerance","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"revrecrevenuerecognitionenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"servicetype","nullable":true,"type":"integer"},{"metadata":{},"name":"isretaildiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"istenderdiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantablepricerangebasetype","nullable":true,"type":"integer"},{"metadata":{},"name":"lowerwarrantablepricerangelimit","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"upperwarrantablepricerangelimit","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"warrantydurationtime","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantydurationtimeunit","nullable":true,"type":"integer"},{"metadata":{},"name":"hsdepth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"hswidth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"hsheight","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"alternativeproductversionid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductversionid","nullable":true,"type":"string"},{"metadata":{},"name":"costcalculationbomlevel","nullable":true,"type":"integer"},{"metadata":{},"name":"salesretailinventoryavailabilitybuffer","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salesretailinventoryavailabilitylevelprofile","nullable":true,"type":"string"},{"metadata":{},"name":"dataareaid","nullable":true,"type":"string"},{"metadata":{},"name":"syncstartdatetime","nullable":true,"type":"timestamp"}],"type":"struct"},"tableIdentifier":null}],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823663,"submitTime":1622642822506,"finishTime":1622642837915,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Load Incoming File into Dataframe","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"22101cdd-f64b-414c-8ebf-828f3aacd64b"},{"version":"CommandV1","origId":2986605948003660,"guid":"f7206329-5946-4bc5-9fbe-c2b55c5fe926","subtype":"command","commandType":"auto","position":9.0,"command":"if success:\n    try:\n        sdf = spark.table(vw_name_raw) # 11s \n\n        # FIRST: Add sha2 hash over all source columns for SCD Type 2\n        # https://stackoverflow.com/questions/52292180/pyspark-generate-row-hash-of-specific-columns-and-add-it-as-a-new-column\n        # Note:  concat_ws removes NULLs during the process, so it should work as expected\n        sdf = sdf.withColumn(\"dl_rowhash\", sha2(concat_ws(\"||\", *sdf.columns), 256))\n\n        # Generate a key for partitioning based on first 1 char of rowhash (should be max of 16)\n        # Note: We also partition by Is_Current\n        sdf = sdf.withColumn('dl_partitionkey', (substring(\"dl_rowhash\",0,1)))\n            \n        # Add an Active flag, setting it to True\n        sdf = sdf.withColumn(\"dl_iscurrent\", lit(True)) #Add column indicating the pDeltaObjectName #2.24m\n\n        # Add Record dates for SCD Type 2\n        sdf = sdf.withColumn('dl_recordstartdateutc', lit(batchTime)) #add timestamp when processed\n        sdf = sdf.withColumn(\"dl_recordenddateutc\", to_timestamp(lit(None), \"yyyy-MM-dd HH:mm:ss\")) #Add column indicating the type 2 end date\n\n        # Add the source file name - traceability\n        # Don: It might be better to have a delta table with all the file name and paths?  This is a lot of extra data.\n        sdf = sdf.withColumn(\"dl_sourcefilename\", lit(filename)) #Add column indicating the source file name\n\n        # If a schema wasn't passed in (and cleaned), clean up column names now\n        if not schemaStruct:\n            # Clean up column names\n            # strip_string function in sharedUtils; clean up all existing column names; convert to lower case\n            new_column_name_list = list(map(lambda x: strip_string(x, \"\", True), sdf.columns))\n\n            # Assign cleaned column names to existing dataframe\n            sdf = sdf.toDF(*new_column_name_list)\n\n        # Make sure string column data is trimmed (some datasets use \"char\" datatypes)\n        for c, t in sdf.dtypes:\n            if t == \"string\":\n                sdf = sdf.withColumn(c, ltrim(rtrim(sdf[c])))\n\n        ## No apparent benefit to repartition upon read\n        # sdf = sdf.repartition(*partition_by)\n        \n        # Create a temp table from Dataframe to use in MERGE statement into Delta Lake\n        vw_name_enriched = 'tmpvw_enriched_' + pDeltaSchemaName + '_' + pDeltaObjectName\n        sdf.persist(StorageLevel.MEMORY_AND_DISK)\n        sdf.createOrReplaceTempView(vw_name_enriched)\n        print(vw_name_enriched)\n    except Exception as E:\n        output_message = f\"ERROR adding metadata columns! {E:100}\"\n        success = False\n    except:\n        output_message = f\"ERROR adding metadata columns!\"\n        success = False\n","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\">tmpvw_enriched_classicbrands_ecoresreleasedproductv2staging\n</div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[{"name":"sdf","typeStr":"pyspark.sql.dataframe.DataFrame","schema":{"fields":[{"metadata":{},"name":"productsubtype","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultledgerdimensiondisplayvalue","nullable":true,"type":"string"},{"metadata":{},"name":"aretransportationmanagementprocessesenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"costchargesquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"saleschargesquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasechargesquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"pkwiucode","nullable":true,"type":"string"},{"metadata":{},"name":"definitiongroup","nullable":true,"type":"string"},{"metadata":{},"name":"executionid","nullable":true,"type":"string"},{"metadata":{},"name":"isselected","nullable":true,"type":"integer"},{"metadata":{},"name":"transferstatus","nullable":true,"type":"integer"},{"metadata":{},"name":"producttype","nullable":true,"type":"integer"},{"metadata":{},"name":"productnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productsearchname","nullable":true,"type":"string"},{"metadata":{},"name":"itemnumber","nullable":true,"type":"string"},{"metadata":{},"name":"searchname","nullable":true,"type":"string"},{"metadata":{},"name":"productgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"itemmodelgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"storagedimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"trackingdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"inventoryreservationhierarchyname","nullable":true,"type":"string"},{"metadata":{},"name":"defaultordertype","nullable":true,"type":"integer"},{"metadata":{},"name":"marginabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"revenueabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"carryingcostabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"valueabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"alternativeproductusagecondition","nullable":true,"type":"integer"},{"metadata":{},"name":"alternativeproductconfigurationid","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeproductcolorid","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeproductsizeid","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeproductstyleid","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeitemnumber","nullable":true,"type":"string"},{"metadata":{},"name":"willinventoryissueautomaticallyreportasfinished","nullable":true,"type":"integer"},{"metadata":{},"name":"batchmergedatecalculationmethod","nullable":true,"type":"integer"},{"metadata":{},"name":"batchnumbergroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"costcalculationgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"bomlevel","nullable":true,"type":"integer"},{"metadata":{},"name":"bomunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"flushingprinciple","nullable":true,"type":"integer"},{"metadata":{},"name":"willinventoryreceiptignoreflushingprinciple","nullable":true,"type":"integer"},{"metadata":{},"name":"commissionproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"costgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"isunitcostautomaticallyupdated","nullable":true,"type":"integer"},{"metadata":{},"name":"productionconsumptiondensityconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productionconsumptiondepthconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productionconsumptionwidthconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productionconsumptionheightconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"grossdepth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"grossproductheight","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"grossproductwidth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productvolume","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"netproductweight","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"buyergroupid","nullable":true,"type":"string"},{"metadata":{},"name":"isunitcostproductvariantspecific","nullable":true,"type":"integer"},{"metadata":{},"name":"purchasepricetolerancegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"vendorinvoicelinematchingpolicy","nullable":true,"type":"integer"},{"metadata":{},"name":"packingmaterialgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"bestbeforeperioddays","nullable":true,"type":"integer"},{"metadata":{},"name":"freightallocationgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"salesrebateproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"potencybaseattributevalueentryevent","nullable":true,"type":"integer"},{"metadata":{},"name":"shelfadviceperioddays","nullable":true,"type":"integer"},{"metadata":{},"name":"shelflifeperioddays","nullable":true,"type":"integer"},{"metadata":{},"name":"potencybaseattibutetargetvalue","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"approvedvendorcheckmethod","nullable":true,"type":"integer"},{"metadata":{},"name":"isphantom","nullable":true,"type":"integer"},{"metadata":{},"name":"planningformulaitemnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productiontype","nullable":true,"type":"integer"},{"metadata":{},"name":"yieldpercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"primaryvendoraccountnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productiongroupid","nullable":true,"type":"string"},{"metadata":{},"name":"productionpoolid","nullable":true,"type":"string"},{"metadata":{},"name":"projectcategoryid","nullable":true,"type":"string"},{"metadata":{},"name":"necessaryproductionworkingtimeschedulingpropertyid","nullable":true,"type":"string"},{"metadata":{},"name":"ispurchasepriceautomaticallyupdated","nullable":true,"type":"integer"},{"metadata":{},"name":"productcoveragegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"salespricecalculationmodel","nullable":true,"type":"integer"},{"metadata":{},"name":"basesalespricesource","nullable":true,"type":"integer"},{"metadata":{},"name":"salespricecalculationcontributionratio","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salespricecalculationchargespercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"constantscrapquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"variablescrappercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"serialnumbergroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"shippingandreceivingsortordercode","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultproductconfigurationid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductcolorid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductsizeid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductstyleid","nullable":true,"type":"string"},{"metadata":{},"name":"tareproductweight","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"packingdutyquantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"arrivalhandlingtime","nullable":true,"type":"integer"},{"metadata":{},"name":"warehousefiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"warehousefiltergroup","nullable":true,"type":"string"},{"metadata":{},"name":"maximumpickquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"packageclassid","nullable":true,"type":"string"},{"metadata":{},"name":"packsizecategoryid","nullable":true,"type":"string"},{"metadata":{},"name":"physicaldimensiongroupid","nullable":true,"type":"string"},{"metadata":{},"name":"willworkcenterpickingallownegativeinventory","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultreceivingquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"warehousemobiledevicedescriptionline1","nullable":true,"type":"string"},{"metadata":{},"name":"warehousemobiledevicedescriptionline2","nullable":true,"type":"string"},{"metadata":{},"name":"rawmaterialpickingprinciple","nullable":true,"type":"integer"},{"metadata":{},"name":"unitconversionsequencegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"catchweightunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"maximumcatchweightquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"minimumcatchweightquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"issalespriceadjustmentallowed","nullable":true,"type":"integer"},{"metadata":{},"name":"continuityeventduration","nullable":true,"type":"integer"},{"metadata":{},"name":"continuityscheduleid","nullable":true,"type":"string"},{"metadata":{},"name":"isrestrictedforcoupons","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultdirectdeliverywarehouse","nullable":true,"type":"string"},{"metadata":{},"name":"isdelivereddirectly","nullable":true,"type":"integer"},{"metadata":{},"name":"isexemptfromautomaticnotificationandcancellation","nullable":true,"type":"integer"},{"metadata":{},"name":"isinstallmenteligible","nullable":true,"type":"integer"},{"metadata":{},"name":"willpickingworkbenchapplyboxinglogic","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaserebateproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"sellenddate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"sellstartdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"isshipaloneenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"shipstartdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"barcodesetupid","nullable":true,"type":"string"},{"metadata":{},"name":"comparisonpricebaseunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"isscaleproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"isposregistrationblocked","nullable":true,"type":"integer"},{"metadata":{},"name":"posregistrationblockeddate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"posregistrationplannedblockeddate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"posregistrationactivationdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"keyinpricerequirementsatposregister","nullable":true,"type":"integer"},{"metadata":{},"name":"keyinquantityrequirementsatposregister","nullable":true,"type":"integer"},{"metadata":{},"name":"mustkeyincommentatposregister","nullable":true,"type":"integer"},{"metadata":{},"name":"iszeropriceposregistrationallowed","nullable":true,"type":"integer"},{"metadata":{},"name":"isdiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"isposregistrationquantitynegative","nullable":true,"type":"integer"},{"metadata":{},"name":"productlifecycleseasoncode","nullable":true,"type":"string"},{"metadata":{},"name":"productlifecyclevalidfromdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"productlifecyclevalidtodate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"isvariantshelflabelsprintingenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"transferorderoverdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"transferorderunderdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"unitcost","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"unitcostdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"unitcostquantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"isunitcostincludingcharges","nullable":true,"type":"integer"},{"metadata":{},"name":"fixedcostcharges","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"inventoryunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"isintercompanysalesusageblocked","nullable":true,"type":"integer"},{"metadata":{},"name":"salessalestaxitemgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"willtotalsalesdiscountcalculationincludeproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"saleslinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"salesmultilinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"salessupplementaryproductproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"saleschargeproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"salesoverdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salesunderdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salespricingprecision","nullable":true,"type":"integer"},{"metadata":{},"name":"salesprice","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salespricedate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"salespricequantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"salesunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"issalespriceincludingcharges","nullable":true,"type":"integer"},{"metadata":{},"name":"fixedsalespricecharges","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasesalestaxitemgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"isintercompanypurchaseusageblocked","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaselinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"purchasemultilinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"willtotalpurchasediscountcalculationincludeproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"purchasesupplementaryproductproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"purchaseoverdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchaseunderdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasepricingprecision","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaseprice","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasepricedate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"purchasepricequantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"purchaseunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"purchasechargeproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"ispurchasepriceincludingcharges","nullable":true,"type":"integer"},{"metadata":{},"name":"fixedpurchasepricecharges","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"intrastatcommoditycode","nullable":true,"type":"string"},{"metadata":{},"name":"intrastatchargepercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"origincountryregionid","nullable":true,"type":"string"},{"metadata":{},"name":"originstateid","nullable":true,"type":"string"},{"metadata":{},"name":"approximatesalestaxpercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"itemfiscalclassificationexceptioncode","nullable":true,"type":"string"},{"metadata":{},"name":"productfiscalinformationtype","nullable":true,"type":"string"},{"metadata":{},"name":"isicmstaxappliedonservice","nullable":true,"type":"integer"},{"metadata":{},"name":"producttaxationorigin","nullable":true,"type":"integer"},{"metadata":{},"name":"itemfiscalclassificationcode","nullable":true,"type":"string"},{"metadata":{},"name":"servicefiscalinformationcode","nullable":true,"type":"string"},{"metadata":{},"name":"issaleswithholdingtaxcalculated","nullable":true,"type":"integer"},{"metadata":{},"name":"salesitemwithholdingtaxgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"ispurchasewithholdingtaxcalculated","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaseitemwithholdingtaxgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"salesgstreliefcategorycode","nullable":true,"type":"string"},{"metadata":{},"name":"purchasegstreliefcategorycode","nullable":true,"type":"string"},{"metadata":{},"name":"inventorygstreliefcategorycode","nullable":true,"type":"string"},{"metadata":{},"name":"ngpcode","nullable":true,"type":"integer"},{"metadata":{},"name":"sadgroup","nullable":true,"type":"string"},{"metadata":{},"name":"ismanualdiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"inventorycountingreasoncodepolicyname","nullable":true,"type":"string"},{"metadata":{},"name":"hsncode","nullable":true,"type":"string"},{"metadata":{},"name":"serviceaccountingcode","nullable":true,"type":"string"},{"metadata":{},"name":"exempt","nullable":true,"type":"integer"},{"metadata":{},"name":"customsimporttariffcode","nullable":true,"type":"string"},{"metadata":{},"name":"customsexporttariffcode","nullable":true,"type":"string"},{"metadata":{},"name":"satcode","nullable":true,"type":"string"},{"metadata":{},"name":"scaleindicator","nullable":true,"type":"integer"},{"metadata":{},"name":"cnpj","nullable":true,"type":"string"},{"metadata":{},"name":"taxsubstitutioncode","nullable":true,"type":"string"},{"metadata":{},"name":"partition","nullable":true,"type":"string"},{"metadata":{},"name":"updateproductvariantlifecyclestate","nullable":true,"type":"integer"},{"metadata":{},"name":"catchweightitemhandlingpolicyname","nullable":true,"type":"string"},{"metadata":{},"name":"firstproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"secondproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"thirdproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"fourthproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"firstproductfiltergroupid","nullable":true,"type":"string"},{"metadata":{},"name":"secondproductfiltergroupid","nullable":true,"type":"string"},{"metadata":{},"name":"taxratetype","nullable":true,"type":"string"},{"metadata":{},"name":"productlifecyclestateid","nullable":true,"type":"string"},{"metadata":{},"name":"nongst","nullable":true,"type":"integer"},{"metadata":{},"name":"productdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"iscatchweightproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecrevenuetype","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecbundle","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecexcludefromcarveout","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecmedianprice","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecmedianpricemaximumtolerance","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"revrecmedianpriceminimumtolerance","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"revrecrevenuerecognitionenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"servicetype","nullable":true,"type":"integer"},{"metadata":{},"name":"isretaildiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"istenderdiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantablepricerangebasetype","nullable":true,"type":"integer"},{"metadata":{},"name":"lowerwarrantablepricerangelimit","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"upperwarrantablepricerangelimit","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"warrantydurationtime","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantydurationtimeunit","nullable":true,"type":"integer"},{"metadata":{},"name":"hsdepth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"hswidth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"hsheight","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"alternativeproductversionid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductversionid","nullable":true,"type":"string"},{"metadata":{},"name":"costcalculationbomlevel","nullable":true,"type":"integer"},{"metadata":{},"name":"salesretailinventoryavailabilitybuffer","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salesretailinventoryavailabilitylevelprofile","nullable":true,"type":"string"},{"metadata":{},"name":"dataareaid","nullable":true,"type":"string"},{"metadata":{},"name":"syncstartdatetime","nullable":true,"type":"timestamp"},{"metadata":{},"name":"dl_rowhash","nullable":true,"type":"string"},{"metadata":{},"name":"dl_partitionkey","nullable":true,"type":"string"},{"metadata":{},"name":"dl_iscurrent","nullable":false,"type":"boolean"},{"metadata":{},"name":"dl_recordstartdateutc","nullable":false,"type":"string"},{"metadata":{},"name":"dl_recordenddateutc","nullable":true,"type":"timestamp"},{"metadata":{},"name":"dl_sourcefilename","nullable":false,"type":"string"}],"type":"struct"},"tableIdentifier":null}],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642837921,"submitTime":1622642822510,"finishTime":1622642853348,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Add Additional metadata columns for traceability and CDC","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"2044c7b1-f103-4302-8adb-8833424ef0c0"},{"version":"CommandV1","origId":2986605948003661,"guid":"183e6093-b62b-41c4-ba2a-cf4ea7520625","subtype":"command","commandType":"auto","position":10.0,"command":"if success:\n    candidate_key_list = [item.lower() for item in candidate_key]\n    if not all(item in sdf.columns for item in candidate_key_list):\n        output_message = f\"There are columns in the Candidate Key that are not in the datafile!\"\n        print(output_message)\n        success = False","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642853352,"submitTime":1622642822517,"finishTime":1622642853373,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Verify that columns in Candidate Key actually exist in new data file","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"4e03b7ea-1635-4952-b0df-644a7a591db4"},{"version":"CommandV1","origId":2986605948003662,"guid":"fbaa96ea-2d2a-4a92-9fa7-541b9a686d13","subtype":"command","commandType":"auto","position":11.0,"command":"if success:\n    db_exists = check_if_db_exists(pDeltaSchemaName, True, delta_location)","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642853376,"submitTime":1622642822521,"finishTime":1622642853781,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Check/Create Database","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"32b7314e-c2b4-4223-8c85-942c81dcd0d1"},{"version":"CommandV1","origId":2986605948003663,"guid":"a37213f2-b65f-42cd-ac95-50d7573cfdb0","subtype":"command","commandType":"auto","position":11.5,"command":"if success:\n    db_exists = check_if_db_exists(pDeltaSchemaSilverName, True, delta_location)","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642853784,"submitTime":1622642822525,"finishTime":1622642853903,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":null,"yColumns":null,"pivotColumns":null,"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Check/Create Silver Database","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"9777e7e9-6f2c-4549-a0f8-99f63cd5da22"},{"version":"CommandV1","origId":2986605948003664,"guid":"078cd3c3-bc97-435d-ae30-31e48da55e5b","subtype":"command","commandType":"auto","position":12.0,"command":"#######################################################\n# This is for the INITIAL load of this table only!\n# This will write an empty table, per requirements. \n# Subsequent checks for empty datasets must be performed \n# after this step, as appropriate.\n#######################################################\n\nif success:\n    # Make sure table exists (IN DELTA LAKE)\n    table_exists = check_if_table_exists(pDeltaObjectName, pDeltaSchemaName)\n\n    if len(partition_keys) == 0:\n        partition_by = [\"dl_partitionkey\",\"dl_iscurrent\"]\n    else:\n        partition_by = partition_keys\n    \n    # Create table if it doesn't exist\n    if not table_exists:\n        print('Creating table for Dataset: {}.{}'.format(pDeltaSchemaName, pDeltaObjectName))\n        print(\"Partition Keys:\", partition_by)\n        sdf.write \\\n            .format(\"delta\") \\\n            .partitionBy(partition_by) \\\n            .saveAsTable(f\"{pDeltaSchemaName}.{pDeltaObjectName}\")\n","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642853908,"submitTime":1622642822529,"finishTime":1622642854021,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Check/Create Table","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"cc50edae-9f9b-4386-9a91-123a30d4a9de"},{"version":"CommandV1","origId":2986605948003665,"guid":"21960ef0-2185-468d-86be-6e3e97dc4e4d","subtype":"command","commandType":"auto","position":13.0,"command":"###\n# https://docs.databricks.com/delta/optimizations/file-mgmt.html#enhanced-checkpoints\n###\n\nif success:\n    try:\n        query = f\"\"\"ALTER TABLE {pDeltaSchemaName}.{pDeltaObjectName} SET TBLPROPERTIES\n        (\n         'delta.checkpoint.writeStatsAsStruct' = 'true',\n         'delta.checkpoint.writeStatsAsJson' = 'false'\n        )\n        \"\"\"\n        print(query)\n        spark.sql(query)\n    except Exception as E:\n        output_message = f\"ERROR setting table properties! {E:100}\"\n        success = False\n    except:\n        output_message = f\"ERROR setting table properties!\"\n        success = False","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\">ALTER TABLE classicbrands.ecoresreleasedproductv2staging SET TBLPROPERTIES\n        (\n         &#39;delta.checkpoint.writeStatsAsStruct&#39; = &#39;true&#39;,\n         &#39;delta.checkpoint.writeStatsAsJson&#39; = &#39;false&#39;\n        )\n        \n</div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642854024,"submitTime":1622642822533,"finishTime":1622642855342,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Enable Enhanced Checkpoint Performance in Spark 3/DBR 7.3+","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"5c940177-e075-4f58-b8ee-19fa29bb2964"},{"version":"CommandV1","origId":2986605948003666,"guid":"126809eb-658a-407c-9b66-b9b58c70c130","subtype":"command","commandType":"auto","position":14.0,"command":"%md\n## MERGE\nThere are a couple execution paths here.\n\n+ **IF we just loaded the table the first time:**\n - *No action is needed here.*\n \n+ **IF we are FULL loading:**\n - We have the candidate key -> *We ignore it.* It's a waste of processing to use the candidate key since we are partitioned by the hash.\n - We don't have the candidate key -> We can simply invalidate based on the existance or non-existance of the row hash. \n\n+ **IF we are INCREMENTAL loading:**\n - We need to use the delta candidate key.  No other option.\n - Failure to specify a candidate key should have generated an error at the top of this notebook.\n\n-----------------------\n#### Incremental Merge Block\n*This first block is for incremental loading.*","commandVersion":1,"state":"finished","results":null,"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b553ff43-75af-4ed8-b174-df0d4093491c"},{"version":"CommandV1","origId":2986605948003667,"guid":"5aacec24-b1e6-4b10-b653-f6ab8ccf3c13","subtype":"command","commandType":"auto","position":15.0,"command":"if table_exists and success and pFileType == \"INCREMENTAL\":\n    try:\n        print(f\"{datetime.now()}: Started incremental merge of new dataset into into base table...\")\n\n        # Concatenated merge keys for both aliases (updates and target) for the MERGE statement\n        merge_key = \"updates.\" + \", updates.\".join(candidate_key)\n        updates_merge_key = \"concat_ws('|', \" + merge_key + \")\"\n        merge_key = \"baseTable.\" + \", baseTable.\".join(candidate_key)\n        target_merge_key = \"concat_ws('|', \" + merge_key + \")\"\n\n        # Create the list of columns to join on based on the candidate keys\n        join_condition = [col(\"updates.\" + f) == col(\"basetable.\" + s) for (f, s) in zip(candidate_key, candidate_key)]\n        # Add dl_rowhash and dl_iscurrent to the JOIN conditions\n        join_condition.append(col(\"basetable.dl_iscurrent\") == \"True\")\n        join_condition.append(col(\"updates.dl_rowhash\") != col(\"basetable.dl_rowhash\"))\n        \n        # Refresh Delta baseTable object\n        baseTable = DeltaTable.forPath(spark, mount_path)\n        basetable_df = baseTable.toDF()\n\n        # Create a dataframe of new rows to be added\n        # These are rows where there are matches on candidate keys, but the rowhash is different,\n        # so there will be an update on the old row, and these will be inserted.\n        newInserts = sdf.alias(\"updates\").join(\n              basetable_df.alias(\"basetable\"),\n                join_condition\n                , \"inner\")\n\n        # Stage the update by unioning two sets of rows\n        # 1. Rows that will be inserted in the `whenNotMatched` clause\n        # 2. Rows that will either UPDATE the current records or INSERT new records \n        stagedUpdates = (newInserts\n                         .selectExpr(\"NULL as dl_mergekey\", \"updates.*\")  # Rows for 1\n                         .union(sdf.alias(\"updates\").selectExpr(f\"{updates_merge_key} as dl_mergekey\", \"*\"))) # Rows for 2\n\n        # MERGE dataframe into table, using candidate key(s)\n        baseTable.alias(\"basetable\").merge(\n              source = stagedUpdates.alias(\"staged_updates\"), \n              condition = expr(f\"staged_updates.dl_mergekey = {target_merge_key} AND basetable.dl_iscurrent = true\")\n          ).whenMatchedUpdate(\n              condition = expr(\"basetable.dl_rowhash <> staged_updates.dl_rowhash\"), \n              set = {\"basetable.dl_iscurrent\": \"false\" ,\n                     \"basetable.dl_recordenddateutc\": f\"'{BATCH_TIME_STR}'\"}\n          ).whenNotMatchedInsertAll(\n          ).execute()\n\n        print(f\"{datetime.now()}: Completed incremental merge of new dataset into into base table...\")\n    except Exception as E:\n        output_message = f\"ERROR merging incremental file! {str(E):100}\"\n        print(output_message)\n        success = False\n    except:\n        output_message = f\"ERROR merging incremental file!\"\n        print(output_message)\n        success = False","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642855346,"submitTime":1622642822553,"finishTime":1622642855374,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Incremental Merge","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"8f77b394-06a1-4f1f-ac82-fa254d421728"},{"version":"CommandV1","origId":2986605948003668,"guid":"0cb68143-3935-49a1-b7f3-ab367e35c138","subtype":"command","commandType":"auto","position":16.0,"command":"%md\n#### Full Merge Block\n\nFor full loads only.  Rowhash is used as key for CDC.\n1. Update/Expire rows in table as \"non current\" if they don't exist in the update (new) dataset\n2. Add new rows from update dataset that don't exist in table (where target rowhash exists and target iscurrent)","commandVersion":1,"state":"finished","results":null,"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"a07e6b5e-fdff-4778-8a78-6b12586af5d6"},{"version":"CommandV1","origId":2986605948003669,"guid":"03479a7d-0c63-40c8-a2bd-002b8bfccefd","subtype":"command","commandType":"auto","position":17.0,"command":"if table_exists and success and pFileType == \"FULL\":\n    try:\n        print(f\"{datetime.now()}: Started full load merge of new dataset into into base table...\")\n\n        # Create dataframe from Delta baseTable object\n        baseTable = DeltaTable.forPath(spark, mount_path)\n        basetable_df = baseTable.toDF()\n\n        ############################################################################\n        # 1. Update/expire current rows in base table.  These are\n        #    rows where there is no match on row hash in incoming dataset.\n        #    Since this is a FULL (not incremental) dataset, \n        #    Update all rows in table where the rowhash is not in the new dataset\n        ############################################################################\n        print(f\"{datetime.now()}: Updating/expiring current rows in base table...\")\n\n        # Create a dataframe of row hashes in table that are not in new file \n        # Note:  Only need records where they are not already inactive.\n        updateDF = (basetable_df.alias(\"basetable\")\n                    .join(sdf.select(\"dl_rowhash\",\"dl_partitionkey\").alias(\"updates\"),\n                            (col(\"basetable.dl_iscurrent\") == \"True\") \n                          & (col(\"updates.dl_partitionkey\") == col(\"basetable.dl_partitionkey\")) # Added for partition elimination\n                          & (col(\"updates.dl_rowhash\") == col(\"basetable.dl_rowhash\"))\n                          , \"left_outer\")\n                    .filter(\"updates.dl_rowhash IS NULL AND basetable.dl_iscurrent == 'True'\"))\n\n        # UPDATE (EXPIRE) all records in table that do not have a corresponding row hash in new file\n        baseTable.alias(\"basetable\").merge(\n              source = updateDF.select(\"basetable.*\").alias(\"expired\"), \n             condition = expr(f\"\"\"expired.dl_partitionkey = basetable.dl_partitionkey \n                                  AND basetable.dl_iscurrent = true  \n                                  AND expired.dl_rowhash = basetable.dl_rowhash\"\"\")\n          ).whenMatchedUpdate(\n              set = {\"basetable.dl_iscurrent\": \"false\" ,\n                     \"basetable.dl_recordenddateutc\": f\"'{BATCH_TIME_STR}'\"}\n          ).execute()\n\n\n        ############################################################################\n        # 2. Insert new records from updates dataset where there \n        #    is no match on the row hash (and is already current)\n        ############################################################################\n        print(f\"{datetime.now()}: Adding new (or updated) rows to base table...\")\n\n        # FOR SCHEMA EVOLUTION:\n        #   Create a list of columns to use in MERGE statement\n        #   that has all columns from the base table and incoming dataset\n        diffcols = list(set(sdf.columns)-set(basetable_df.columns)) # Diff incoming dataset with basetable\n        added_cols = basetable_df.columns + diffcols # Add any new columns from dataset to basetable columns\n        diffcols = list(set(basetable_df.columns)-set(sdf.columns)) # Diff incoming dataset with basetable\n        all_columns = [\"updates.\" + col for col in added_cols if not col in diffcols ] # all columns, minus any dropped columns, aliased for MERGE\n\n        # Refresh dataframe (due to previous merge)\n        basetable_df = baseTable.toDF()\n        \n        # Create a dataframe of records in new file that do not have\n        # a corresponding row hash already in table, that is active\n        insertDF = (sdf.alias(\"updates\")\n                        .join(basetable_df.alias(\"basetable\"),\n                            (col(\"basetable.dl_iscurrent\") == \"True\") #MUST HAVE THIS INCLUDED FOR THIS TO WORK PROPERLY\n                           &  (col(\"updates.dl_partitionkey\") == col(\"basetable.dl_partitionkey\")) # Added for partition elimination\n                           &  (col(\"updates.dl_rowhash\") == col(\"basetable.dl_rowhash\")) #Note, there can be existing records with this hash\n                           , \"left_outer\")\n                        .filter(\"basetable.dl_rowhash IS NULL\"))\n    \n        # INSERT new records into table, using row hash for key\n        baseTable.alias(\"target\").merge(\n              #source = sdf.select(all_columns).alias(\"updates\"), \n              source = insertDF.select(all_columns).alias(\"updates\"), \n             condition = expr(f\"\"\"target.dl_partitionkey == updates.dl_partitionkey \n                                  AND updates.dl_rowhash == target.dl_rowhash \n                                  AND target.dl_iscurrent = true\"\"\") #must have all 3 conditions\n          ).whenNotMatchedInsertAll(\n          ).execute()\n\n        print(f\"{datetime.now()}: Completed full load merge of new dataset into into base table...\")\n    except Exception as E:\n        output_message = f\"ERROR merging full file! {str(E)[:100]}\"\n        print(output_message)\n        success = False\n    except:\n        output_message = f\"ERROR merging full file!\"\n        print(output_message)\n        success = False","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\">2021-06-02 14:07:35.397202: Started full load merge of new dataset into into base table...\n2021-06-02 14:07:35.531534: Updating/expiring current rows in base table...\n2021-06-02 14:07:51.511551: Adding new (or updated) rows to base table...\n2021-06-02 14:08:00.381023: Completed full load merge of new dataset into into base table...\n</div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[{"name":"updateDF","typeStr":"pyspark.sql.dataframe.DataFrame","schema":{"fields":[{"metadata":{},"name":"definitiongroup","nullable":true,"type":"string"},{"metadata":{},"name":"executionid","nullable":true,"type":"string"},{"metadata":{},"name":"isselected","nullable":true,"type":"integer"},{"metadata":{},"name":"transferstatus","nullable":true,"type":"integer"},{"metadata":{},"name":"producttype","nullable":true,"type":"integer"},{"metadata":{},"name":"productsubtype","nullable":true,"type":"integer"},{"metadata":{},"name":"productnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productname","nullable":true,"type":"string"},{"metadata":{},"name":"productsearchname","nullable":true,"type":"string"},{"metadata":{},"name":"productdescription","nullable":true,"type":"string"},{"metadata":{},"name":"iscatchweightproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"productdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"storagedimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"trackingdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"variantconfigurationtechnology","nullable":true,"type":"integer"},{"metadata":{},"name":"areidenticalconfigurationsallowed","nullable":true,"type":"integer"},{"metadata":{},"name":"isautomaticvariantgenerationenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"isproductvariantunitconversionenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"retailproductcategoryname","nullable":true,"type":"string"},{"metadata":{},"name":"productcolorgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"productsizegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"productstylegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"isproductkit","nullable":true,"type":"integer"},{"metadata":{},"name":"stcccode","nullable":true,"type":"string"},{"metadata":{},"name":"harmonizedsystemcode","nullable":true,"type":"string"},{"metadata":{},"name":"nmfccode","nullable":true,"type":"string"},{"metadata":{},"name":"partition","nullable":true,"type":"string"},{"metadata":{},"name":"productvariantnamenomenclaturename","nullable":true,"type":"string"},{"metadata":{},"name":"productvariantnumbernomenclaturename","nullable":true,"type":"string"},{"metadata":{},"name":"servicetype","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantydurationtime","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantydurationtimeunit","nullable":true,"type":"integer"},{"metadata":{},"name":"syncstartdatetime","nullable":true,"type":"timestamp"},{"metadata":{},"name":"recid","nullable":true,"type":"long"},{"metadata":{},"name":"dl_rowhash","nullable":true,"type":"string"},{"metadata":{},"name":"dl_partitionkey","nullable":true,"type":"string"},{"metadata":{},"name":"dl_iscurrent","nullable":true,"type":"boolean"},{"metadata":{},"name":"dl_recordstartdateutc","nullable":true,"type":"string"},{"metadata":{},"name":"dl_recordenddateutc","nullable":true,"type":"timestamp"},{"metadata":{},"name":"dl_sourcefilename","nullable":true,"type":"string"},{"metadata":{},"name":"dl_rowhash","nullable":true,"type":"string"},{"metadata":{},"name":"dl_partitionkey","nullable":true,"type":"string"}],"type":"struct"},"tableIdentifier":null},{"name":"basetable_df","typeStr":"pyspark.sql.dataframe.DataFrame","schema":{"fields":[{"metadata":{},"name":"definitiongroup","nullable":true,"type":"string"},{"metadata":{},"name":"executionid","nullable":true,"type":"string"},{"metadata":{},"name":"isselected","nullable":true,"type":"integer"},{"metadata":{},"name":"transferstatus","nullable":true,"type":"integer"},{"metadata":{},"name":"producttype","nullable":true,"type":"integer"},{"metadata":{},"name":"productsubtype","nullable":true,"type":"integer"},{"metadata":{},"name":"productnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productname","nullable":true,"type":"string"},{"metadata":{},"name":"productsearchname","nullable":true,"type":"string"},{"metadata":{},"name":"productdescription","nullable":true,"type":"string"},{"metadata":{},"name":"iscatchweightproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"productdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"storagedimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"trackingdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"variantconfigurationtechnology","nullable":true,"type":"integer"},{"metadata":{},"name":"areidenticalconfigurationsallowed","nullable":true,"type":"integer"},{"metadata":{},"name":"isautomaticvariantgenerationenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"isproductvariantunitconversionenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"retailproductcategoryname","nullable":true,"type":"string"},{"metadata":{},"name":"productcolorgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"productsizegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"productstylegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"isproductkit","nullable":true,"type":"integer"},{"metadata":{},"name":"stcccode","nullable":true,"type":"string"},{"metadata":{},"name":"harmonizedsystemcode","nullable":true,"type":"string"},{"metadata":{},"name":"nmfccode","nullable":true,"type":"string"},{"metadata":{},"name":"partition","nullable":true,"type":"string"},{"metadata":{},"name":"productvariantnamenomenclaturename","nullable":true,"type":"string"},{"metadata":{},"name":"productvariantnumbernomenclaturename","nullable":true,"type":"string"},{"metadata":{},"name":"servicetype","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantydurationtime","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantydurationtimeunit","nullable":true,"type":"integer"},{"metadata":{},"name":"syncstartdatetime","nullable":true,"type":"timestamp"},{"metadata":{},"name":"recid","nullable":true,"type":"long"},{"metadata":{},"name":"dl_rowhash","nullable":true,"type":"string"},{"metadata":{},"name":"dl_partitionkey","nullable":true,"type":"string"},{"metadata":{},"name":"dl_iscurrent","nullable":true,"type":"boolean"},{"metadata":{},"name":"dl_recordstartdateutc","nullable":true,"type":"string"},{"metadata":{},"name":"dl_recordenddateutc","nullable":true,"type":"timestamp"},{"metadata":{},"name":"dl_sourcefilename","nullable":true,"type":"string"}],"type":"struct"},"tableIdentifier":"dbfs:/mnt/DELTA/classicbrands/ecoresreleasedproductv2staging"},{"name":"insertDF","typeStr":"pyspark.sql.dataframe.DataFrame","schema":{"fields":[{"metadata":{},"name":"productsubtype","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultledgerdimensiondisplayvalue","nullable":true,"type":"string"},{"metadata":{},"name":"aretransportationmanagementprocessesenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"costchargesquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"saleschargesquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasechargesquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"pkwiucode","nullable":true,"type":"string"},{"metadata":{},"name":"definitiongroup","nullable":true,"type":"string"},{"metadata":{},"name":"executionid","nullable":true,"type":"string"},{"metadata":{},"name":"isselected","nullable":true,"type":"integer"},{"metadata":{},"name":"transferstatus","nullable":true,"type":"integer"},{"metadata":{},"name":"producttype","nullable":true,"type":"integer"},{"metadata":{},"name":"productnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productsearchname","nullable":true,"type":"string"},{"metadata":{},"name":"itemnumber","nullable":true,"type":"string"},{"metadata":{},"name":"searchname","nullable":true,"type":"string"},{"metadata":{},"name":"productgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"itemmodelgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"storagedimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"trackingdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"inventoryreservationhierarchyname","nullable":true,"type":"string"},{"metadata":{},"name":"defaultordertype","nullable":true,"type":"integer"},{"metadata":{},"name":"marginabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"revenueabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"carryingcostabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"valueabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"alternativeproductusagecondition","nullable":true,"type":"integer"},{"metadata":{},"name":"alternativeproductconfigurationid","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeproductcolorid","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeproductsizeid","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeproductstyleid","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeitemnumber","nullable":true,"type":"string"},{"metadata":{},"name":"willinventoryissueautomaticallyreportasfinished","nullable":true,"type":"integer"},{"metadata":{},"name":"batchmergedatecalculationmethod","nullable":true,"type":"integer"},{"metadata":{},"name":"batchnumbergroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"costcalculationgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"bomlevel","nullable":true,"type":"integer"},{"metadata":{},"name":"bomunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"flushingprinciple","nullable":true,"type":"integer"},{"metadata":{},"name":"willinventoryreceiptignoreflushingprinciple","nullable":true,"type":"integer"},{"metadata":{},"name":"commissionproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"costgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"isunitcostautomaticallyupdated","nullable":true,"type":"integer"},{"metadata":{},"name":"productionconsumptiondensityconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productionconsumptiondepthconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productionconsumptionwidthconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productionconsumptionheightconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"grossdepth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"grossproductheight","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"grossproductwidth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productvolume","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"netproductweight","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"buyergroupid","nullable":true,"type":"string"},{"metadata":{},"name":"isunitcostproductvariantspecific","nullable":true,"type":"integer"},{"metadata":{},"name":"purchasepricetolerancegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"vendorinvoicelinematchingpolicy","nullable":true,"type":"integer"},{"metadata":{},"name":"packingmaterialgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"bestbeforeperioddays","nullable":true,"type":"integer"},{"metadata":{},"name":"freightallocationgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"salesrebateproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"potencybaseattributevalueentryevent","nullable":true,"type":"integer"},{"metadata":{},"name":"shelfadviceperioddays","nullable":true,"type":"integer"},{"metadata":{},"name":"shelflifeperioddays","nullable":true,"type":"integer"},{"metadata":{},"name":"potencybaseattibutetargetvalue","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"approvedvendorcheckmethod","nullable":true,"type":"integer"},{"metadata":{},"name":"isphantom","nullable":true,"type":"integer"},{"metadata":{},"name":"planningformulaitemnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productiontype","nullable":true,"type":"integer"},{"metadata":{},"name":"yieldpercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"primaryvendoraccountnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productiongroupid","nullable":true,"type":"string"},{"metadata":{},"name":"productionpoolid","nullable":true,"type":"string"},{"metadata":{},"name":"projectcategoryid","nullable":true,"type":"string"},{"metadata":{},"name":"necessaryproductionworkingtimeschedulingpropertyid","nullable":true,"type":"string"},{"metadata":{},"name":"ispurchasepriceautomaticallyupdated","nullable":true,"type":"integer"},{"metadata":{},"name":"productcoveragegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"salespricecalculationmodel","nullable":true,"type":"integer"},{"metadata":{},"name":"basesalespricesource","nullable":true,"type":"integer"},{"metadata":{},"name":"salespricecalculationcontributionratio","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salespricecalculationchargespercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"constantscrapquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"variablescrappercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"serialnumbergroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"shippingandreceivingsortordercode","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultproductconfigurationid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductcolorid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductsizeid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductstyleid","nullable":true,"type":"string"},{"metadata":{},"name":"tareproductweight","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"packingdutyquantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"arrivalhandlingtime","nullable":true,"type":"integer"},{"metadata":{},"name":"warehousefiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"warehousefiltergroup","nullable":true,"type":"string"},{"metadata":{},"name":"maximumpickquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"packageclassid","nullable":true,"type":"string"},{"metadata":{},"name":"packsizecategoryid","nullable":true,"type":"string"},{"metadata":{},"name":"physicaldimensiongroupid","nullable":true,"type":"string"},{"metadata":{},"name":"willworkcenterpickingallownegativeinventory","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultreceivingquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"warehousemobiledevicedescriptionline1","nullable":true,"type":"string"},{"metadata":{},"name":"warehousemobiledevicedescriptionline2","nullable":true,"type":"string"},{"metadata":{},"name":"rawmaterialpickingprinciple","nullable":true,"type":"integer"},{"metadata":{},"name":"unitconversionsequencegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"catchweightunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"maximumcatchweightquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"minimumcatchweightquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"issalespriceadjustmentallowed","nullable":true,"type":"integer"},{"metadata":{},"name":"continuityeventduration","nullable":true,"type":"integer"},{"metadata":{},"name":"continuityscheduleid","nullable":true,"type":"string"},{"metadata":{},"name":"isrestrictedforcoupons","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultdirectdeliverywarehouse","nullable":true,"type":"string"},{"metadata":{},"name":"isdelivereddirectly","nullable":true,"type":"integer"},{"metadata":{},"name":"isexemptfromautomaticnotificationandcancellation","nullable":true,"type":"integer"},{"metadata":{},"name":"isinstallmenteligible","nullable":true,"type":"integer"},{"metadata":{},"name":"willpickingworkbenchapplyboxinglogic","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaserebateproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"sellenddate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"sellstartdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"isshipaloneenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"shipstartdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"barcodesetupid","nullable":true,"type":"string"},{"metadata":{},"name":"comparisonpricebaseunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"isscaleproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"isposregistrationblocked","nullable":true,"type":"integer"},{"metadata":{},"name":"posregistrationblockeddate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"posregistrationplannedblockeddate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"posregistrationactivationdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"keyinpricerequirementsatposregister","nullable":true,"type":"integer"},{"metadata":{},"name":"keyinquantityrequirementsatposregister","nullable":true,"type":"integer"},{"metadata":{},"name":"mustkeyincommentatposregister","nullable":true,"type":"integer"},{"metadata":{},"name":"iszeropriceposregistrationallowed","nullable":true,"type":"integer"},{"metadata":{},"name":"isdiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"isposregistrationquantitynegative","nullable":true,"type":"integer"},{"metadata":{},"name":"productlifecycleseasoncode","nullable":true,"type":"string"},{"metadata":{},"name":"productlifecyclevalidfromdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"productlifecyclevalidtodate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"isvariantshelflabelsprintingenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"transferorderoverdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"transferorderunderdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"unitcost","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"unitcostdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"unitcostquantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"isunitcostincludingcharges","nullable":true,"type":"integer"},{"metadata":{},"name":"fixedcostcharges","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"inventoryunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"isintercompanysalesusageblocked","nullable":true,"type":"integer"},{"metadata":{},"name":"salessalestaxitemgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"willtotalsalesdiscountcalculationincludeproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"saleslinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"salesmultilinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"salessupplementaryproductproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"saleschargeproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"salesoverdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salesunderdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salespricingprecision","nullable":true,"type":"integer"},{"metadata":{},"name":"salesprice","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salespricedate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"salespricequantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"salesunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"issalespriceincludingcharges","nullable":true,"type":"integer"},{"metadata":{},"name":"fixedsalespricecharges","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasesalestaxitemgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"isintercompanypurchaseusageblocked","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaselinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"purchasemultilinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"willtotalpurchasediscountcalculationincludeproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"purchasesupplementaryproductproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"purchaseoverdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchaseunderdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasepricingprecision","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaseprice","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasepricedate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"purchasepricequantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"purchaseunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"purchasechargeproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"ispurchasepriceincludingcharges","nullable":true,"type":"integer"},{"metadata":{},"name":"fixedpurchasepricecharges","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"intrastatcommoditycode","nullable":true,"type":"string"},{"metadata":{},"name":"intrastatchargepercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"origincountryregionid","nullable":true,"type":"string"},{"metadata":{},"name":"originstateid","nullable":true,"type":"string"},{"metadata":{},"name":"approximatesalestaxpercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"itemfiscalclassificationexceptioncode","nullable":true,"type":"string"},{"metadata":{},"name":"productfiscalinformationtype","nullable":true,"type":"string"},{"metadata":{},"name":"isicmstaxappliedonservice","nullable":true,"type":"integer"},{"metadata":{},"name":"producttaxationorigin","nullable":true,"type":"integer"},{"metadata":{},"name":"itemfiscalclassificationcode","nullable":true,"type":"string"},{"metadata":{},"name":"servicefiscalinformationcode","nullable":true,"type":"string"},{"metadata":{},"name":"issaleswithholdingtaxcalculated","nullable":true,"type":"integer"},{"metadata":{},"name":"salesitemwithholdingtaxgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"ispurchasewithholdingtaxcalculated","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaseitemwithholdingtaxgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"salesgstreliefcategorycode","nullable":true,"type":"string"},{"metadata":{},"name":"purchasegstreliefcategorycode","nullable":true,"type":"string"},{"metadata":{},"name":"inventorygstreliefcategorycode","nullable":true,"type":"string"},{"metadata":{},"name":"ngpcode","nullable":true,"type":"integer"},{"metadata":{},"name":"sadgroup","nullable":true,"type":"string"},{"metadata":{},"name":"ismanualdiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"inventorycountingreasoncodepolicyname","nullable":true,"type":"string"},{"metadata":{},"name":"hsncode","nullable":true,"type":"string"},{"metadata":{},"name":"serviceaccountingcode","nullable":true,"type":"string"},{"metadata":{},"name":"exempt","nullable":true,"type":"integer"},{"metadata":{},"name":"customsimporttariffcode","nullable":true,"type":"string"},{"metadata":{},"name":"customsexporttariffcode","nullable":true,"type":"string"},{"metadata":{},"name":"satcode","nullable":true,"type":"string"},{"metadata":{},"name":"scaleindicator","nullable":true,"type":"integer"},{"metadata":{},"name":"cnpj","nullable":true,"type":"string"},{"metadata":{},"name":"taxsubstitutioncode","nullable":true,"type":"string"},{"metadata":{},"name":"partition","nullable":true,"type":"string"},{"metadata":{},"name":"updateproductvariantlifecyclestate","nullable":true,"type":"integer"},{"metadata":{},"name":"catchweightitemhandlingpolicyname","nullable":true,"type":"string"},{"metadata":{},"name":"firstproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"secondproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"thirdproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"fourthproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"firstproductfiltergroupid","nullable":true,"type":"string"},{"metadata":{},"name":"secondproductfiltergroupid","nullable":true,"type":"string"},{"metadata":{},"name":"taxratetype","nullable":true,"type":"string"},{"metadata":{},"name":"productlifecyclestateid","nullable":true,"type":"string"},{"metadata":{},"name":"nongst","nullable":true,"type":"integer"},{"metadata":{},"name":"productdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"iscatchweightproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecrevenuetype","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecbundle","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecexcludefromcarveout","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecmedianprice","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecmedianpricemaximumtolerance","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"revrecmedianpriceminimumtolerance","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"revrecrevenuerecognitionenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"servicetype","nullable":true,"type":"integer"},{"metadata":{},"name":"isretaildiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"istenderdiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantablepricerangebasetype","nullable":true,"type":"integer"},{"metadata":{},"name":"lowerwarrantablepricerangelimit","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"upperwarrantablepricerangelimit","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"warrantydurationtime","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantydurationtimeunit","nullable":true,"type":"integer"},{"metadata":{},"name":"hsdepth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"hswidth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"hsheight","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"alternativeproductversionid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductversionid","nullable":true,"type":"string"},{"metadata":{},"name":"costcalculationbomlevel","nullable":true,"type":"integer"},{"metadata":{},"name":"salesretailinventoryavailabilitybuffer","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salesretailinventoryavailabilitylevelprofile","nullable":true,"type":"string"},{"metadata":{},"name":"dataareaid","nullable":true,"type":"string"},{"metadata":{},"name":"syncstartdatetime","nullable":true,"type":"timestamp"},{"metadata":{},"name":"dl_rowhash","nullable":true,"type":"string"},{"metadata":{},"name":"dl_partitionkey","nullable":true,"type":"string"},{"metadata":{},"name":"dl_iscurrent","nullable":false,"type":"boolean"},{"metadata":{},"name":"dl_recordstartdateutc","nullable":false,"type":"string"},{"metadata":{},"name":"dl_recordenddateutc","nullable":true,"type":"timestamp"},{"metadata":{},"name":"dl_sourcefilename","nullable":false,"type":"string"},{"metadata":{},"name":"definitiongroup","nullable":true,"type":"string"},{"metadata":{},"name":"executionid","nullable":true,"type":"string"},{"metadata":{},"name":"isselected","nullable":true,"type":"integer"},{"metadata":{},"name":"transferstatus","nullable":true,"type":"integer"},{"metadata":{},"name":"producttype","nullable":true,"type":"integer"},{"metadata":{},"name":"productsubtype","nullable":true,"type":"integer"},{"metadata":{},"name":"productnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productname","nullable":true,"type":"string"},{"metadata":{},"name":"productsearchname","nullable":true,"type":"string"},{"metadata":{},"name":"productdescription","nullable":true,"type":"string"},{"metadata":{},"name":"iscatchweightproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"productdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"storagedimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"trackingdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"variantconfigurationtechnology","nullable":true,"type":"integer"},{"metadata":{},"name":"areidenticalconfigurationsallowed","nullable":true,"type":"integer"},{"metadata":{},"name":"isautomaticvariantgenerationenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"isproductvariantunitconversionenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"retailproductcategoryname","nullable":true,"type":"string"},{"metadata":{},"name":"productcolorgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"productsizegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"productstylegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"isproductkit","nullable":true,"type":"integer"},{"metadata":{},"name":"stcccode","nullable":true,"type":"string"},{"metadata":{},"name":"harmonizedsystemcode","nullable":true,"type":"string"},{"metadata":{},"name":"nmfccode","nullable":true,"type":"string"},{"metadata":{},"name":"partition","nullable":true,"type":"string"},{"metadata":{},"name":"productvariantnamenomenclaturename","nullable":true,"type":"string"},{"metadata":{},"name":"productvariantnumbernomenclaturename","nullable":true,"type":"string"},{"metadata":{},"name":"servicetype","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantydurationtime","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantydurationtimeunit","nullable":true,"type":"integer"},{"metadata":{},"name":"syncstartdatetime","nullable":true,"type":"timestamp"},{"metadata":{},"name":"recid","nullable":true,"type":"long"},{"metadata":{},"name":"dl_rowhash","nullable":true,"type":"string"},{"metadata":{},"name":"dl_partitionkey","nullable":true,"type":"string"},{"metadata":{},"name":"dl_iscurrent","nullable":true,"type":"boolean"},{"metadata":{},"name":"dl_recordstartdateutc","nullable":true,"type":"string"},{"metadata":{},"name":"dl_recordenddateutc","nullable":true,"type":"timestamp"},{"metadata":{},"name":"dl_sourcefilename","nullable":true,"type":"string"}],"type":"struct"},"tableIdentifier":null}],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642855377,"submitTime":1622642822574,"finishTime":1622642880410,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Full Merge","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"3304e5e2-3229-45db-8e9c-c92bfc7156c4"},{"version":"CommandV1","origId":2986605948003670,"guid":"8c6c9742-5117-45b3-8a29-069a762c27aa","subtype":"command","commandType":"auto","position":18.0,"command":"%md\n### Update Delta State table for nightly Optimize job\nNOTE:  Failing to write/update stats table won't fail job.","commandVersion":1,"state":"finished","results":null,"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"fac1dc9a-850a-4530-a417-4c7515b9134c"},{"version":"CommandV1","origId":2986605948003671,"guid":"539e19e3-c9f5-47a0-840a-8a32b0f0179d","subtype":"command","commandType":"auto","position":19.0,"command":"if success:\n    stats_df = spark.table(f\"{pDeltaSchemaName}.{pDeltaObjectName}\")\n\n    # Get current schema to store in metadata table\n    try:\n        sch = stats_df.schema\n        str_clean_cols = sch.simpleString().strip('struct').replace(':', ' ').replace(\"<\",\"\").replace(\">\",\"\").lower()\n    except:\n        str_clean_cols = ''\n\n    print(str_clean_cols)","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\">definitiongroup string,executionid string,isselected int,transferstatus int,producttype int,productsubtype int,productnumber string,productname string,productsearchname string,productdescription string,iscatchweightproduct int,productdimensiongroupname string,storagedimensiongroupname string,trackingdimensiongroupname string,variantconfigurationtechnology int,areidenticalconfigurationsallowed int,isautomaticvariantgenerationenabled int,isproductvariantunitconversionenabled int,retailproductcategoryname string,productcolorgroupid string,productsizegroupid string,productstylegroupid string,isproductkit int,stcccode string,harmonizedsystemcode string,nmfccode string,partition string,productvariantnamenomenclaturename string,productvariantnumbernomenclaturename string,servicetype int,warrantydurationtime int,warrantydurationtimeunit int,syncstartdatetime timestamp,recid bigint,dl_rowhash string,dl_partitionkey string,dl_iscurrent boolean,dl_recordstartdateutc string,dl_recordenddateutc timestamp,dl_sourcefilename string,ispurchasepriceautomaticallyupdated int,isinstallmenteligible int,defaultproductconfigurationid string,pkwiucode string,thirdproductfiltercode string,revrecrevenuerecognitionenabled int,variablescrappercentage decimal(32,6),purchasepricequantity decimal(32,12),hsdepth decimal(32,6),grossproductheight decimal(32,6),isshipaloneenabled int,continuityeventduration int,originstateid string,purchaseprice decimal(32,6),warrantablepricerangebasetype int,costgroupid string,buyergroupid string,basesalespricesource int,isicmstaxappliedonservice int,intrastatchargepercentage decimal(32,6),shipstartdate timestamp,transferorderunderdeliverypercentage decimal(32,6),alternativeproductstyleid string,willinventoryreceiptignoreflushingprinciple int,maximumpickquantity decimal(32,6),ngpcode int,nongst int,salespricingprecision int,dataareaid string,issalespriceadjustmentallowed int,revrecexcludefromcarveout int,unitcostdate timestamp,posregistrationplannedblockeddate timestamp,inventorycountingreasoncodepolicyname string,salesretailinventoryavailabilitylevelprofile string,firstproductfiltercode string,istenderdiscountposregistrationprohibited int,hswidth decimal(32,6),isphantom int,comparisonpricebaseunitsymbol string,purchasechargeproductgroupid string,salespricecalculationmodel int,isexemptfromautomaticnotificationandcancellation int,catchweightunitsymbol string,saleschargesquantity decimal(32,6),bomunitsymbol string,defaultproductstyleid string,isdelivereddirectly int,flushingprinciple int,shelfadviceperioddays int,isunitcostproductvariantspecific int,productlifecycleseasoncode string,minimumcatchweightquantity decimal(32,6),issalespriceincludingcharges int,potencybaseattibutetargetvalue decimal(32,6),isposregistrationblocked int,approximatesalestaxpercentage decimal(32,6),purchasepricingprecision int,secondproductfiltercode string,purchaseitemwithholdingtaxgroupcode string,productionpoolid string,inventoryreservationhierarchyname string,purchasepricetolerancegroupid string,salesunderdeliverypercentage decimal(32,6),secondproductfiltergroupid string,batchmergedatecalculationmethod int,purchasesalestaxitemgroupcode string,revrecbundle int,isvariantshelflabelsprintingenabled int,keyinpricerequirementsatposregister int,productionconsumptiondensityconversionfactor decimal(32,6),productionconsumptiondepthconversionfactor decimal(32,6),revrecmedianprice int,searchname string,isdiscountposregistrationprohibited int,costcalculationbomlevel int,isrestrictedforcoupons int,productlifecyclestateid string,isposregistrationquantitynegative int,ispurchasepriceincludingcharges int,purchaserebateproductgroupid string,customsexporttariffcode string,fixedpurchasepricecharges decimal(32,6),warehousefiltergroup string,isintercompanypurchaseusageblocked int,keyinquantityrequirementsatposregister int,unitcostquantity decimal(32,12),iszeropriceposregistrationallowed int,revrecmedianpricemaximumtolerance decimal(32,6),costcalculationgroupid string,revrecrevenuetype int,isunitcostautomaticallyupdated int,barcodesetupid string,willtotalpurchasediscountcalculationincludeproduct int,catchweightitemhandlingpolicyname string,valueabccode int,salesrebateproductgroupid string,itemmodelgroupid string,grossdepth decimal(32,6),salespricecalculationchargespercentage decimal(32,6),transferorderoverdeliverypercentage decimal(32,6),willtotalsalesdiscountcalculationincludeproduct int,sellenddate timestamp,unitconversionsequencegroupid string,sadgroup string,productvolume decimal(32,6),primaryvendoraccountnumber string,ispurchasewithholdingtaxcalculated int,defaultordertype int,purchaseunderdeliverypercentage decimal(32,6),productfiscalinformationtype string,posregistrationactivationdate timestamp,productionconsumptionheightconversionfactor decimal(32,6),inventorygstreliefcategorycode string,alternativeproductcolorid string,freightallocationgroupid string,taxratetype string,serviceaccountingcode string,exempt int,purchasesupplementaryproductproductgroupid string,productiontype int,salesoverdeliverypercentage decimal(32,6),lowerwarrantablepricerangelimit decimal(32,6),rawmaterialpickingprinciple int,salesgstreliefcategorycode string,alternativeproductconfigurationid string,productiongroupid string,cnpj string,purchasegstreliefcategorycode string,saleslinediscountproductgroupcode string,salesretailinventoryavailabilitybuffer decimal(32,6),salesmultilinediscountproductgroupcode string,salesunitsymbol string,defaultdirectdeliverywarehouse string,salespricequantity decimal(32,12),sellstartdate timestamp,defaultproductversionid string,yieldpercentage decimal(32,6),itemfiscalclassificationcode string,productlifecyclevalidtodate timestamp,revenueabccode int,purchasemultilinediscountproductgroupcode string,serialnumbergroupcode string,packingdutyquantity decimal(32,12),salesitemwithholdingtaxgroupcode string,constantscrapquantity decimal(32,6),alternativeproductsizeid string,marginabccode int,vendorinvoicelinematchingpolicy int,revrecmedianpriceminimumtolerance decimal(32,6),fixedsalespricecharges decimal(32,6),defaultledgerdimensiondisplayvalue string,isunitcostincludingcharges int,inventoryunitsymbol string,purchaseunitsymbol string,taxsubstitutioncode string,alternativeproductusagecondition int,costchargesquantity decimal(32,6),planningformulaitemnumber string,purchasechargesquantity decimal(32,6),willinventoryissueautomaticallyreportasfinished int,alternativeproductversionid string,salessalestaxitemgroupcode string,bomlevel int,unitcost decimal(32,6),warehousefiltercode string,fourthproductfiltercode string,scaleindicator int,fixedcostcharges decimal(32,6),projectcategoryid string,physicaldimensiongroupid string,isretaildiscountposregistrationprohibited int,isintercompanysalesusageblocked int,servicefiscalinformationcode string,tareproductweight decimal(32,12),grossproductwidth decimal(32,6),netproductweight decimal(32,12),mustkeyincommentatposregister int,salessupplementaryproductproductgroupid string,saleschargeproductgroupid string,ismanualdiscountposregistrationprohibited int,aretransportationmanagementprocessesenabled int,warehousemobiledevicedescriptionline2 string,origincountryregionid string,customsimporttariffcode string,continuityscheduleid string,purchaseoverdeliverypercentage decimal(32,6),shelflifeperioddays int,productcoveragegroupid string,willpickingworkbenchapplyboxinglogic int,commissionproductgroupid string,packingmaterialgroupid string,willworkcenterpickingallownegativeinventory int,hsncode string,productgroupid string,defaultproductcolorid string,purchaselinediscountproductgroupcode string,itemfiscalclassificationexceptioncode string,defaultproductsizeid string,producttaxationorigin int,arrivalhandlingtime int,maximumcatchweightquantity decimal(32,6),carryingcostabccode int,necessaryproductionworkingtimeschedulingpropertyid string,itemnumber string,packsizecategoryid string,productlifecyclevalidfromdate timestamp,shippingandreceivingsortordercode int,approvedvendorcheckmethod int,purchasepricedate timestamp,packageclassid string,bestbeforeperioddays int,posregistrationblockeddate timestamp,upperwarrantablepricerangelimit decimal(32,6),salespricecalculationcontributionratio decimal(32,6),satcode string,updateproductvariantlifecyclestate int,salesprice decimal(32,6),warehousemobiledevicedescriptionline1 string,intrastatcommoditycode string,alternativeitemnumber string,salespricedate timestamp,batchnumbergroupcode string,productionconsumptionwidthconversionfactor decimal(32,6),potencybaseattributevalueentryevent int,isscaleproduct int,firstproductfiltergroupid string,hsheight decimal(32,6),defaultreceivingquantity decimal(32,6),issaleswithholdingtaxcalculated int\n</div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[{"name":"stats_df","typeStr":"pyspark.sql.dataframe.DataFrame","schema":{"fields":[{"metadata":{},"name":"definitiongroup","nullable":true,"type":"string"},{"metadata":{},"name":"executionid","nullable":true,"type":"string"},{"metadata":{},"name":"isselected","nullable":true,"type":"integer"},{"metadata":{},"name":"transferstatus","nullable":true,"type":"integer"},{"metadata":{},"name":"producttype","nullable":true,"type":"integer"},{"metadata":{},"name":"productsubtype","nullable":true,"type":"integer"},{"metadata":{},"name":"productnumber","nullable":true,"type":"string"},{"metadata":{},"name":"productname","nullable":true,"type":"string"},{"metadata":{},"name":"productsearchname","nullable":true,"type":"string"},{"metadata":{},"name":"productdescription","nullable":true,"type":"string"},{"metadata":{},"name":"iscatchweightproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"productdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"storagedimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"trackingdimensiongroupname","nullable":true,"type":"string"},{"metadata":{},"name":"variantconfigurationtechnology","nullable":true,"type":"integer"},{"metadata":{},"name":"areidenticalconfigurationsallowed","nullable":true,"type":"integer"},{"metadata":{},"name":"isautomaticvariantgenerationenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"isproductvariantunitconversionenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"retailproductcategoryname","nullable":true,"type":"string"},{"metadata":{},"name":"productcolorgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"productsizegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"productstylegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"isproductkit","nullable":true,"type":"integer"},{"metadata":{},"name":"stcccode","nullable":true,"type":"string"},{"metadata":{},"name":"harmonizedsystemcode","nullable":true,"type":"string"},{"metadata":{},"name":"nmfccode","nullable":true,"type":"string"},{"metadata":{},"name":"partition","nullable":true,"type":"string"},{"metadata":{},"name":"productvariantnamenomenclaturename","nullable":true,"type":"string"},{"metadata":{},"name":"productvariantnumbernomenclaturename","nullable":true,"type":"string"},{"metadata":{},"name":"servicetype","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantydurationtime","nullable":true,"type":"integer"},{"metadata":{},"name":"warrantydurationtimeunit","nullable":true,"type":"integer"},{"metadata":{},"name":"syncstartdatetime","nullable":true,"type":"timestamp"},{"metadata":{},"name":"recid","nullable":true,"type":"long"},{"metadata":{},"name":"dl_rowhash","nullable":true,"type":"string"},{"metadata":{},"name":"dl_partitionkey","nullable":true,"type":"string"},{"metadata":{},"name":"dl_iscurrent","nullable":true,"type":"boolean"},{"metadata":{},"name":"dl_recordstartdateutc","nullable":true,"type":"string"},{"metadata":{},"name":"dl_recordenddateutc","nullable":true,"type":"timestamp"},{"metadata":{},"name":"dl_sourcefilename","nullable":true,"type":"string"},{"metadata":{},"name":"ispurchasepriceautomaticallyupdated","nullable":true,"type":"integer"},{"metadata":{},"name":"isinstallmenteligible","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultproductconfigurationid","nullable":true,"type":"string"},{"metadata":{},"name":"pkwiucode","nullable":true,"type":"string"},{"metadata":{},"name":"thirdproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"revrecrevenuerecognitionenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"variablescrappercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasepricequantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"hsdepth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"grossproductheight","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"isshipaloneenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"continuityeventduration","nullable":true,"type":"integer"},{"metadata":{},"name":"originstateid","nullable":true,"type":"string"},{"metadata":{},"name":"purchaseprice","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"warrantablepricerangebasetype","nullable":true,"type":"integer"},{"metadata":{},"name":"costgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"buyergroupid","nullable":true,"type":"string"},{"metadata":{},"name":"basesalespricesource","nullable":true,"type":"integer"},{"metadata":{},"name":"isicmstaxappliedonservice","nullable":true,"type":"integer"},{"metadata":{},"name":"intrastatchargepercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"shipstartdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"transferorderunderdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"alternativeproductstyleid","nullable":true,"type":"string"},{"metadata":{},"name":"willinventoryreceiptignoreflushingprinciple","nullable":true,"type":"integer"},{"metadata":{},"name":"maximumpickquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"ngpcode","nullable":true,"type":"integer"},{"metadata":{},"name":"nongst","nullable":true,"type":"integer"},{"metadata":{},"name":"salespricingprecision","nullable":true,"type":"integer"},{"metadata":{},"name":"dataareaid","nullable":true,"type":"string"},{"metadata":{},"name":"issalespriceadjustmentallowed","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecexcludefromcarveout","nullable":true,"type":"integer"},{"metadata":{},"name":"unitcostdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"posregistrationplannedblockeddate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"inventorycountingreasoncodepolicyname","nullable":true,"type":"string"},{"metadata":{},"name":"salesretailinventoryavailabilitylevelprofile","nullable":true,"type":"string"},{"metadata":{},"name":"firstproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"istenderdiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"hswidth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"isphantom","nullable":true,"type":"integer"},{"metadata":{},"name":"comparisonpricebaseunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"purchasechargeproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"salespricecalculationmodel","nullable":true,"type":"integer"},{"metadata":{},"name":"isexemptfromautomaticnotificationandcancellation","nullable":true,"type":"integer"},{"metadata":{},"name":"catchweightunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"saleschargesquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"bomunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductstyleid","nullable":true,"type":"string"},{"metadata":{},"name":"isdelivereddirectly","nullable":true,"type":"integer"},{"metadata":{},"name":"flushingprinciple","nullable":true,"type":"integer"},{"metadata":{},"name":"shelfadviceperioddays","nullable":true,"type":"integer"},{"metadata":{},"name":"isunitcostproductvariantspecific","nullable":true,"type":"integer"},{"metadata":{},"name":"productlifecycleseasoncode","nullable":true,"type":"string"},{"metadata":{},"name":"minimumcatchweightquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"issalespriceincludingcharges","nullable":true,"type":"integer"},{"metadata":{},"name":"potencybaseattibutetargetvalue","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"isposregistrationblocked","nullable":true,"type":"integer"},{"metadata":{},"name":"approximatesalestaxpercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"purchasepricingprecision","nullable":true,"type":"integer"},{"metadata":{},"name":"secondproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"purchaseitemwithholdingtaxgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"productionpoolid","nullable":true,"type":"string"},{"metadata":{},"name":"inventoryreservationhierarchyname","nullable":true,"type":"string"},{"metadata":{},"name":"purchasepricetolerancegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"salesunderdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"secondproductfiltergroupid","nullable":true,"type":"string"},{"metadata":{},"name":"batchmergedatecalculationmethod","nullable":true,"type":"integer"},{"metadata":{},"name":"purchasesalestaxitemgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"revrecbundle","nullable":true,"type":"integer"},{"metadata":{},"name":"isvariantshelflabelsprintingenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"keyinpricerequirementsatposregister","nullable":true,"type":"integer"},{"metadata":{},"name":"productionconsumptiondensityconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productionconsumptiondepthconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"revrecmedianprice","nullable":true,"type":"integer"},{"metadata":{},"name":"searchname","nullable":true,"type":"string"},{"metadata":{},"name":"isdiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"costcalculationbomlevel","nullable":true,"type":"integer"},{"metadata":{},"name":"isrestrictedforcoupons","nullable":true,"type":"integer"},{"metadata":{},"name":"productlifecyclestateid","nullable":true,"type":"string"},{"metadata":{},"name":"isposregistrationquantitynegative","nullable":true,"type":"integer"},{"metadata":{},"name":"ispurchasepriceincludingcharges","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaserebateproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"customsexporttariffcode","nullable":true,"type":"string"},{"metadata":{},"name":"fixedpurchasepricecharges","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"warehousefiltergroup","nullable":true,"type":"string"},{"metadata":{},"name":"isintercompanypurchaseusageblocked","nullable":true,"type":"integer"},{"metadata":{},"name":"keyinquantityrequirementsatposregister","nullable":true,"type":"integer"},{"metadata":{},"name":"unitcostquantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"iszeropriceposregistrationallowed","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecmedianpricemaximumtolerance","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"costcalculationgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"revrecrevenuetype","nullable":true,"type":"integer"},{"metadata":{},"name":"isunitcostautomaticallyupdated","nullable":true,"type":"integer"},{"metadata":{},"name":"barcodesetupid","nullable":true,"type":"string"},{"metadata":{},"name":"willtotalpurchasediscountcalculationincludeproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"catchweightitemhandlingpolicyname","nullable":true,"type":"string"},{"metadata":{},"name":"valueabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"salesrebateproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"itemmodelgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"grossdepth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salespricecalculationchargespercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"transferorderoverdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"willtotalsalesdiscountcalculationincludeproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"sellenddate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"unitconversionsequencegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"sadgroup","nullable":true,"type":"string"},{"metadata":{},"name":"productvolume","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"primaryvendoraccountnumber","nullable":true,"type":"string"},{"metadata":{},"name":"ispurchasewithholdingtaxcalculated","nullable":true,"type":"integer"},{"metadata":{},"name":"defaultordertype","nullable":true,"type":"integer"},{"metadata":{},"name":"purchaseunderdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"productfiscalinformationtype","nullable":true,"type":"string"},{"metadata":{},"name":"posregistrationactivationdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"productionconsumptionheightconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"inventorygstreliefcategorycode","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeproductcolorid","nullable":true,"type":"string"},{"metadata":{},"name":"freightallocationgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"taxratetype","nullable":true,"type":"string"},{"metadata":{},"name":"serviceaccountingcode","nullable":true,"type":"string"},{"metadata":{},"name":"exempt","nullable":true,"type":"integer"},{"metadata":{},"name":"purchasesupplementaryproductproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"productiontype","nullable":true,"type":"integer"},{"metadata":{},"name":"salesoverdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"lowerwarrantablepricerangelimit","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"rawmaterialpickingprinciple","nullable":true,"type":"integer"},{"metadata":{},"name":"salesgstreliefcategorycode","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeproductconfigurationid","nullable":true,"type":"string"},{"metadata":{},"name":"productiongroupid","nullable":true,"type":"string"},{"metadata":{},"name":"cnpj","nullable":true,"type":"string"},{"metadata":{},"name":"purchasegstreliefcategorycode","nullable":true,"type":"string"},{"metadata":{},"name":"saleslinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"salesretailinventoryavailabilitybuffer","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salesmultilinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"salesunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"defaultdirectdeliverywarehouse","nullable":true,"type":"string"},{"metadata":{},"name":"salespricequantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"sellstartdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"defaultproductversionid","nullable":true,"type":"string"},{"metadata":{},"name":"yieldpercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"itemfiscalclassificationcode","nullable":true,"type":"string"},{"metadata":{},"name":"productlifecyclevalidtodate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"revenueabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"purchasemultilinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"serialnumbergroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"packingdutyquantity","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"salesitemwithholdingtaxgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"constantscrapquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"alternativeproductsizeid","nullable":true,"type":"string"},{"metadata":{},"name":"marginabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"vendorinvoicelinematchingpolicy","nullable":true,"type":"integer"},{"metadata":{},"name":"revrecmedianpriceminimumtolerance","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"fixedsalespricecharges","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"defaultledgerdimensiondisplayvalue","nullable":true,"type":"string"},{"metadata":{},"name":"isunitcostincludingcharges","nullable":true,"type":"integer"},{"metadata":{},"name":"inventoryunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"purchaseunitsymbol","nullable":true,"type":"string"},{"metadata":{},"name":"taxsubstitutioncode","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeproductusagecondition","nullable":true,"type":"integer"},{"metadata":{},"name":"costchargesquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"planningformulaitemnumber","nullable":true,"type":"string"},{"metadata":{},"name":"purchasechargesquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"willinventoryissueautomaticallyreportasfinished","nullable":true,"type":"integer"},{"metadata":{},"name":"alternativeproductversionid","nullable":true,"type":"string"},{"metadata":{},"name":"salessalestaxitemgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"bomlevel","nullable":true,"type":"integer"},{"metadata":{},"name":"unitcost","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"warehousefiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"fourthproductfiltercode","nullable":true,"type":"string"},{"metadata":{},"name":"scaleindicator","nullable":true,"type":"integer"},{"metadata":{},"name":"fixedcostcharges","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"projectcategoryid","nullable":true,"type":"string"},{"metadata":{},"name":"physicaldimensiongroupid","nullable":true,"type":"string"},{"metadata":{},"name":"isretaildiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"isintercompanysalesusageblocked","nullable":true,"type":"integer"},{"metadata":{},"name":"servicefiscalinformationcode","nullable":true,"type":"string"},{"metadata":{},"name":"tareproductweight","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"grossproductwidth","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"netproductweight","nullable":true,"type":"decimal(32,12)"},{"metadata":{},"name":"mustkeyincommentatposregister","nullable":true,"type":"integer"},{"metadata":{},"name":"salessupplementaryproductproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"saleschargeproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"ismanualdiscountposregistrationprohibited","nullable":true,"type":"integer"},{"metadata":{},"name":"aretransportationmanagementprocessesenabled","nullable":true,"type":"integer"},{"metadata":{},"name":"warehousemobiledevicedescriptionline2","nullable":true,"type":"string"},{"metadata":{},"name":"origincountryregionid","nullable":true,"type":"string"},{"metadata":{},"name":"customsimporttariffcode","nullable":true,"type":"string"},{"metadata":{},"name":"continuityscheduleid","nullable":true,"type":"string"},{"metadata":{},"name":"purchaseoverdeliverypercentage","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"shelflifeperioddays","nullable":true,"type":"integer"},{"metadata":{},"name":"productcoveragegroupid","nullable":true,"type":"string"},{"metadata":{},"name":"willpickingworkbenchapplyboxinglogic","nullable":true,"type":"integer"},{"metadata":{},"name":"commissionproductgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"packingmaterialgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"willworkcenterpickingallownegativeinventory","nullable":true,"type":"integer"},{"metadata":{},"name":"hsncode","nullable":true,"type":"string"},{"metadata":{},"name":"productgroupid","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductcolorid","nullable":true,"type":"string"},{"metadata":{},"name":"purchaselinediscountproductgroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"itemfiscalclassificationexceptioncode","nullable":true,"type":"string"},{"metadata":{},"name":"defaultproductsizeid","nullable":true,"type":"string"},{"metadata":{},"name":"producttaxationorigin","nullable":true,"type":"integer"},{"metadata":{},"name":"arrivalhandlingtime","nullable":true,"type":"integer"},{"metadata":{},"name":"maximumcatchweightquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"carryingcostabccode","nullable":true,"type":"integer"},{"metadata":{},"name":"necessaryproductionworkingtimeschedulingpropertyid","nullable":true,"type":"string"},{"metadata":{},"name":"itemnumber","nullable":true,"type":"string"},{"metadata":{},"name":"packsizecategoryid","nullable":true,"type":"string"},{"metadata":{},"name":"productlifecyclevalidfromdate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"shippingandreceivingsortordercode","nullable":true,"type":"integer"},{"metadata":{},"name":"approvedvendorcheckmethod","nullable":true,"type":"integer"},{"metadata":{},"name":"purchasepricedate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"packageclassid","nullable":true,"type":"string"},{"metadata":{},"name":"bestbeforeperioddays","nullable":true,"type":"integer"},{"metadata":{},"name":"posregistrationblockeddate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"upperwarrantablepricerangelimit","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"salespricecalculationcontributionratio","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"satcode","nullable":true,"type":"string"},{"metadata":{},"name":"updateproductvariantlifecyclestate","nullable":true,"type":"integer"},{"metadata":{},"name":"salesprice","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"warehousemobiledevicedescriptionline1","nullable":true,"type":"string"},{"metadata":{},"name":"intrastatcommoditycode","nullable":true,"type":"string"},{"metadata":{},"name":"alternativeitemnumber","nullable":true,"type":"string"},{"metadata":{},"name":"salespricedate","nullable":true,"type":"timestamp"},{"metadata":{},"name":"batchnumbergroupcode","nullable":true,"type":"string"},{"metadata":{},"name":"productionconsumptionwidthconversionfactor","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"potencybaseattributevalueentryevent","nullable":true,"type":"integer"},{"metadata":{},"name":"isscaleproduct","nullable":true,"type":"integer"},{"metadata":{},"name":"firstproductfiltergroupid","nullable":true,"type":"string"},{"metadata":{},"name":"hsheight","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"defaultreceivingquantity","nullable":true,"type":"decimal(32,6)"},{"metadata":{},"name":"issaleswithholdingtaxcalculated","nullable":true,"type":"integer"}],"type":"struct"},"tableIdentifier":"dbfs:/mnt/DELTA/classicbrands/ecoresreleasedproductv2staging"}],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642880415,"submitTime":1622642822595,"finishTime":1622642880629,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Get a clean list of columns to store in metadata table","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"153267a4-aac6-4fc0-aa83-b3be05060a7a"},{"version":"CommandV1","origId":2986605948003672,"guid":"bcd15459-c742-4356-b2c7-b5c2c62ccc40","subtype":"command","commandType":"auto","position":20.0,"command":"if success:\n    db_exists = check_if_db_exists(state_db_name, True, delta_location)","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642880633,"submitTime":1622642822599,"finishTime":1622642880790,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Check/Create Metadata Database","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"c6595f14-a345-41c7-ab6c-f724e5eceec4"},{"version":"CommandV1","origId":2986605948003673,"guid":"11ce8d84-9943-48e1-819b-034a63e32581","subtype":"command","commandType":"auto","position":21.0,"command":"if success:\n    # Make sure table exists (IN DELTA LAKE)\n    stats_table_exists = check_if_table_exists(state_tbl_name, state_db_name)\n\n    # Create table if it doesn't exist\n    if not stats_table_exists:\n\n        print(f'Creating table for Dataset: {state_db_name}.{state_tbl_name}')\n\n        query = f\"\"\"\n            CREATE TABLE IF NOT EXISTS {state_db_name}.{state_tbl_name}\n            (DeltaSchema string, DeltaTable string, CandidateKey string, Schema string)\n            USING DELTA\n            PARTITIONED BY(DeltaSchema, DeltaTable)\n        \"\"\"\n\n        print(query)\n        spark.sql(query)\n","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642880794,"submitTime":1622642822606,"finishTime":1622642880901,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Check/Create Metadata Table","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"309cdaf7-e38b-4eef-aa1d-af2c01b737c3"},{"version":"CommandV1","origId":2986605948003674,"guid":"d11343c6-e109-45ab-ab60-273a313165b6","subtype":"command","commandType":"auto","position":22.0,"command":"if success and stats_table_exists:\n    # Enable cross joins, however this query is only 1 record, so no worries.\n    spark.conf.set(\"spark.sql.crossJoin.enabled\", \"true\")\n    dt_now = datetime.utcnow()\n    \n    query = f\"\"\"\n        MERGE INTO {state_db_name}.{state_tbl_name} AS stats\n        USING (\n            SELECT lower(\"{pDeltaSchemaName}\") as DeltaSchema, \n                   lower(\"{pDeltaObjectName}\") AS DeltaTable, \n                   lower(\"{pCandidateKey}\") AS CandidateKey, \n                   lower(\"{str_clean_cols}\") AS Schema,\n                   cast('{dt_now}' as timestamp) AS LastUpdated\n          ) AS updates\n        ON stats.DeltaSchema = lower(updates.DeltaSchema)\n          AND stats.DeltaTable = lower(updates.DeltaTable)\n          AND stats.DeltaSchema = lower(\"{pDeltaSchemaName}\")\n          AND stats.DeltaTable = lower(\"{pDeltaObjectName}\")\n        --WHEN MATCHED AND (stats.CandidateKey <> updates.CandidateKey OR stats.Schema <> updates.Schema) THEN\n        WHEN MATCHED THEN -- We always want to update the LastUpdated column each time this notebook runs\n          UPDATE SET *\n        WHEN NOT MATCHED THEN\n          INSERT *\n    \"\"\"\n    try:\n        spark.sql(query)\n    except:\n        pass\n        print(f\"WARNING:  Unable to update schema in {state_db_name}.{state_tbl_name}!\")\n    ","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642880905,"submitTime":1622642822611,"finishTime":1622642883835,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Update metadata table if changes to candidate key or schema","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"8ea0ea51-49a4-4eaf-be66-553b5a891292"},{"version":"CommandV1","origId":2986605948003675,"guid":"eacf9ccb-41a9-45fb-9ab4-0e5d5b5573d1","subtype":"command","commandType":"auto","position":22.5,"command":"# df = spark.table(f\"{pDeltaSchemaName}.{pDeltaObjectName}\")\n# df = df.filter(df.dl_iscurrent == 1)\n# df.write.format(\"delta\").mode(\"overwrite\").saveAsTable(f\"{pDeltaSchemaSilverName}.{pDeltaObjectName}\")\nquery = f\"\"\" CREATE OR REPLACE VIEW {pDeltaSchemaSilverName}.{pDeltaObjectName} AS SELECT * FROM {pDeltaSchemaName}.{pDeltaObjectName} WHERE dl_iscurrent = 1\n\"\"\"\nspark.sql(query)","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\">Out[207]: DataFrame[]</div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642883839,"submitTime":1622642822615,"finishTime":1622642886465,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":null,"yColumns":null,"pivotColumns":null,"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Write to Silver Layer","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"1f41ed37-2b70-4449-adc3-c9bb04e3ea8a"},{"version":"CommandV1","origId":2986605948003676,"guid":"dd8546d9-2836-4253-8e02-ed979163a13f","subtype":"command","commandType":"auto","position":23.0,"command":"# clean up temp views and cache before exiting\ntry:\n    spark.catalog.dropTempView(vw_name_raw)\n    spark.catalog.dropTempView(vw_name_enriched)\n    sdf.unpersist()\nexcept:\n    pass\n","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642886468,"submitTime":1622642822620,"finishTime":1622642886489,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Clean up temp views and cache before exiting","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"9aa4982e-289b-43ac-b3af-003ddc21196d"},{"version":"CommandV1","origId":2986605948003677,"guid":"1f80905f-0df7-4293-a541-e7b7c16af182","subtype":"command","commandType":"auto","position":24.0,"command":"# If this is the initial write/create, we must ensure it's flagged as \"data has changed\"\nif not table_exists:\n    data_changed = 1\n\n#Convert to an INT to store in State system\nsuccess_output = int(success)\n\n# Using DELTA table metadata, determine if table has changed during the\n# most recent action on the table, where action is updates/inserts/merges/writes\nif success:\n    query = f\"\"\"\n    select max(Success) as Success, Max(Message) as Message, Max(DeltaTable) as DeltaTable, max(DataChanged) as DataChanged \n    from (\n    select\n        {success_output} as Success,\n        \"{output_message}\" AS Message,\n        CONCAT('{pDeltaSchemaName}', '.', '{pDeltaObjectName}') as DeltaTable,\n          CASE WHEN operation = 'WRITE' THEN 1\n             WHEN operation = 'UPDATE' and (operationMetrics.numTargetRowsUpdated + operationMetrics.numTargetRowsInserted) > 0 THEN 1\n             WHEN operation like 'CREATE%' THEN 1\n             WHEN operation like 'MERGE' and (operationMetrics.numTargetRowsUpdated + operationMetrics.numTargetRowsInserted) > 0 THEN 1\n             ELSE 0\n        END AS DataChanged\n    from (DESCRIBE HISTORY {pDeltaSchemaName}.{pDeltaObjectName} limit 10)\n    where datediff(current_date, cast(timestamp as DATE)) <= 1\n    and (operation in (\"UPDATE\", \"WRITE\", \"MERGE\")\n        OR operation like 'CREATE%')\n    )\"\"\"\nelse:\n    query = f\"select {success_output} as Success, '{output_message}' as Message, CONCAT('{pDeltaSchemaName}', '.', '{pDeltaObjectName}') as DeltaTable, 0 as DataChanged\"\n\n# Execute query\ndf = spark.sql(query)\n\n# Construct JSON string from dataframe for output\noutput_json = df.select(to_json(struct(col(\"Success\"), col(\"Message\"), col(\"DeltaTable\"), col(\"DataChanged\"))).alias(\"json\")).collect()[0][\"json\"]\n\n########################################################\n# Future functionality:  \n#   Remove the raise exception, and let ADF \"handle\" the success/failure based on JSON content.\n#   This will prevent ADF from blindly re-trying the notebook again for failures bound to happen \n#   again due to data issues, etc.\n########################################################\nif success:\n    # Exit notebook, returning JSON to ADF pipeline\n    dbutils.notebook.exit(output_json)\nelse:\n    # Exit, raising an exception for calling process\n    raise ValueError(output_json)","commandVersion":1,"state":"finished","results":{"type":"exit","data":"{\"Success\":1,\"Message\":\"\",\"DeltaTable\":\"classicbrands.ecoresreleasedproductv2staging\",\"DataChanged\":1}","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642886491,"submitTime":1622642822626,"finishTime":1622642887914,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"Generate output string (JSON) and exit notebook","showCommandTitle":true,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"5f9664bb-ce6d-44fe-ad09-5532dbbe2060"},{"version":"CommandV1","origId":3160520700535558,"guid":"64643529-9c77-4c78-8020-3f54aca8b344","subtype":"script","commandType":"auto","position":3.033333333333333,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822785,"submitTime":0,"finishTime":1622642822805,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"d62311a7-be3c-498c-b12e-ca94329db284"},{"version":"CommandV1","origId":3160520700535559,"guid":"ad1da74f-a375-4896-bc0b-b028f5e03df9","subtype":"script","commandType":"auto","position":3.066666666666667,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822808,"submitTime":0,"finishTime":1622642822825,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"6a9eb0d7-50a9-44e8-90e8-59a7369c43bf"},{"version":"CommandV1","origId":3160520700535560,"guid":"3966164b-aee0-4f3e-9b05-86fa633d98a2","subtype":"script","commandType":"auto","position":3.1,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822828,"submitTime":0,"finishTime":1622642822847,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"98848bd5-ca5c-4049-9c09-249a58ffd562"},{"version":"CommandV1","origId":3160520700535561,"guid":"b0852aec-5994-4e47-830f-dc8ff93c9c6c","subtype":"script","commandType":"auto","position":3.1333333333333333,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822850,"submitTime":0,"finishTime":1622642822873,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"16b69105-4b11-4b61-b3da-aa8772c529c7"},{"version":"CommandV1","origId":3160520700535562,"guid":"54de251d-94a8-41b5-b008-5a82570b8944","subtype":"script","commandType":"auto","position":3.1666666666666665,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822876,"submitTime":0,"finishTime":1622642822892,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b26ae44c-4b25-45c1-81f2-d7545c4eec8a"},{"version":"CommandV1","origId":3160520700535563,"guid":"f5b7021f-854f-4e31-ac01-6885fd3f448d","subtype":"script","commandType":"auto","position":3.2,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822894,"submitTime":0,"finishTime":1622642822912,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"cce1a3d1-b0dc-4512-811f-294e7aa2af0b"},{"version":"CommandV1","origId":3160520700535564,"guid":"4d0e9041-d31b-4a0b-8424-604f3e3a25c1","subtype":"script","commandType":"auto","position":3.2333333333333334,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822914,"submitTime":0,"finishTime":1622642822930,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"abdc8deb-2f27-4cd3-9e0b-e69ef54a874c"},{"version":"CommandV1","origId":3160520700535565,"guid":"b3341ae0-9e7e-4e10-885c-e81d78611d3f","subtype":"script","commandType":"auto","position":3.2666666666666666,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822932,"submitTime":0,"finishTime":1622642822948,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"7f528320-e891-4403-9e60-bb5b532b7384"},{"version":"CommandV1","origId":3160520700535566,"guid":"94be1b1c-a99d-45e6-afd2-eb1e10b0f901","subtype":"script","commandType":"auto","position":3.3,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822950,"submitTime":0,"finishTime":1622642822966,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"6c5783ec-7193-497c-ae48-8f359dd15d79"},{"version":"CommandV1","origId":3160520700535567,"guid":"ea56ca77-1bbf-4ef6-9e5d-76bad7190b60","subtype":"script","commandType":"auto","position":3.3333333333333335,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822968,"submitTime":0,"finishTime":1622642822991,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b6ecd665-7a6f-4c6b-bb5d-2e7cafaa1ece"},{"version":"CommandV1","origId":3160520700535568,"guid":"dcabe424-e0db-4ff8-b521-65820c33821d","subtype":"script","commandType":"auto","position":3.3666666666666667,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642822993,"submitTime":0,"finishTime":1622642823016,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b4d11207-d953-4fea-80bd-48307d1f101e"},{"version":"CommandV1","origId":3160520700535569,"guid":"bec045ef-6f67-45c0-8a98-e06400e3b721","subtype":"script","commandType":"auto","position":3.4,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823018,"submitTime":0,"finishTime":1622642823036,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b7b1e43b-7474-4cf0-ae05-b94893f23300"},{"version":"CommandV1","origId":3160520700535570,"guid":"b1bbf196-24ea-48fb-8121-29ee5671bc15","subtype":"script","commandType":"auto","position":3.4333333333333336,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823038,"submitTime":0,"finishTime":1622642823055,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"67be1ec5-042a-4820-91b5-546c62afa16b"},{"version":"CommandV1","origId":3160520700535571,"guid":"a8540249-0b34-4227-92c2-c05a4fd5fd08","subtype":"script","commandType":"auto","position":3.466666666666667,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823058,"submitTime":0,"finishTime":1622642823078,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"c9ba476d-bc46-4cb5-8273-442ad0c13aa1"},{"version":"CommandV1","origId":3160520700535572,"guid":"65bac13e-a8aa-4f75-954b-2ec3faa88c4a","subtype":"script","commandType":"auto","position":3.5,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823080,"submitTime":0,"finishTime":1622642823103,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"61923e5c-f069-4991-bce1-d1998f03ca50"},{"version":"CommandV1","origId":3160520700535573,"guid":"d0e4c10c-b24e-426b-83cc-e95a0463ac8d","subtype":"script","commandType":"auto","position":3.533333333333333,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823106,"submitTime":0,"finishTime":1622642823135,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"6a2386a8-3585-4f5b-8d1a-6a221008a3a8"},{"version":"CommandV1","origId":3160520700535574,"guid":"3846d055-1c18-4ede-a9d6-9495323798ac","subtype":"script","commandType":"auto","position":3.5666666666666664,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823138,"submitTime":0,"finishTime":1622642823162,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"2034a836-ca3b-4e74-9c99-9c5406fae914"},{"version":"CommandV1","origId":3160520700535575,"guid":"305219d6-0e1e-43da-b66d-ed81ebd38421","subtype":"script","commandType":"auto","position":3.6,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823165,"submitTime":0,"finishTime":1622642823187,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"fa7038b8-eeae-4faf-b67c-3533f592c4be"},{"version":"CommandV1","origId":3160520700535576,"guid":"57efd594-18c2-47b5-9ff3-1a4410310552","subtype":"script","commandType":"auto","position":3.6333333333333333,"command":"","commandVersion":9,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823190,"submitTime":0,"finishTime":1622642823213,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"f7e130f3-a10c-4be5-b766-bf22f0eaa005"},{"version":"CommandV1","origId":3160520700535577,"guid":"42d8aa40-205d-4735-b756-25c6f198812c","subtype":"script","commandType":"auto","position":3.6666666666666665,"command":"","commandVersion":19,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823216,"submitTime":0,"finishTime":1622642823240,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"cd3b7f3c-4cff-4081-8f26-8fee0f0b4dd6"},{"version":"CommandV1","origId":3160520700535578,"guid":"084c03c1-9647-448f-95ba-ba405036546e","subtype":"script","commandType":"auto","position":3.7,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823243,"submitTime":0,"finishTime":1622642823272,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"4a226d2a-a3e4-4c6d-98fa-85f8ba9827c9"},{"version":"CommandV1","origId":3160520700535579,"guid":"2f4a7391-31d4-446c-99cd-687326c041bf","subtype":"script","commandType":"auto","position":3.7333333333333334,"command":"","commandVersion":5,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823274,"submitTime":0,"finishTime":1622642823296,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"e1c71d16-34f1-47f0-bf92-ee4ac052e889"},{"version":"CommandV1","origId":3160520700535580,"guid":"8dbcda8d-e6e7-4d91-85b0-3814edcbb1f3","subtype":"script","commandType":"auto","position":3.7666666666666666,"command":"","commandVersion":4,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823299,"submitTime":0,"finishTime":1622642823322,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"0b9ccd37-947a-47e6-ae42-d1e165f3c085"},{"version":"CommandV1","origId":3160520700535581,"guid":"b9c920f4-7ad7-4064-aa30-a6e5175db86e","subtype":"script","commandType":"auto","position":3.8,"command":"","commandVersion":11,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823325,"submitTime":0,"finishTime":1622642823351,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":null,"yColumns":null,"pivotColumns":null,"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b0e73abd-c26a-4354-ba36-0b5f8d95caee"},{"version":"CommandV1","origId":3160520700535582,"guid":"12a411eb-80ef-45fd-bdd9-fd9a7c95b72a","subtype":"script","commandType":"auto","position":3.8333333333333335,"command":"","commandVersion":4,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823354,"submitTime":0,"finishTime":1622642823377,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"9e4e7bf9-f126-472c-9fbc-f4e8613533fb"},{"version":"CommandV1","origId":3160520700535583,"guid":"526c0d2d-85a7-41d8-9449-644a95627a75","subtype":"script","commandType":"auto","position":3.8666666666666667,"command":"","commandVersion":42,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823379,"submitTime":0,"finishTime":1622642823408,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"2ebbc8ec-3d9a-4199-8649-df651f2387b6"},{"version":"CommandV1","origId":3160520700535584,"guid":"63d24ae6-29aa-475d-902b-d5726895aee4","subtype":"script","commandType":"auto","position":3.9,"command":"","commandVersion":3,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823411,"submitTime":0,"finishTime":1622642823431,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":null,"yColumns":null,"pivotColumns":null,"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"966dd1bf-77f9-4969-87ce-57a3dc32afb7"},{"version":"CommandV1","origId":3160520700535585,"guid":"e2635c9e-3850-4132-b8ec-8f2ee9bd548c","subtype":"script","commandType":"auto","position":3.9333333333333336,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823434,"submitTime":0,"finishTime":1622642823458,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"5da50073-429f-401d-b40f-6379eeead438"},{"version":"CommandV1","origId":3160520700535586,"guid":"a9df4144-44a1-4fef-a202-3bdfb058d29d","subtype":"script","commandType":"auto","position":3.966666666666667,"command":"","commandVersion":1,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1622642823460,"submitTime":0,"finishTime":1622642823479,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":["bf6e9cdb-24e4-4751-bbf3-8f98e4bce556"],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"7a379d31-5ff4-4887-8d39-12f09f6645d0"}],"dashboards":[],"guid":"ab67e82b-1208-4686-91bb-fe643a2c2027","globalVars":{},"iPythonMetadata":{},"inputWidgets":{"CandidateKey":{"nuid":"e9850f0a-9a85-4d27-9f69-e5a3cdb0bf80","currentValue":"","widgetInfo":{"widgetType":"text","name":"CandidateKey","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"deltaCandidateKey":{"nuid":"e2908798-8f25-4eef-b3a0-fd758e5edf4a","currentValue":"hmy","widgetInfo":{"widgetType":"text","name":"deltaCandidateKey","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"deltaObjectName":{"nuid":"0e59ff88-b4c2-4647-a921-a2735c07ebf1","currentValue":"EcoResReleasedProductV2Staging","widgetInfo":{"widgetType":"text","name":"deltaObjectName","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"PartitionKey":{"nuid":"9606aab7-e300-4ea2-8974-25d397afaf4f","currentValue":"","widgetInfo":{"widgetType":"text","name":"PartitionKey","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"deltaPartitionKey":{"nuid":"543a904f-a73f-4f29-9f51-ee68a294fda0","currentValue":"","widgetInfo":{"widgetType":"text","name":"deltaPartitionKey","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"preprocessingParams":{"nuid":"f779a068-1dc7-470f-8ba9-9471251459e6","currentValue":"","widgetInfo":{"widgetType":"text","name":"preprocessingParams","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"fileType":{"nuid":"6683b178-b095-44b9-bd34-056893fa9811","currentValue":"FULL","widgetInfo":{"widgetType":"dropdown","name":"fileType","defaultValue":"FULL","label":null,"options":{"widgetType":"dropdown","choices":["FULL","INCREMENTAL"]}}},"fileColDelimiter":{"nuid":"688aec7f-a92d-4a31-b338-408984529403","currentValue":"\u0007","widgetInfo":{"widgetType":"text","name":"fileColDelimiter","defaultValue":"\u0007","label":null,"options":{"widgetType":"text","validationRegex":null}}},"fileQuoteCharacter":{"nuid":"2851ba07-3116-4703-ab56-739ee2b06191","currentValue":"\u000E","widgetInfo":{"widgetType":"text","name":"fileQuoteCharacter","defaultValue":"\u000E","label":null,"options":{"widgetType":"text","validationRegex":null}}},"ObjectName":{"nuid":"40e03731-cb1e-41f1-b72b-4211d576e424","currentValue":"EcoResReleasedProductV2Staging","widgetInfo":{"widgetType":"text","name":"ObjectName","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"fileHeader":{"nuid":"7aa887d9-eb3a-4d4d-9dc9-ca74966230be","currentValue":"FALSE","widgetInfo":{"widgetType":"dropdown","name":"fileHeader","defaultValue":"FALSE","label":null,"options":{"widgetType":"dropdown","choices":["TRUE","FALSE"]}}},"filePath":{"nuid":"9560be9a-7c8d-4faf-9e26-17825bfb8188","currentValue":"/RAW/classicbrands/byod/EcoResReleasedProductV2Staging/2021/05/17/EcoResReleasedProductV2Staging_Full_2021-05-17T13-30-55.parquet","widgetInfo":{"widgetType":"text","name":"filePath","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"SchemaName":{"nuid":"646a35d7-6bd7-49f6-abad-2f5bc92ab672","currentValue":"classicbrands","widgetInfo":{"widgetType":"text","name":"SchemaName","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"excludedColumns":{"nuid":"c1ed2c0a-168f-4bdf-a4a8-44780a6136e5","currentValue":"","widgetInfo":{"widgetType":"text","name":"excludedColumns","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"dateFormat":{"nuid":"79bb7800-308f-47d7-8972-c2b429d1f42c","currentValue":"yyyy-MM-dd HH:mm:ss.S","widgetInfo":{"widgetType":"text","name":"dateFormat","defaultValue":"yyyy-MM-dd HH:mm:ss.S","label":null,"options":{"widgetType":"text","validationRegex":null}}},"batchId":{"nuid":"e494c1be-c3be-426e-abae-eb3b740d6566","currentValue":"15168","widgetInfo":{"widgetType":"text","name":"batchId","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"StageFolderPath":{"nuid":"6b006a43-e406-44c5-92a5-d394299516e7","currentValue":"","widgetInfo":{"widgetType":"text","name":"StageFolderPath","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"fileSchema":{"nuid":"2b16b543-4a03-4c6d-b482-92da8d07b33e","currentValue":"","widgetInfo":{"widgetType":"text","name":"fileSchema","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"deltaSchemaName":{"nuid":"6b869664-a516-4582-a8b9-f71b4a435816","currentValue":"classicbrands","widgetInfo":{"widgetType":"text","name":"deltaSchemaName","defaultValue":"","label":null,"options":{"widgetType":"text","validationRegex":null}}},"fileFormat":{"nuid":"ff4c1c56-ddaa-4294-9ec6-0a860c2d2534","currentValue":"PARQUET","widgetInfo":{"widgetType":"dropdown","name":"fileFormat","defaultValue":"ORC","label":null,"options":{"widgetType":"dropdown","choices":["PARQUET","CSV","ORC"]}}}},"notebookMetadata":{"pythonIndentUnit":2}}
