{"version":"NotebookV1","origId":3273046978841386,"name":"SharedUtils","language":"python","commands":[{"version":"CommandV1","origId":3273046978841387,"guid":"6b08e75e-719b-48c1-8711-d7eec04039b4","subtype":"command","commandType":"auto","position":2.0,"command":"#\n# Generic functions to share across notebooks\n#","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"d62311a7-be3c-498c-b12e-ca94329db284"},{"version":"CommandV1","origId":3273046978841388,"guid":"cb5375a7-d525-4402-9d7a-8ec17d1b6d76","subtype":"command","commandType":"auto","position":3.0,"command":"def get_column_position(df, column_name):\n    \"\"\"\n    Description:\n      Finds the position of a column in dataframe\n      This is a case-insensitive search.\n\n    Params:\n      df         : dataframe (Mandatory) Name of dataframe\n      column_name: string (Mandatory) The column name you're searching for\n    \n    Example:\n      pos = get_column_position(df, \"META__SourceKeyAttribute\")\n      print(pos)\n    \"\"\"\n    # Get a list of dataframe columns\n    df_cols = df.columns\n    \n    # Convert list to lower case \n    df_cols = [item.lower() for item in df_cols]\n\n    # Get # of cols in case we don't find the desired column, \n    # so we can just return the last col position\n    df_len = len(df_cols) \n    \n    try:\n        col_pos = df_cols.index(column_name.lower())\n    except:\n        col_pos = df_len\n    \n    return col_pos","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"6a9eb0d7-50a9-44e8-90e8-59a7369c43bf"},{"version":"CommandV1","origId":3273046978841389,"guid":"7a353536-4e79-4adf-9bdf-4089ca831287","subtype":"command","commandType":"auto","position":4.0,"command":"#\n# PARALLEL NOTEBOOK PROCESSING\n#\nfrom concurrent.futures import ThreadPoolExecutor\nimport multiprocessing as mp\n\n# Get core count to limit number of parallel notebooks if nothing else is used\ncores = mp.cpu_count()\n\nclass NotebookData:\n    \n    def __init__(self, path, timeout, parameters=None, retry=0):\n        self.path = path\n        self.timeout = timeout\n        self.parameters = parameters\n        self.retry = retry\n\ndef submitNotebook(notebook):\n    #print(\"Running notebook %s\" % notebook.path)\n    try:\n        if (notebook.parameters):\n            return dbutils.notebook.run(notebook.path, notebook.timeout, notebook.parameters)\n        else:\n            return dbutils.notebook.run(notebook.path, notebook.timeout)\n    except Exception:\n        if notebook.retry < 1:\n            raise\n        print(\"Retrying notebook %s\" % notebook.path)\n        notebook.retry = notebook.retry - 1\n    submitNotebook(notebook)\n\ndef parallelNotebooks(notebooks, numInParallel=cores):\n    # If you create too many notebooks in parallel the driver may crash when you submit all of the jobs at once. \n    # This code limits the number of parallel notebooks.\n    with ThreadPoolExecutor(max_workers=numInParallel) as ec:\n        return [ec.submit(submitNotebook, notebook) for notebook in notebooks]\n","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"98848bd5-ca5c-4049-9c09-249a58ffd562"},{"version":"CommandV1","origId":3273046978841390,"guid":"47c7bba0-6011-4333-a7ca-9894c195b9b2","subtype":"command","commandType":"auto","position":5.0,"command":"import ast\n\ndef get_value_from_connection_string(conn_str, cs_key, delimiter=\";\"):\n    \"\"\"\n    Description:\n      Converts a connection string to dictionary,\n        then searches for a particular key\n      This is a case-insensitive search.\n\n    Params:\n      conn_str  : string (Mandatory) The connection string\n      key       : string (Mandatory) The key value you're searching for\n      delimiter : string (Optional)  Delimiter used in connection string\n    \n    Example:\n      db_name = get_value_from_connection_string(\"Database=db1, Schema=sch1, User=usr1, Pw=pw1\", \"Database\", \",\")\n      print(db_name)\n    \"\"\"\n    cs_dict = {}\n    cs_key = cs_key.lower()\n    \n    # Prepare string to convert into dictionary\n    conn_str = conn_str.replace(\"=\", '\":\"').replace(delimiter, '\",\"').replace(',\"\"', \"\")\n    conn_str = '{\"' + conn_str + '\"}'\n\n    # Create dictionary from string\n    cs_dict = ast.literal_eval(conn_str)\n    \n    # Convert keys to lower case for case-insensitve search\n    cs_dict = {k.lower(): v for k, v in cs_dict.items()}\n    \n    # Get the value for the desired key\n    ret_val = cs_dict.get(cs_key)\n    return ret_val\n","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"16b69105-4b11-4b61-b3da-aa8772c529c7"},{"version":"CommandV1","origId":3273046978841391,"guid":"a977bad3-d181-4f68-bf81-5f4245fb5846","subtype":"command","commandType":"auto","position":6.0,"command":"def run_sql_query (query, execute_query=True):\n    \"\"\"\n    Generic function to run a Spark SQL query\n    and handle the error.\n    \n    Parameters:\n        query :  str, mandatory\n        execute_query :  boolean, optional\n    \n    Output:\n        boolean\n    \"\"\"\n    if execute_query:\n        try:\n            spark.sql(query)\n        except Exception as E:\n            print(f\"WARNING: Error executing query: {query}\")\n            print(f\"\\t{E}\")\n            pass\n            return(False)\n    else:\n        print(\" -->\", query)\n    \n    return(True)","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b26ae44c-4b25-45c1-81f2-d7545c4eec8a"},{"version":"CommandV1","origId":3273046978841392,"guid":"6c1e27f2-ea4b-4aa3-97d7-cc06ad448b74","subtype":"command","commandType":"auto","position":7.0,"command":"import json\nimport zlib\nimport base64\n\ndef compress_string (source_string):\n    \"\"\"\n    Compresses a string.\n    \n    Parameters:\n        source_string :  str, mandatory\n    \n    Output:\n        base64 encoded, zlib compressed string\n    \"\"\"\n    try:\n        str_compressed = base64.b64encode(zlib.compress(json.dumps(source_string).encode('utf-8')))\n    except Exception as e:\n        print(\"ERROR: Failed to compress string!\")\n        str_compressed = source_string\n        raise e\n    return (str_compressed)\n\n\t","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"cce1a3d1-b0dc-4512-811f-294e7aa2af0b"},{"version":"CommandV1","origId":3273046978841393,"guid":"997a7883-85e7-4a47-b9b2-3237777bebb8","subtype":"command","commandType":"auto","position":8.0,"command":"import json\nimport zlib\nimport base64\n\ndef decompress_string (compressed_string):\n    \"\"\"\n    DeCompresses a base64, zlib compressed string.\n    \n    Parameters:\n        source_string :  str, mandatory\n    \n    Output:\n        base64 decoded, decompressed string\n    \"\"\"\n    try:\n        str_decompressed = json.loads(zlib.decompress(base64.b64decode(compressed_string)))\n    except Exception as e:\n        #print(\"ERROR: Failed to decompress string!\")\n        str_decompressed = compressed_string\n        raise e\n    return (str_decompressed)","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"abdc8deb-2f27-4cd3-9e0b-e69ef54a874c"},{"version":"CommandV1","origId":3273046978841394,"guid":"8fa2b974-3d36-419d-9295-d63ff436911a","subtype":"command","commandType":"auto","position":9.0,"command":"import json\nimport gzip\n\ndef compress_string_gzip (source_string):\n    \"\"\"\n    Compresses a string.\n    \n    Parameters:\n        source_string :  str, mandatory\n    \n    Output:\n        base64 encoded, gzip compressed string\n    \"\"\"\n    try:\n        #str_compressed = base64.b64encode(zlib.compress(json.dumps(source_string).encode('utf-8')))\n        data = bytes(json.dumps(source_string), 'utf-8')\n        str_compressed = gzip.compress(data)\n    except Exception as e:\n        print(\"ERROR: Failed to compress string!\")\n        str_compressed = source_string\n        raise e\n    return (str_compressed)","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"7f528320-e891-4403-9e60-bb5b532b7384"},{"version":"CommandV1","origId":3273046978841395,"guid":"a8333776-48eb-44be-bb95-3780dd18f83b","subtype":"command","commandType":"auto","position":10.0,"command":"import json\nimport gzip\n\ndef decompress_string_gzip (compressed_string):\n    \"\"\"\n    DeCompresses a base64, zlib compressed string.\n    \n    Parameters:\n        source_string :  str, mandatory\n    \n    Output:\n        gzip decompressed string\n    \"\"\"\n    try:\n        str_decompressed = json.loads(gzip.decompress(compressed_string))\n    except Exception as e:\n        print(\"ERROR: Failed to decompress string!\")\n        str_decompressed = compressed_string\n        raise e\n    return (str_decompressed)","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"6c5783ec-7193-497c-ae48-8f359dd15d79"},{"version":"CommandV1","origId":3273046978841396,"guid":"c5fee578-ec81-4c77-bcbc-dc2a90140275","subtype":"command","commandType":"auto","position":11.0,"command":"def remove_vowels_from_string (source_string):\n    \"\"\"\n    Strips out all vowels from a string.\n    \n    Parameters:\n        source_string :  str, mandatory\n    \n    Output:\n        Input string without vowels\n    \"\"\"\n    newstr = source_string.lower()\n    vowels = ('a', 'e', 'i', 'o', 'u');\n    for x in source_string.lower():\n        if x in vowels:\n            newstr = newstr.replace(x,\"\");\n\n    return newstr","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b6ecd665-7a6f-4c6b-bb5d-2e7cafaa1ece"},{"version":"CommandV1","origId":3273046978841397,"guid":"310ea01a-f1f3-4bd1-9bee-2c2fd44520eb","subtype":"command","commandType":"auto","position":12.0,"command":"import ast\n\ndef run_notebook_with_retry(notebook, params, timeout = 600, retries = 1, debug=False):\n    \"\"\"\n    Runs a Databricks notebook\n    See:  https://docs.azuredatabricks.net/notebooks/notebook-workflows.html#\n    \n    Parameters  :\n        notebook  :  str, mandatory\n        args      :  json string, optional\n        timeout   :  int, optional (default 600 seconds/10 minutes)\n        retries   :  int, optional (default = 1)\n    \n    Output      :\n        Value in called notebook's dbutils.notebook.exit(\"\") command.\n        \n    Example     :\n        col_delim   = \",\" #\"\\x07\"\n        quote_char  = '\"' #\"\\x0e\"\n        candidateKey= \"\"\n        save_stats  = \"FALSE\"\n        file_type   = \"FULL\"\n        header      = \"TRUE\"\n        date_format = \"MM/dd/YYYY HH:mm:ss.S\"\n        db_schema   = \"LIVBH\"\n        db_object   = \"COMMUNITS\"\n        schema      = \"\"\n        csv_file    = \"RAW/liv-bh/CommUnits/CommUnits_4798_20190806093615.csv\"\n        timeout     = 600\n        nb = '/Dev/IngestCSVandWriteDelta'\n        args = '{\"filePath\": csv_file, \n                \"fileSchema\": schema, \n                \"fileType\": file_type, \n                \"fileHeader\": header, \n                \"saveStatistics\": save_stats,\n                \"deltaCandidateKey\": candidateKey,\n                \"dateFormat\": date_format,\n                \"deltaSchemaName\": db_schema,\n                \"deltaObjectName\": db_object,\n                \"fileColDelimiter\": col_delim,\n                \"fileQuoteCharacter\": quote_char\n               }'\n\n        run_notebook_with_retry(nb, args, timeout, retries = 0)\n    \"\"\"\n    #print('-----------------------------------------')\n    if debug:\n        print(f'Running notebook: {notebook}')\n        print('Params:')\n        print(f'\\t{params}'.replace(',', '\\n\\t'))\n    #print('-----------------------------------------')\n    \n    args = params\n    num_retries = 1\n    while True:\n        try:\n            return dbutils.notebook.run(notebook, timeout, args)\n        except Exception as e:\n            if num_retries >= retries:\n                raise e\n            else:\n                print(\"Retrying error\", e)\n                num_retries += 1\n","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b4d11207-d953-4fea-80bd-48307d1f101e"},{"version":"CommandV1","origId":3273046978841398,"guid":"e374dd3d-8e81-484b-9160-7acb11af00a8","subtype":"command","commandType":"auto","position":13.0,"command":"def dataframeToDict(dataFrame):\n    \"\"\"\n    Creates a dictionary from a dataframe, whereby\n    allowing key name lookups, as well as\n    iterating over values.\n    \n    Parameters  :\n        dataFrame  :  str, mandatory\n    \n    Output      :\n        Dictionary with \"'ColumnName': ([list of values])\".\n        \n    Example     :\n        dfa_dict = dataframeToDict(dfa)\n        \n    \"\"\"\n    colDict = dict(zip(dataFrame.schema.names, zip(*dataFrame.collect())))\n    return colDict if colDict else dict.fromkeys(dataFrame.schema.names, ())\n","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b7b1e43b-7474-4cf0-ae05-b94893f23300"},{"version":"CommandV1","origId":3273046978841399,"guid":"d0f124e6-bdaf-4f8f-8014-a554bbe064b9","subtype":"command","commandType":"auto","position":14.0,"command":"import pyspark.sql.functions as f\n\ndef date_dimension(from_date, to_date):\n    \"\"\"\n    Creates a dataframe of Dates\n    \n    Parameters  :\n        from_date  :  str, mandatory\n        to_date    :  str, mandatory\n    \n    Output      :\n        Dataframe of dates.\n        \n    Example     :\n        from_date = \"2019-01-01\"\n        to_date = \"2019-01-31\"\n        df_dimdate = date_dimension(from_date, to_date)\n        display(df_dimdate)\n    \"\"\"\n    data = [(from_date, to_date)]\n    df_date = spark.createDataFrame(data, [\"minDate\", \"maxDate\"])\n\n    df_date = df_date.withColumn(\"dateDiff\", f.datediff(\"maxDate\", \"minDate\"))\\\n        .withColumn(\"repeat\", f.expr(\"split(repeat(',', dateDiff), ',')\"))\\\n        .select(\"*\", f.posexplode(\"repeat\").alias(\"date\", \"val\")) \\\n        .withColumn(\"date\", f.expr(\"date_add(minDate, date)\"))\\\n        .select('date')\n\n    return df_date","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"67be1ec5-042a-4820-91b5-546c62afa16b"},{"version":"CommandV1","origId":3273046978841400,"guid":"0a6d0ee6-f951-4836-aae6-01a0737ca30d","subtype":"command","commandType":"auto","position":15.0,"command":"from datetime import datetime\n\ndef date_diff(then_ts, now_ts=datetime.utcnow(), units=\"days\"):\n    \"\"\"\n    Simple function to return time difference in units desired.\n    \n    Parameters  :\n        then_ts :  datetime, mandatory\n        now_ts  :  datetime, mandatory\n        units   :  string (days, hours, minutes, seconds), optional\n    \n    Output      :\n        (INT) Time different in units specified\n\n    Example:\n      from datetime import timedelta\n      units = timedelta(days=1)\n      then = datetime.utcnow()-units\n      now = datetime.utcnow()\n      print(date_diff(then, now, \"days\"))    \n    \"\"\"\n    units = units.lower()\n    \n    diff = abs(now_ts - then_ts)\n    seconds = int(diff.total_seconds())\n    \n    if units == \"days\":\n        ret = int((seconds / 86400))\n    elif units == \"hours\":\n        ret = int((seconds / 3600))\n    elif units == \"minutes\":\n        ret = int(seconds / 60)\n    elif units == \"seconds\":\n        ret = int(seconds)\n\n    return ret\n","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"c9ba476d-bc46-4cb5-8273-442ad0c13aa1"},{"version":"CommandV1","origId":3273046978841401,"guid":"5e5da856-c9b0-46f9-b590-0ac340c3f160","subtype":"command","commandType":"auto","position":16.0,"command":"def file_exists(file_name):\n    \"\"\"\n    Performs simple filesystem utility \"ls\" command\n    and returns whether it exists or not.\n    Leave to code whether an exception is actually raised or not.\n    \n    Parameters  :\n        file_name    :  str, mandatory\n    \n    Output      :\n        Boolean indicator if file is found.\n    \"\"\"\n    found = False\n    try:\n        dbutils.fs.ls(file_name)\n        found = True\n    except:  # Didn't find the file\n        pass\n        \n    return(found)","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"61923e5c-f069-4991-bce1-d1998f03ca50"},{"version":"CommandV1","origId":3273046978841402,"guid":"065aa32c-82c8-4000-8bee-b49abb081e95","subtype":"command","commandType":"auto","position":17.0,"command":"def slash_join(list_args, leading_slash=True):\n    \"\"\"\n    Joins strings in \"args\" tuple/array together\n    separated with a forward slash.\n    Typically used for ADLS paths.\n    Optionally adds leading forward slash.\n\n    Parameters :\n        args            : str, mandatory\n        leading_slash   : boolean, optional (default is True)\n        \n    Output     :\n        Returns string joined by slashes \n    \n    Examples    :\n        args = [\"mnt\", \"RAW/file.csv\"]\n        csvfile = slash_join(args, True) \n        Outputs:  /mnt/RAW/file.csv\n    \"\"\"\n    if leading_slash:\n        list_args.insert(0, \"/\")    # Insert a new first element into list\n    all_args = tuple(list_args) # Convert back into Tuple\n    return \"/\".join(arg.strip(\"/\") for arg in all_args)","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"6a2386a8-3585-4f5b-8d1a-6a221008a3a8"},{"version":"CommandV1","origId":3273046978841403,"guid":"84146bfa-7a67-4fbc-9fba-ddfec62e4ad0","subtype":"command","commandType":"auto","position":18.0,"command":"import re\nimport itertools\n\ndef strip_string(string_to_clean, replace_with=\"\", to_lower=False, remove_vowels=False, trim_length=0, dedupe=False):\n    \"\"\"\n    Removes any non-alphanumeric characters from a\n    string, with exception of underscore.\n    Optionally replaces characters with supplied character.\n    Optionally changes string to lower case.\n    Optionally removes vowels from string.\n    Optionally returns only left(n) characters from string.\n    Optionally dedupes the string for consecutively duplicated letters.\n\n    Parameters :\n        string_to_clean : str, mandatory\n        replace_with    : str, optional (default is \"\")\n        to_lower        : boolean, optional (default is False)\n        remove_vowels   : boolean, optional (default is False)\n        trim_length     : int, optional\n        dedupe          : boolean, optional (default is false)\n        \n    Output     :\n        Returns cleaned string, optionally with vowels removed\n        and/or trimmed to specified length.\n    \n    Examples    :\n        clean_string = strip_string(\"*The!_Field_Name 1\")\n        Outputs:  The_Field_Name1\n    \"\"\"\n    if string_to_clean:\n        regex = re.compile('[^a-zA-Z0-9_]')\n        out_string = regex.sub(replace_with, string_to_clean)\n        if to_lower:\n            out_string = out_string.lower()\n\n        if remove_vowels:\n            out_string = remove_vowels_from_string(out_string)\n\n        if trim_length > 0:\n            out_string = out_string[:trim_length]\n\n        if dedupe:\n            out_string = ''.join(ch for ch, _ in itertools.groupby(out_string))\n    else:\n        out_string = string_to_clean\n        \n    return out_string\n","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"2034a836-ca3b-4e74-9c99-9c5406fae914"},{"version":"CommandV1","origId":3273046978841404,"guid":"a36cd592-3425-496f-b562-ed05a729f70b","subtype":"command","commandType":"auto","position":19.0,"command":"from datetime import timedelta, datetime\n\ndef get_datetime_key(dateTime=datetime.utcnow()):\n    \"\"\"\n    Returns INT based key from a datetime/timestamp value\n    such as: 20180331\n    \n    Parameters :\n        dateTime   :  datetime, optional (default is current UTC time)\n    \n    Example:\n        t = datetime.utcnow()\n        dt = get_datetime_key(t)\n        print(dt, type(dt))\n    \"\"\"\n    dateKey = dateTime.year * 10000 + dateTime.month * 100 + dateTime.day\n  \n    return dateKey\n","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"fa7038b8-eeae-4faf-b67c-3533f592c4be"},{"version":"CommandV1","origId":3273046978841405,"guid":"30090659-541a-4a8c-be0d-2e5c51da26a1","subtype":"command","commandType":"auto","position":20.0,"command":"from pyspark.sql.functions import col\n\ndef check_if_db_exists(db_name, create_if_not_exist=False, delta_location='DELTA'):\n    \"\"\"\n    Checks if database exists\n    Works with Spark 2.x and 3.x (with new column name)\n\n    Parameters :\n        db_name             : str, mandatory\n        create_if_not_exist :  boolean, optional (default=False)\n        delta_location      :  str, optional (default='DELTA')\n\n    Output     :\n        Boolean (existence of table)\n    \n    Example    :\n        db_exists = check_if_db_exists(\"myDatabase\", True, 'DELTA')\n    \"\"\"\n\n    # Change to lower case since all tables are stored this way and \n    # for the dataframe query below, it is case sensitive.\n    db_name = db_name.lower()\n\n    if create_if_not_exist:\n        query = f\"CREATE DATABASE IF NOT EXISTS {db_name} LOCATION '/mnt/{delta_location}/{db_name}'\"\n        spark.sql(query)\n\n    # Check if database exists\n    _df = spark.sql(\"show databases\")\n    \n    if _df.select(_df[0]).where(_df[0] == f\"{db_name}\").count():\n        return True\n    else:\n        return False      \n","commandVersion":9,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"f7e130f3-a10c-4be5-b766-bf22f0eaa005"},{"version":"CommandV1","origId":3273046978841406,"guid":"f8ca3074-0f9c-41d3-a8cb-c8cd329a8440","subtype":"command","commandType":"auto","position":21.0,"command":"def check_if_table_exists(table_name, db_name=\"default\"):\n    \"\"\"\n    Checks if table exists in database\n\n    Parameters :\n        table_name : str, mandatory\n        db_name    : str, optional (default is \"default\")\n\n    Output     :\n        Boolean (existence of table)\n    \n    Example    :\n        table_exists = check_if_table_exists(\"tableA\", \"myDatabase\")\n    \"\"\"\n\n    # Change to lower case since all tables are stored this way and \n    # for the dataframe query below, it is case sensitive.\n    table_name = table_name.lower()\n    \n    if spark.sql(f\"show tables in {db_name} like '{table_name}'\").count() > 0:\n        return True\n    else:\n        return False\n","commandVersion":19,"state":"finished","results":{"type":"listResults","data":[{"type":"html","data":"<div class=\"ansiout\"></div>","arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}}],"arguments":{},"addedWidgets":{},"removedWidgets":[],"datasetInfos":[],"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":1608320760898,"submitTime":1608320760882,"finishTime":1608320760921,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"cd3b7f3c-4cff-4081-8f26-8fee0f0b4dd6"},{"version":"CommandV1","origId":3273046978841407,"guid":"b5aef0fc-f962-4251-81ac-b6e5cc1d9be3","subtype":"command","commandType":"auto","position":22.0,"command":"from dateutil.parser import parse\n\ndef infer_datatype(value):\n    \"\"\"\n    Test/infer datatype of given value\n\n    Parameters :\n        value : str, mandatory\n\n    Output     :\n        Python data type\n   \n    \"\"\"\n    if value is not None:\n        data_types = ['int','datetime.datetime','boolean','float','str']\n        for dtype in data_types:\n            # Handle dates separately with dateutils.parse function\n            if 'datetime' in dtype:\n                try:\n                    if parse(value):\n                        return(dtype)\n                except:\n                    pass\n            else:\n                try:\n                    if isinstance(eval(value), eval(dtype)):\n                        # When test for a number, ensure it's actually numeric (e.g. no dashes interpreted as minuses)\n                        if dtype == 'int' and value.isnumeric():\n                            return(dtype)\n                        elif dtype == 'float':\n                            return(dtype)\n                except:\n                    try:\n                        if isinstance(value, eval(dtype)):\n                            return(dtype)\n                    except:\n                        pass\n        # If we make it this far, just return a string\n        return('str')\n    else:\n        # No way to determine what a NULL is...\n        #print(\"***Unable to determine datatype for value {}!  Defaulting to 'string' type.***\".format(value)) \n        return('str')\n       \ndef python_to_spark_datatype(value, python_data_type):\n    \"\"\"\n    Map Python to Spark datatype\n\n    Parameters :\n        value : str, mandatory\n        python_data_type : str, mandatory\n\n    Output     :\n        Python data type\n   \n    Spark Data Types: \n        TINYINT\n        SMALLINT\n        INT\n        BIGINT\n        FLOAT\n        DOUBLE\n        BOOLEAN\n        STRING\n        TIMESTAMP\n    \"\"\"\n    if python_data_type == \"int\":\n        # Test if the number is a Long (BIGINT)\n        try:\n            if fabs(eval(value)) > 2147483647:\n                python_data_type = 'long'\n        except:\n           pass\n    switcher = {\n        \"str\": \"STRING\",\n        \"datetime.datetime\": \"TIMESTAMP\",\n        \"float\": \"FLOAT\",\n        \"boolean\": \"BOOLEAN\",\n        \"int\": \"INT\",\n        \"long\": \"BIGINT\"\n    }\n    return(switcher.get(python_data_type, \"STRING\"))\n\n# # EXAMPLE (assumes a dataframe exists already)\n# buf = []\n\n# # Get a row of data, ignoring nulls\n# row_data = df.select([first(x, ignorenulls=True).alias(x) for x in df.columns]).first()\n\n# # Iterate thorugh columns (excluding ones added above)\n# for col in row_data.asDict():\n#     if not col in ['rowhash','current','enddate','dataset','sourcefilename','processtsutc','processdatekey']:\n#         val = row_data[col] # Stored value \n#         dtype = infer_datatype(val) # Get the datatype\n#         data_type = python_to_spark_datatype(val, dtype) # Map datatype from Python to Spark\n#         col_definition = '{} {}'.format(col, data_type) # Produce column definition for script\n#         buf.append(col_definition) # Add to list\n#         #print(col, \"||\", val, \"||\", data_type)\n\n# print(buf)\n","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"4a226d2a-a3e4-4c6d-98fa-85f8ba9827c9"},{"version":"CommandV1","origId":3273046978841408,"guid":"cb415e7b-d9f2-4284-9193-230e3076814e","subtype":"command","commandType":"auto","position":23.0,"command":"def sql_to_spark_datatype(sql_data_type):\n    \"\"\"\n    Map SQL to Spark datatype\n\n    Parameters :\n        sql_data_type : str, mandatory\n\n    Output     :\n        Spark data type\n    \"\"\"\n    sql_data_type = sql_data_type.lower()\n    if sql_data_type.find(\"(\") > 0:\n        sql_data_type = sql_data_type[:sql_data_type.find(\"(\")]\n    switcher = {\n        \"varchar\": \"StringType()\",\n        \"char\": \"StringType()\",\n        \"text\": \"StringType()\",\n        \"nvarchar\": \"StringType()\",\n        \"nchar\": \"StringType()\",\n        \"ntext\": \"StringType()\",\n        \"guid\": \"StringType()\",\n        \"binary\": \"StringType()\",    #\"BinaryType()\",\n        \"varbinary\": \"StringType()\", #\"BinaryType()\",\n        \"date\": \"DateType()\",\n        \"time\": \"TimestampType()\",\n        \"datetime\": \"TimestampType()\",\n        \"datetime2\": \"TimestampType()\",\n        \"datetimeoffset\": \"StringType()\", # No Spark equivalent to DTO\n        \"timestamp\": \"TimestampType()\",\n        \"real\": \"FloatType()\",\n        \"double\": \"DoubleType()\",\n        \"decimal\": \"DoubleType()\",\n        \"numeric\": \"DoubleType()\",\n        \"bit\": \"BooleanType()\",\n        \"tinyint\": \"IntegerType()\",\n        \"int\": \"IntegerType()\",\n        \"bigint\": \"LongType()\",\n        \"money\": \"FloatType()\"\n    }\n    return(switcher.get(sql_data_type, \"StringType()\"))\n","commandVersion":5,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"e1c71d16-34f1-47f0-bf92-ee4ac052e889"},{"version":"CommandV1","origId":3273046978841409,"guid":"213d1601-c460-4487-b8a7-7cd496daabb6","subtype":"command","commandType":"auto","position":24.0,"command":"def sql_to_pyspark_datatype(sql_data_type):\n    \"\"\"\n    Map SQL to Spark datatype\n\n    Parameters :\n        sql_data_type : str, mandatory\n\n    Output     :\n        Spark data type\n    \"\"\"\n    sql_data_type = sql_data_type.lower()\n    pos = sql_data_type.find(\"(\")\n    if pos > 0:\n        base_col_type = sql_data_type[:sql_data_type.find(\"(\")]\n        sql_size = sql_data_type[sql_data_type.find(\"(\"):].replace(\"(\",\"\").replace(\")\",\"\")\n        # If there aren't any decimals, go ahead and just convert to INTs.\n        if sql_size.find(',0') > 0:\n            base_col_type = \"int\"\n    else:\n        sql_size = \"38,18\" # provide default, just in case\n        base_col_type = sql_data_type\n   \n    switcher = {\n        \"varchar\": \"StringType()\",\n        \"char\": \"StringType()\",\n        \"text\": \"StringType()\",\n        \"nvarchar\": \"StringType()\",\n        \"nchar\": \"StringType()\",\n        \"ntext\": \"StringType()\",\n        \"guid\": \"StringType()\",\n        \"binary\": \"StringType()\",    #\"BinaryType()\",\n        \"varbinary\": \"StringType()\", #\"BinaryType()\",\n        \"date\": \"DateType()\",\n        \"time\": \"TimestampType()\",\n        \"datetime\": \"TimestampType()\",\n        \"datetime2\": \"TimestampType()\",\n        \"datetimeoffset\": \"StringType()\", # No Spark equivalent to DTO\n        \"timestamp\": \"BinaryType()\",\n        \"real\": \"FloatType()\",\n        \"double\": f\"DecimalType({sql_size})\",\n        \"decimal\": f\"DecimalType({sql_size})\",\n        \"numeric\": f\"DecimalType({sql_size})\",\n        \"money\": f\"DecimalType({sql_size})\",\n        \"bit\": \"BooleanType()\",\n        \"tinyint\": \"IntegerType()\",\n        \"int\": \"IntegerType()\",\n        \"smallint\": \"IntegerType()\",\n        \"bigint\": \"LongType()\",\n        \"money\": \"DecimalType(19,4)\",\n        \"boolean\": \"BooleanType()\"\n    } \n    \n    return(switcher.get(base_col_type, \"StringType()\"))","commandVersion":4,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"0b9ccd37-947a-47e6-ae42-d1e165f3c085"},{"version":"CommandV1","origId":3273046978841410,"guid":"5e2cc662-d935-4948-85e1-009610301d04","subtype":"command","commandType":"auto","position":24.5,"command":"def sqldw_to_pyspark_datatype(sql_data_type):\n    \"\"\"\n    Map SQL DW datatypes for Snowflake Exports\n\n    Parameters :\n        sql_data_type : str, mandatory\n\n    Output     :\n        Spark data type\n    \"\"\"\n    sql_data_type = sql_data_type.lower()\n    pos = sql_data_type.find(\"(\")\n    if pos > 0:\n        base_col_type = sql_data_type[:sql_data_type.find(\"(\")]\n        sql_size = sql_data_type[sql_data_type.find(\"(\"):].replace(\"(\",\"\").replace(\")\",\"\")\n        # If there aren't any decimals, go ahead and just convert to INTs.\n        if sql_size.find(',0') > 0:\n            base_col_type = \"int\"\n    else:\n        sql_size = \"38,18\" # provide default, just in case\n        base_col_type = sql_data_type\n   \n    switcher = {\n        \"varchar\": \"StringType()\",\n        \"char\": \"StringType()\",\n        \"text\": \"StringType()\",\n        \"nvarchar\": \"StringType()\",\n        \"nchar\": \"StringType()\",\n        \"ntext\": \"StringType()\",\n        \"guid\": \"StringType()\",\n        \"binary\": \"StringType()\",    #\"BinaryType()\",\n        \"varbinary\": \"StringType()\", #\"BinaryType()\",\n        \"date\": \"DateType()\",\n        \"time\": \"TimestampType()\",\n        \"datetime\": \"TimestampType()\",\n        \"datetime2\": \"TimestampType()\",\n        \"datetimeoffset\": \"StringType()\", # No Spark equivalent to DTO\n        \"timestamp\": \"TimestampType()\",\n        \"real\": \"FloatType()\",\n        \"double\": f\"DecimalType({sql_size})\",\n        \"decimal\": f\"DecimalType({sql_size})\",\n        \"numeric\": f\"DecimalType({sql_size})\",\n        \"money\": f\"DecimalType({sql_size})\",\n        \"bit\": \"BooleanType()\",\n        \"tinyint\": \"IntegerType()\",\n        \"int\": \"IntegerType()\",\n        \"smallint\": \"IntegerType()\",\n        \"bigint\": \"LongType()\",\n        \"money\": \"DecimalType(19,4)\",\n        \"boolean\": \"BooleanType()\"\n    } \n    \n    return(switcher.get(base_col_type, \"StringType()\"))","commandVersion":11,"state":"error","results":null,"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":null,"yColumns":null,"pivotColumns":null,"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"b0e73abd-c26a-4354-ba36-0b5f8d95caee"},{"version":"CommandV1","origId":3273046978841411,"guid":"b4082efb-a58a-49bc-b15c-b6dc41f4c7ba","subtype":"command","commandType":"auto","position":25.0,"command":"def spark_to_sql_datatype(sql_data_type):\n    \"\"\"\n    Map Spark to SQL datatype\n\n    Parameters :\n        sql_data_type : str, mandatory\n\n    Output     :\n        SQL data type\n    \"\"\"\n    switcher = {\n        \"tinyint\": \"tinyint\",\n        \"smallint\": \"smallint\",\n        \"int\": \"int\",\n        \"bigint\": \"bigint\",\n        \"boolean\": \"bit\",\n        \"double\": \"float\",\n        \"float\": \"real\",\n        \"string\": \"nvarchar\",\n        \"nvarchar\": \"nvarchar\",\n        \"binary\": \"varbinary(max)\",\n        \"timestamp\": \"datetime2\",\n        \"date\": \"datetime2\",\n        \"decimal\": \"decimal\"\n        #\"double\": \"double\",\n        #\"double\": \"smallmoney\",\n        #\"string\": \"char\",\n        #\"string\": \"nchar\",\n        #\"binary\": \"varbinary\",\n        #\"timestamp\": \"smalldatetime\",\n        #\"timestamp\": \"datetime2\",\n        #\"timestamp\": \"datetime\",\n        #\"timestamp\": \"time\"\n    }\n    return(switcher.get(sql_data_type, \"nvarchar(255)\"))","commandVersion":4,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"9e4e7bf9-f126-472c-9fbc-f4e8613533fb"},{"version":"CommandV1","origId":3273046978841412,"guid":"1f54bc71-ac68-446d-b8b8-c6d895a9cae7","subtype":"command","commandType":"auto","position":26.0,"command":"def spark_to_snowflake_datatype(sql_data_type):\n    \"\"\"\n    Map Spark to Snowflake datatypes\n    Refs:\n        https://docs.snowflake.com/en/user-guide/spark-connector-use.html#moving-data-from-spark-to-snowflake\n        https://docs.snowflake.com/en/sql-reference/data-types.html\n        \n    Parameters :\n        sql_data_type : str, mandatory\n\n    Output     :\n        Snowflake data type\n    \"\"\"\n    sql_data_type = sql_data_type.lower()\n    pos = sql_data_type.find(\"(\")\n    if pos > 0:\n        base_col_type = sql_data_type[:sql_data_type.find(\"(\")]\n        sql_size = sql_data_type[sql_data_type.find(\"(\"):].replace(\"(\",\"\").replace(\")\",\"\")\n        # If there aren't any decimals, go ahead and just convert to INTs.\n        if sql_size.find(',0') > 0:\n            base_col_type = \"int\"\n    else:\n        sql_size = \"38,18\" # provide default, just in case\n        base_col_type = sql_data_type\n        \n    switcher = {\n        #Pyspark types\n        \"arraytype\": \"VARIANT\",\n        \"binarytype\": \"BINARY\",  #Snowflake documentation says BinaryType not supported\n        \"booleantype\": \"BOOLEAN\",\n        \"bytetype\": \"INTEGER\",\n        \"datetype\": \"DATE\",\n        \"decimaltype\": \"DECIMAL(38,18)\",\n        \"doubletype\": \"DOUBLE\",\n        \"floattype\": \"FLOAT\",\n        \"integertype\": \"INTEGER\",\n        \"longtype\": \"INTEGER\",\n        \"maptype\": \"VARIANT\",\n        \"shorttype\": \"INTEGER\",\n        \"stringtype\": \"VARCHAR\", #Length can be specified, or defaults to max length (16MB - 8MB unicode)\n        \"structtype\": \"VARIANT\",\n        \"timestamptype\": \"TIMESTAMP\",\n        # Hive/SQL data types\n        \"tinyint\": \"INTEGER\",\n        \"smallint\": \"INTEGER\",\n        \"int\": \"INTEGER\",\n        \"bigint\": \"INTEGER\",\n        \"boolean\": \"BOOLEAN\",\n        \"double\": \"DOUBLE\",\n        \"float\": \"FLOAT\",\n        \"string\": \"VARCHAR\",\n        \"nvarchar\": \"VARCHAR\",\n        \"binary\": \"BINARY\",\n        \"timestamp\": \"TIMESTAMP\",\n        \"date\": \"DATE\",\n        \"decimal\": \"DECIMAL(38,18)\"\n    }\n    return(switcher.get(base_col_type, \"VARCHAR(4000)\"))","commandVersion":42,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"2ebbc8ec-3d9a-4199-8649-df651f2387b6"},{"version":"CommandV1","origId":3273046978841413,"guid":"0c7b1248-b0e1-48f4-9191-9c5b536e3380","subtype":"command","commandType":"auto","position":26.5,"command":"def snowflake_to_spark_datatype(sf_data_type):\n    \"\"\"\n    Map Snowflake to Spark datatypes\n    Refs:\n        https://docs.snowflake.com/en/user-guide/spark-connector-use.html#moving-data-from-spark-to-snowflake\n        https://docs.snowflake.com/en/sql-reference/data-types.html\n        \n    Parameters :\n        sf_data_type : str, mandatory\n\n    Output     :\n        Spark data type\n    \"\"\"\n    sf_data_type = sf_data_type.upper()\n    if sf_data_type.find(',0') > 0:\n        sf_data_type = \"INTEGER\"\n    elif sf_data_type.find(\"(\") > 0:\n        sf_data_type = sf_data_type[:sf_data_type.find(\"(\")]\n        \n    switcher = {\n        #Pyspark types\n        \"VARIANT\":  \"ArrayType()\",\n        \"BOOLEAN\":  \"BooleanType()\",\n        \"DATE\":     \"DateType()\",\n        \"DECIMAL\":  \"DecimalType()\",\n        \"DOUBLE\":   \"DoubleType()\",\n        \"FLOAT\":    \"FloatType()\",\n        \"INTEGER\":  \"IntegerType()\",\n        \"NUMBER\":   \"DecimalType()\",\n        \"VARCHAR\":  \"StringType()\",\n        \"TIMESTAMP\":\"TimestampType()\",\n        \"TIMESTAMP_NTZ\":\"TimestampType()\"\n    }\n    return(switcher.get(sf_data_type, \"StringType()\"))","commandVersion":3,"state":"finished","results":null,"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":null,"yColumns":null,"pivotColumns":null,"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"966dd1bf-77f9-4969-87ce-57a3dc32afb7"},{"version":"CommandV1","origId":3273046978841414,"guid":"8e029b84-de0f-4f41-9104-dee27f8388a9","subtype":"command","commandType":"auto","position":27.0,"command":"import pandas as pd\n\ndef save_summary_statistics(df, dbname, table, dbSchema, dbObject, filename, debug=False):\n    \"\"\"\n    Grabs and stores summary stats on the data, using Pandas.describe()\n\n    Parameters  :\n        df      : str, mandatory - Dataframe to save stats on\n        dbname  : str, mandatory - Database name\n        table   : str, mandatory - Table name\n        dbSchema: str, mandatory - Delta schema name\n        dbObject: str, mandatory - Delta table name\n        filename: str, mandatory - source file name\n        debug   : Boolean, optional (default=False)\n\n    Returns    : N/A\n    Output     : If debug, then prints a few rows of stats\n    \"\"\"\n\n    dataset = dbSchema + \"_\" + dbObject\n    \n    # dump stats out to Pandas dataframe \n    pdf = df.select([c for c in df.columns if c not in {'dl_rowhash', 'dl_iscurrent', 'dl_enddate', 'dl_dataset', 'dl_sourcefilename', 'dl_processtsutc', 'dl_processdatekey', 'dl_batchid', 'dl_partitionkey', 'dl_recordstartdateutc','dl_recordenddateutc'}]).toPandas().describe(include='all')\n\n    # swap columns and rows so schema doesn't need to change\n    pdf = pdf.transpose().rename_axis('column_name')#.reset_index(drop=True, inplace=True)\n\n    # Drop these since they are only present if there are timestamps in the data\n    pdf.drop(['first', 'last'], axis=1, inplace=True, errors='ignore')\n\n    # If we only have a dataframe of strings, add the other columns for numeric values\n    if not \"mean\" in pdf.columns:\n        #pdf = pd.concat([pdf,pd.DataFrame(columns=list(new_cols))])    \n        #pdf = pdf.reindex(columns=pdf.columns.tolist() + new_cols)\n        pdf[\"mean\"] = ''\n        pdf[\"std\"] = ''\n        pdf[\"min\"] = ''\n        pdf[\"25%\"] = ''\n        pdf[\"50%\"] = ''\n        pdf[\"75%\"] = ''\n        pdf[\"max\"] = ''\n\n    # Create a schema for converting back to pyspark\n    customSchema = StructType([\n      StructField(\"dataset\", StringType(), True),\n      StructField(\"columnName\", StringType(), True),\n      StructField(\"count\", StringType(), True),\n      StructField(\"unique\", StringType(), True),\n      StructField(\"top\", StringType(), True),\n      StructField(\"freq\", StringType(), True),\n      StructField(\"mean\", StringType(), True),\n      StructField(\"std\", StringType(), True),\n      StructField(\"min\", StringType(), True),\n      StructField(\"25%\", StringType(), True),\n      StructField(\"50%\", StringType(), True),\n      StructField(\"75%\", StringType(), True),\n      StructField(\"max\", StringType(), True),\n      StructField(\"sourceFile\", StringType(), True),\n      StructField(\"processedTSUTC\", StringType(), True)\n    ])\n\n    # Get rid of all NaNs\n    pdf = pdf.where(pdf.notnull(), \"\")\n    pdf = pdf.reset_index()\n\n    # Add a column indicating which dataset this is from\n    pdf.insert(0, \"dataset\", dataset)\n\n    pdf[\"sourceFile\"] = filename\n\n    # Add a processed timstamp\n    pdf[\"processedTSUTC\"] = datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S.%f')\n\n    # Convert entire dataframe to a list (since pyspark doesn't like the numpy columns)\n    lst_pdf = pdf.astype(str).values.tolist()\n\n    # Convert back to pyspark dataframe so we can save it to Delta Lake\n    df_stats = spark.createDataFrame(lst_pdf, schema=customSchema)\n\n    # Create a temp table from Dataframe to use in CREATE TABLE or MERGE statement\n    vw_stats = 'tmpStats' + dataset\n    df_stats.createOrReplaceTempView(vw_stats)\n\n    # Create database if it doesn't exist\n    db_exists = check_if_db_exists(dbname)\n    if not db_exists:\n        query = \"\"\"\n            CREATE DATABASE IF NOT EXISTS {dbname}\n            LOCATION '/mnt/{deltalocation}/{dbname}'\n        \"\"\".format(dbname=dbname, deltalocation=delta_location)\n\n        sqlContext.sql(query)  \n    \n    # Create stats table if it doesn't exist\n    stats_exists = check_if_table_exists(table, dbname)\n    if not stats_exists:\n        query = \"\"\"\n            CREATE TABLE IF NOT EXISTS {dbname}.{tblName}\n            USING DELTA\n            LOCATION '/mnt/{deltalocation}/{dbname}/{tblName}'\n            PARTITIONED BY(dataset)\n            AS \n            SELECT * FROM {view}\n        \"\"\".format(dbname=dbname, tblName=table, view=vw_stats, deltalocation=delta_location)\n    else:\n        query = \"\"\"\n            MERGE INTO {dbname}.{tblName} AS target\n            USING (\n              SELECT updates.*\n              FROM {view} AS updates \n            ) source\n            ON source.sourceFile = target.sourceFile\n            AND source.dataset = target.dataset\n            AND source.columnName = target.columnName\n            WHEN MATCHED THEN\n              UPDATE SET *\n            WHEN NOT MATCHED THEN\n              INSERT *\n        \"\"\".format(dbname=dbname, tblName=table, view=vw_stats)\n\n    sqlContext.sql(query)\n    print(\"Statistics saved to table: {dbname}.{tblName}\".format(dbname=dbname, tblName=table))\n\n    if debug:\n        print(df_stats.show())\n  ","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"5da50073-429f-401d-b40f-6379eeead438"},{"version":"CommandV1","origId":3273046978841415,"guid":"de83e70c-a66a-424f-a2f4-ca2552754835","subtype":"command","commandType":"auto","position":28.0,"command":"########################################################################\n# Set of Functions to DIFF two dataframes and union/merge them together\n#\n# https://stackoverflow.com/questions/39758045/how-to-perform-union-on-two-dataframes-with-different-amounts-of-columns-in-spark\n#\n# EXAMPLE:\n# unionDF = unionDFs(df1,df2)\n########################################################################\n\ndef __orderDFAndAddMissingCols(df, columnsOrderList, dfMissingFields):\n    ''' return ordered dataFrame by the columns order list with null in missing columns '''\n    if not dfMissingFields:  #no missing fields for the df\n        return df.select(columnsOrderList)\n    else:\n        columns = []\n        for colName in columnsOrderList:\n            if colName not in dfMissingFields:\n                columns.append(colName)\n            else:\n                columns.append(lit(None).alias(colName))\n        return df.select(columns)\n\ndef __addMissingColumns(df, missingColumnNames):\n    ''' Add missing columns as null in the end of the columns list '''\n    listMissingColumns = []\n    for col in missingColumnNames:\n        listMissingColumns.append(lit(None).alias(col))\n\n    return df.select(df.schema.names + listMissingColumns)\n\ndef __orderAndUnionDFs( leftDF, rightDF, leftListMissCols, rightListMissCols):\n    ''' return union of data frames with ordered columns by leftDF. '''\n    leftDfAllCols = __addMissingColumns(leftDF, leftListMissCols)\n    rightDfAllCols = __orderDFAndAddMissingCols(rightDF, leftDfAllCols.schema.names, rightListMissCols)\n    return leftDfAllCols.union(rightDfAllCols)\n\ndef unionDFs(leftDF,rightDF):\n    ''' Union between two dataFrames, if there is a gap of column fields,\n     it will append all missing columns as nulls '''\n    # Check for None input\n    if leftDF == None:\n        raise ValueError('leftDF parameter should not be None')\n    if rightDF == None:\n        raise ValueError('rightDF parameter should not be None')\n        #For data frames with equal columns and order- regular union\n    if leftDF.schema.names == rightDF.schema.names:\n        return leftDF.union(rightDF)\n    else: # Different columns\n        #Save dataFrame columns name list as set\n        leftDFColList = set(leftDF.schema.names)\n        rightDFColList = set(rightDF.schema.names)\n        # Diff columns between leftDF and rightDF\n        rightListMissCols = list(leftDFColList - rightDFColList)\n        leftListMissCols = list(rightDFColList - leftDFColList)\n        return __orderAndUnionDFs(leftDF, rightDF, leftListMissCols, rightListMissCols)","commandVersion":1,"state":"finished","results":{"type":"raw","data":"","arguments":{},"metadata":{}},"errorSummary":null,"errorTraceType":null,"error":null,"workflows":[],"startTime":0,"submitTime":0,"finishTime":0,"collapsed":false,"bindings":{},"inputWidgets":{},"displayType":"table","width":"auto","height":"auto","xColumns":[],"yColumns":[],"pivotColumns":[],"pivotAggregation":null,"useConsistentColors":false,"customPlotOptions":{},"commentThread":[],"commentsVisible":false,"parentHierarchy":[],"diffInserts":[],"diffDeletes":[],"globalVars":{},"latestUser":"a user","latestUserId":null,"commandTitle":"","showCommandTitle":false,"hideCommandCode":false,"hideCommandResult":false,"isLockedInExamMode":false,"iPythonMetadata":null,"streamStates":{},"datasetPreviewNameToCmdIdMap":{},"nuid":"7a379d31-5ff4-4887-8d39-12f09f6645d0"}],"dashboards":[],"guid":"577fab12-76fe-47b1-805a-8acc62e0eb3b","globalVars":{},"iPythonMetadata":null,"inputWidgets":{},"notebookMetadata":{"pythonIndentUnit":2}}